/*
 * wiz_cmd_lord.c
 *
 * This object holds the lord wizards commands. (WIZ_LORD)
 *
 */
#pragma save_binary

inherit "/cmd/std/command_driver";

#include "/sys/macros.h"
#include "/sys/log.h"
#include "/secure/std.h"

#define MAP_OBJ "/std/map/map"
#define CHECK_SO_LORD 	if (WIZ_CHECK < WIZ_LORD) return 0; \
			if (this_interactive() != this_player()) return 0

/* **************************************************************************
 * Return a list of which souls are to be loaded.
 * The souls are listed in order of command search.
 */
string *
get_soul_list()
{
    return ({ WIZ_CMD_LORD,
	      WIZ_CMD_NORMAL,
	      WIZ_CMD_PILGRIM,
	      WIZ_CMD_APPRENTICE
	  });
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return WIZ_LEV_NAME(WIZLEV_LORD);
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "accept":"accept",

	     "demote":"demote",
	     "deny":"deny",

	     "expel":"expel",

	     "limit":"limit",

	     "pave":"pave",
	     "plough":"plough",
	     "promote":"promote",

	     "remap":"remap"
	     ]);
}

/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/

/* **************************************************************************
 * accept - accept someone into a domain.
 */
int
accept(string name)
{
    string err;

    CHECK_SO_LORD;

    write(SECURITY->accept_application(name) + "\n");

    return 1;
}

/* **************************************************************************
 * demote - demote a player to a lower level
 */
int
demote(string arg)
{

    int             lev;
    string          wizname, my_domain;

    CHECK_SO_LORD;

    if (!strlen(arg) || sscanf(arg, "%s %d", wizname, lev) != 2)
    {
	write("usage: demote player_name new_level\n");
	return 1;
    }
    if (lev < 0 || lev > WIZLEV_KEEPER)
    {
	notify_fail("Level has to be in the range 0-" + WIZLEV_KEEPER + ".\n");
	return 0;
    }
    if (SECURITY->query_wiz_level(wizname) <= lev)
    {
	notify_fail("Demote is used for decreasing levels.\n");
	return 0;
    }
    if (WIZ_TYPE(MYLEV) == WIZ_LORD)
    {
	my_domain = SECURITY->query_wiz_dom(this_player()->query_real_name());

	if (lev < WIZLEV_APPRENTICE && 
	    SECURITY->query_domain_madwand(my_domain) != wizname)
	{
	    notify_fail("You can't demote to levels below " + WIZLEV_APPRENTICE + "\n");
	    return 0;
	}
    }
    write(SECURITY->change_wiz_level(wizname, lev) + "\n");

    return 1;
}

/* **************************************************************************
 * deny - deny an apprentice' request.
 */
int
deny(string name)
{
    CHECK_SO_LORD;

    if (!strlen(name))
    {
	notify_fail("Deny who?\n");
	return 0;
    }

    write((string)SECURITY->deny_application(name) + "\n");

    return 1;
}

/* **************************************************************************
 * expel - expel a wizard from a domain
 */
int
expel(string wizname)
{
    CHECK_SO_LORD;

    if (!strlen(wizname))
    {
	notify_fail("usage: expel player_name\n");
	return 0;
    }
    if (!SECURITY->exist_player(wizname))
    {
	notify_fail("Can't find player " + wizname + "\n");
	return 0;
    }
    write (SECURITY->expel_wizard_from_domain(wizname) + "\n");
    return 1;
}

/* **************************************************************************
 * limit - limit the number of players in a domain
 */
int
limit(string arg)
{
    int n_num;
    string domain, *args;

    CHECK_SO_LORD;

    domain = SECURITY->query_wiz_dom(this_interactive()->query_real_name());

    if (!strlen(arg))
    {
	if (!strlen(domain))
	{
	    notify_fail("You're not a member of any domain!\n");
	    return 0;
	}
	write("Current maximum: " + SECURITY->query_domain_max(domain) +
	      " wizards.\n");
	return 1;
    }
    args = explode(arg + " ", " ");

    if (sscanf(args[0], "%d", n_num) == 1)
    {
	if (sizeof(args) > 1)
	    domain = args[1];

	write(SECURITY->set_domain_max(n_num, domain) + "\n");
    }
    else
	write("Current maximum in " + args[0] + " is " +
	      SECURITY->query_domain_max(args[0]) + " wizards.\n");

    return 1;
}

/* **************************************************************************
 * promote - promote a wizard to a higher level.
 */
int
promote(string arg)
{

    int             lev;
    string          wizname;

    CHECK_SO_LORD;

    if (!strlen(arg) || sscanf(arg, "%s %d", wizname, lev) != 2)
    {
	notify_fail("usage: promote player_name new_level\n");
	return 0;
    }
    if (lev < 0 || lev > WIZLEV_KEEPER)
    {
	notify_fail("The level has to be in the range 0-" + WIZLEV_KEEPER + ".\n");
	return 0;
    }
    if (!SECURITY->exist_player(wizname))
    {
	notify_fail("Can't find player " + wizname + "\n");
	return 0;
    }
    if (SECURITY->query_wiz_level(wizname) >= lev)
    {
	notify_fail("Promote is used for increasing levels.\n");
	return 0;
    }
    write(SECURITY->change_wiz_level(wizname, lev) + "\n");
    return 1;
}

int remap(string arg) {

    int x;
    int y;
    int tint;
    string ttype;

    CHECK_SO_LORD;

    if(!strlen(arg) || sscanf(arg, "%d %d %s", x, y, ttype) != 3) {
	notify_fail("usage: remap x y terrain\n");
	return 0;
    }
    if((tint = MAP_OBJ->tstring_to_tint(ttype)) < 1) {
	notify_fail("Unknown terrain type: " + ttype + "\n");
	return 0;
    }

    if(!MAP_OBJ->modify_square(x, y, tint))
	notify_fail("Modification failed\n");

    return 1;
}

int pave(string arg) {
    int x;
    int y;
    string dirs;

    CHECK_SO_LORD;

    if(!strlen(arg) || sscanf(arg, "%d %d %s", x, y, dirs) != 3) {
	notify_fail("usage: pave x y dirs\n");
	return 0;
    }

    x = MAP_OBJ->set_road(x, y, dirs+"R");
    
    if(!x) {
	notify_fail("Modification failed\n");
	return 0;
    }
    
    if(x == -1) {
	notify_fail("Not a road location.\n");
	return 0;
    }

    return 1;
}

int plough(string arg) {
    int x;
    int y;
    string dirs;
    string flows;

    CHECK_SO_LORD;

    if(!strlen(arg) || sscanf(arg, "%d %d %s,%s", x, y, dirs, flows) != 4) {
	notify_fail("usage: plough x y dirs,flow_dirs\n");
	return 0;
    }

    x = MAP_OBJ->set_river(x, y, dirs, flows);
    switch(x) {
    case 1:
	return 1;
    case 0:
	notify_fail("usage: plough x y dirs,flow_dirs\n");
	return 0;
    case -1:
	notify_fail("Not a river location.\n");
	return 0;
    case -2:
	notify_fail("Outside map.\n");
	return 0;
    case -3:
	notify_fail("Illegal direction.\n");
	return 0;
    case -4:
	notify_fail("When four rivers are specified, one or three flows must be given.\n");
	return 0;
    case -5:
	notify_fail("You specified more flows than rivers.\n");
	return 0;
    case -6:
	notify_fail("No river in specified flow direction.\n");
	return 0;
    case -7:
	notify_fail("Unknown flow direction.\n");
	return 0;
    case -8:
	notify_fail("Wrong number of rivers or flows specified.\n");
	return 0;
    default:
	notify_fail("Unknown return value: " + x + "\n");
	return 0;
	break;
    }
    return 1;
}

