/*
 * wiz_cmd_normal.c
 *
 * This object holds the normal wizards commands.
 *
 */

#pragma strict_types
#pragma save_binary

inherit "/cmd/std/command_driver";

#include "/cmd/std/tracer_tool_base.c";

#include "/sys/macros.h"
#include "/sys/log.h"
#include "/sys/filepath.h"
#include "/secure/std.h"
#include "/sys/language.h"
#include "/sys/stdproperties.h"

#define CHECK_SO_WIZ 	if (WIZ_CHECK < WIZ_NORMAL) return 0; \
			if (this_interactive() != this_player()) return 0

#define SHUT_REASON 	"_shut_reason"

static int somelog(string str, string logname, string log);
static int valid_possess(object demon, object possessed);
void do_shut_down(mixed *arg);
int tellall(string str);
static object clone_ob(string what);

/* **************************************************************************
 * Return a list of which souls are to be loaded.
 * The souls are listed in order of command search.
 */
string *
get_soul_list()
{
    return ({ WIZ_CMD_NORMAL,
	      WIZ_CMD_PILGRIM,
	      WIZ_CMD_APPRENTICE });
}

/* **************************************************************************
 * Return a proper name of the soul in order to get a nice printout.
 */
string
get_soul_id()
{
    return WIZ_LEV_NAME(WIZLEV_NORMAL);
}

/* **************************************************************************
 * The list of verbs and functions. Please add new in alfabetical order.
 */
mapping
query_cmdlist()
{
    return ([
	     "banish":"banish",
	     "buglog":"buglog",

	     "clone":"clone",
	     "cmdsoul":"cmdsoul",
	     "control":"control",
	     "cp":"cp_cmd",

	     "destruct":"destruct_ob",
	     "distrust":"distrust",
	     "donelog":"donelog",
	     "du":"du",

	     "echo":"echo",
	     "echoto":"echo_to",
	     "ed":"ed_file",
	     "errlog":"errlog",
	     "exec":"exec_code",

	     "force":"force",

	     "idealog":"idealog",

	     "indent":"indent",
	     "invis":"invis",

	     "leave":"leave",
	     "load":"load",

	     "mkdir":"makedir",
	     "mv":"mv_cmd",

	     "possess":"possess",
	     "praiselog":"praiselog",

	     "remake":"remake_object",
	     "rm":"rm_cmd",
	     "rmdir":"removedir",

	     "sdoc":"sdoc",
	     "shutdown":"shutdown_game",
	     "snoop":"snoop_on",

	     "tellall":"tellall",
	     "toolsoul":"toolsoul",
	     "trans":"trans",
	     "trust":"trust_ob",
	     "typolog":"typolog",

	     "update":"dupd",

	     "vis":"vis",

	     "wizlist":"wiz_score_list"
	     ]);
}
/* 
 * Here are some supplementary functions
 */
 
   /*
	  New functions that handle multiple files with single command
	  Commands: cp, mv, rm
	  Made by Tero.Pelander@utu.fi
	  Freely distributable for non commercial use
    */

/*
 * This code supposes rename() returns 1 when success and 0 when fail as all
 * other file functions. This is otherwise in some (older) GD versions.
 * Has been corrected in CD?? If not, let that that ODD_RENAME stick around.
 *
 * This has not been fixed in CDlib, yet. /JnA
 */
#define ODD_RENAME

/*
 * Maxinum number of files that can be copied/moved/removed with one command
 * Protects from errors like 'rm /std/*' or 'cp /d/Std/* .'
 */
#define MAXFILES 20

/*
 * Function name: copy
 * Description:   Copy a (text) file. Limited to about 50kB files by GD. That
 *                could be circumvented but very few files are larger than
 *                that.
 * Arguments:     From file path, to file path.
 * Returns:       1 if success, otherwise 0.
 */
private int
copy(string path1, string path2)
{
    string buf;

    /* Read file (and test if success) */
    if (!(buf = read_file(path1)))
        return 0;

    /* Test if target file exists and proceed accordingly */
    switch (file_size(path2))
    {
        case -2: return 0;
        case -1:
        case  0: break;
        default: if (!rm(path2)) return 0;
    }
    if (!write_file(path2,buf))
        return 0;
    return 1;
}


#define MULTI_CP 0
#define MULTI_MV 1
#define MULTI_RM 2
#define MULTI_STRS ({"Copied","Moved","Removed"})
/*
 * Function name: do_multi_file
 * Description:   Commands containing multiple files are executed here.
 * Arguments:     operation: MULTI_CP, MULTI_MV, MULTI_RM; what command is done
 *                p1: first path
 *                p2: (optional) second path
 * Returns:       1 if arguments are valid, otherwise 0.
 */
private int
do_multi_file(int operation, string p1, string p2)
{
    string tmp;
    string *old_names, *new_names;
    int    i, count_files, count_dir;

    p1 = FTPATH(this_interactive()->query_path(), p1);

    old_names = get_dir(p1);
    if (!old_names || !sizeof(old_names))
    {
        write("File(s) not found.\n");
        return 0;
    }
    /* Let's first remove possible "non-real" directories */
    old_names -= ({ ".", ".." });
    if (sizeof(old_names) > MAXFILES)
    {
        write("Selected over "+MAXFILES+" files.\n");
        return 0;
    }

    /* Remove file name from p1; "/ftp/*.c" ==> "/ftp/" */
    i = strlen(p1)-1;
    while (p1[i] != '/') --i;
    p1 = p1[0..i];

    /* Is p2 directory or a file name */
    if (p2)
    {
        p2 = FTPATH(this_interactive()->query_path(), p2);
        if (file_size(p2+"/") != -2)
        {
            if (sizeof(old_names) != 1)
            {
                write("Destination must be a directory for multiple copy/move.\n");
                return 0;
            }
            /* let's use a dummy so we don't need to fix p2 */
            new_names = ({ "" });
        }
        else
        {
            p2 = p2+"/";
            new_names = old_names;
        }
    }

    /* First one special case; moving a directory */
    if (operation == MULTI_MV && sizeof(old_names) == 1
        && file_size(p1+old_names[0])==-2)
    {
        write(
#ifdef ODD_RENAME
            (!rename(p1+old_names[0], p2+new_names[1])) ?
#else
            rename(p1+old_names[0], p2+new_names[1]) ?
#endif
            "Moved a directory.\n" :
            "Failed to move a directory.\n" );
        return 1;
    }

    /* Loop for real work */
    for (count_dir = count_files = i = 0;
        i < sizeof(old_names) && i == (count_files + count_dir); ++i)
    {
        tmp = p1 + old_names[i];
        if (file_size(tmp) < 0)
        {
            count_dir++;
        }
        else
        {
            switch (operation)
            {
            case MULTI_CP:
                count_files += copy(tmp, p2+new_names[i]);
                break;
            case MULTI_MV:
                if (file_size(p2+new_names[i]) >= 0)
                    rm(p2+new_names[i]);
#ifdef ODD_RENAME
                count_files += rename(tmp, p2+new_names[i]) ^ 1;
#else
                count_files += rename(tmp, p2+new_names[i]);
#endif
                break;
            case MULTI_RM:
                count_files += rm(tmp);
                break;
            }
        }
    }
    if (count_files + count_dir != sizeof(old_names))
        write("Failed at file '"+tmp+"'.\n");
    write(MULTI_STRS[operation] + " "
        + count_files + " file" + ((count_files==1) ? "" : "s") + ".\n");
    return 1;
}


/*
 * Function name:   exec_write
 * Description:     Write an object file with a function exec_fun()
 *		    that is to be executed. The filename of that file
 *		    is ~/exec_obj.c.
 * Arguments:	    str: the body of the function exec_fun().
 * Returns:	    1 if executing was succesful,
 *		    0 if not; in that case an errormessage has been
 *		    set with notify_fail().
 * Caveats:	    If "str" contains non-executable code, the object
 *		    cannot be loaded, and the gamedriver will give an
 *		    error.
 */
int
exec_write(string str)
{
    object ob;
    string file, master;
    mixed error;

    seteuid(getuid());

    if (!str)
    {
	notify_fail("Nothing to execute.\n");
	return 0;
    }

    file = SECURITY->query_wiz_path(this_player()->query_real_name());

    if (!file)
    {
	notify_fail("Cannot write file.\n");
	return 0;
    }

    master = file + "/exec_obj";

    /* Remove the old copy */
    rm(master + ".c");

    /* Write the new file */
    if (!write_file(master + ".c",
		    "inherit \"/std/object\";"
		    + "\n"
		    + "#include \"/secure/std.h\"\n"
		    + "#include \"/sys/cmdparse.h\"\n"
		    + "#include \"/sys/comb_mag.h\"\n"
		    + "#include \"/sys/composite.h\"\n"
		    + "#include \"/sys/const.h\"\n"
		    + "#include \"/sys/drink_eat.h\"\n"
		    + "#include \"/sys/filepath.h\"\n"
		    + "#include \"/sys/filter_funs.h\"\n"
		    + "#include \"/sys/formulas.h\"\n"
		    + "#include \"/sys/herb.h\"\n"
		    + "#include \"/sys/language.h\"\n"
		    + "#include \"/sys/log.h\"\n"
		    + "#include \"/sys/macros.h\"\n"
		    + "#include \"/sys/map.h\"\n"
		    + "#include \"/sys/math.h\"\n"
		    + "#include \"/sys/money.h\"\n"
		    + "#include \"/sys/mudtime.h\"\n"
		    + "#include \"/sys/obflags.h\"\n"
		    + "#include \"/sys/poison_types.h\"\n"
		    + "#include \"/sys/seqaction.h\"\n"
		    + "#include \"/sys/ss_types.h\"\n"
		    + "#include \"/sys/state_desc.h\"\n"
		    + "#include \"/sys/stdproperties.h\"\n"
		    + "#include \"/sys/subloc.h\"\n"
		    + "#include \"/sys/udp.h\"\n"
		    + "#include \"/sys/wa_types.h\"\n"
		    + "\nexec_fun() {\n    seteuid(getuid());\n    {\n"
		    + str
		    + "\n    }\n}\n"))
    {
	notify_fail("Cannot write file.\n");
	return 0;
    }

    /* Update the object, so it will be reloaded */
    ob = find_object(master);
    if (ob)
	SECURITY->do_debug("destroy", ob);
    
    /* Clone the object and call the function exec_fun() in it */
    if (error = catch(ob = clone_object(master)))
    {
	notify_fail("\nError when loading: " + error + "\n");
	return 0;
    }
    write("Executing...\n"); /* Perhaps a call_out is wiser... */
    if (error = catch(ob->exec_fun()))
	write("\nError when executing: " + error + "\n");
    else
	write("\nDone.\n");

    /* Update the object, to spare memory */
    ob->remove_object();
    ob = find_object(master);
    if (ob)
	SECURITY->do_debug("destroy", ob);

    return 1;
}

/*
 * Function name:   exec_edit
 * Description:     Allow players to write multiple lines of code
 * Arguments:	    The text the player entered.
 * Returns:	    1
 */
int
exec_edit(string str)
{
    object tp;
    int quit_exec, error;

    tp = this_player();
    quit_exec = 0;

    if (str == "~q")
    {
	write("Aborted.\n");
	tp->remove_prop("_player_exec_text");
    }
    else if (str != "**")
    {
	if (tp->query_prop("_player_exec_text"))
	    tp->add_prop("_player_exec_text",
			 tp->query_prop("_player_exec_text") + str + "\n");
	else
	    tp->add_prop("_player_exec_text", str + "\n");

	write("]");
	input_to("exec_edit");
    }
    else
	write("Ok, use 'exec' to save and execute it.\n");
    return 1;
}

/* **************************************************************************
 * Here follows the actual functions. Please add new functions in the 
 * same order as in the function name list.
 * **************************************************************************/

/* **************************************************************************
 * banish - banish a name
 */
int
banish(string arg)
{
    int wtype;
    string *argv, name, what;
    mixed *rval;

    if (strlen(arg) == 0)
    {
	notify_fail("Banish who?\n");
	return 0;
    }

    argv = explode(arg, " ");

    if (sizeof(argv) == 2)
    {
	what = argv[0];
	name = argv[1];
    }
    else
    {
	what = "-i";
	name = argv[0];
    }
    
    if (SECURITY->exist_player(name))
    {
	notify_fail(capitalize(name) + " is a player in the game.\n");
	return 0;
    }

    wtype = WIZ_TYPE(SECURITY->query_wiz_level(this_interactive()->
					       query_real_name()));
    switch (what)
    {
    case "-a":
    case "-add":
	/*
	 * Arch and keeper can banish anywhere.
	 */
	if (wtype != WIZ_KEEPER && wtype != WIZ_ARCH)
	{
	    if (function_exists("create_workroom", 
		environment(this_interactive())) != ADMIN_HOME)
	    {
		notify_fail("You can only banish names in the administrators workroom.\n");
		return 0;
	    }
	}
	    
	/*
	 * Try to banish.
	 */
	rval = SECURITY->banish(name, 2);
	
	if (sizeof(rval) != 0)
	    write(capitalize(name) + " was banished by " + 
		  capitalize(rval[0]) + " at " + ctime(rval[1]) + ".\n");
	else
	    write("You have banished: " + capitalize(name) + ".\n");
	
	break;
	
    case "-i":
    case "-info":
	/*
	 * Look for information.
	 */
	rval = SECURITY->banish(name, 0);
	if (sizeof(rval) != 0)
	    write(capitalize(name) + " was banished by " + 
		  capitalize(rval[0]) + " at " + ctime(rval[1]) + ".\n");
	else
	    write(capitalize(name) + " has not been banished.\n");
	break;
	
    case "-r":
    case "-remove":
	/*
	 * Try to remove.
	 */
	rval = SECURITY->banish(name, 0);
	if (sizeof(rval) != 0)
	{
	    if (wtype != WIZ_KEEPER && wtype != WIZ_ARCH)
	    {
		if (rval[0] != this_interactive()->query_real_name())
		{
		    notify_fail("You have not banished " + 
				capitalize(name) + ".\n");
		    return 0;
		}
	    }
	}
	else
	{
	    notify_fail(capitalize(name) + " has not been banished.\n");
	    return 0;
	}

	rval = SECURITY->banish(name, 1);
	if (sizeof(rval) != 0)
	{
	    write(capitalize(name) + " was banished by " + 
		  capitalize(rval[0]) + " at " + ctime(rval[1]) + ".\n");
	    write("You have removed the banishment of " + 
		  capitalize(name) + ".\n");
	}

	break;
	
    default:
	break;
    }
    return 1;
}

/* **************************************************************************
 * buglog - list a buglog
 */
int
buglog(string str) 
{ 
    return somelog(str, "bug log", "/bugs"); 
}

/* **************************************************************************
 * clone - clone an object
 */
int
clone(string str)
{
    object ob;
    int num, argc;
    string *argv;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Clone what object ?\n");
	return 0;
    }

    argv = explode(str, " ");
    argc = sizeof(argc);

    switch (argv[0])
    {
    case "-i":
	ob = clone_ob(argv[1]);
	if (!ob)
	    return 0;
	ob->move(this_interactive(), 1);
	write("Ok.\n");
	break;

    case "-e":
	ob = clone_ob(argv[1]);
	if (!ob)
	    return 0;
	ob->move(environment(this_interactive()), 1);
	write("Ok.\n");
	break;

    default:
	ob = clone_ob(argv[0]);
	if (!ob)
	    return 0;

	num = (int)ob->move(this_interactive());
	switch (num)
	{
	case 0:
	    write("Ok.\n");
	    break;
	    
	case 1:
	    write("Too heavy for destination.\n");
	    break;
	    
	case 2:
	    write("Can't be dropped.\n");
	    break;
	    
	case 3:
	    write("Can't take it out of it's container.\n");
	    break;
	    
	case 4:
	    write("The object can't be inserted into bags etc.\n");
	    break;
	    
	case 5:
	    write("The destination doesn't allow insertions of objects.\n");
	    break;
	    
	case 6:
	    write("The object can't be picked up.\n");
	    break;
	    
	case 7:
	    write("Other (Error message printed inside move() function).\n");
	    break;
	    
	case 8:
	    write("Too big volume for destination.\n");
	    break;
	    
	default:
	    write("Strange, very strange error in move: " + num + "\n");
	    break;
	}
	if (num)
	    num = (int)ob->move(environment(this_interactive()));
	break;
    }
    return 1;
}

static object
clone_ob(string what)
{
    string str, mess;
    object ob;

    str = FTPATH((string)this_interactive()->query_path(), what);
    if (!strlen(str))
    {
	notify_fail("Invalid file.\n");
	return 0;
    }

    if (file_size(str + ".c") < 0 && file_size(str) < 0)
    {
	notify_fail("No such file.\n");
	return 0;
    }
    
    ob = clone_object(str);
    if (!ob)
    {
	notify_fail("You can not clone: " + str + "\n");
	return 0;
    }
    mess = " fetches something from another dimension.\n";
    say( ({ METNAME + mess, TART_NONMETNAME + mess, "" }) );

    return ob;
}

/* **************************************************************************
 * cmdsoul - affect your command souls
 */
int
cmdsoul(string str)
{
    string *cmdsoul_list;

    CHECK_SO_WIZ;

    cmdsoul_list = (string *)this_interactive()->query_cmdsoul_list();
    if (!strlen(str))
    {
	write(break_string("Current tools: " + implode(cmdsoul_list, ", ") + "\n", 76));
	return 1;
    }

    str = FTPATH((string)this_interactive()->query_path(), str);

    if (member_array(str, cmdsoul_list) >= 0)
    {
	if (this_interactive()->remove_cmdsoul(str))
	    write("Removed.\n");
	else
	    write("Failed.\n");
	return 1;
    }

    if (this_interactive()->add_cmdsoul(str))
	write("Added.\n");
    else
	write("Failed.\n");
    return 1;
}


/* **************************************************************************
 * control - Get a 'control wand' for a specific monster
 */
int
control(string npc)
{
    object mon, owner;
    string str;

    CHECK_SO_WIZ;

    notify_fail("Control who?\n");

    if (!npc)
	return 0;

    mon = present(npc, environment(this_player()));

    if (!mon)
	mon = find_living(npc);

    if (!mon)
    {
	notify_fail("I can't seem to locate " + capitalize(npc) + "\n");
	return 0;
    }

    if (!mon->query_npc())
    {
	notify_fail("You feel a strong resistance towards control.\n");
	return 0;
    }

    if (owner = (object)mon->query_link_remote())
    {
	owner = environment(owner);
	if (living(owner))
	    str = "is held by " + owner->short();
	else
	    str = "seems to be in: " + owner->short();
	write(capitalize(npc) + " is already controlled. The wand " +
	      str + "\n");
	return 1;
    }
    mon->set_link_remote();
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * cp - copy multiple files (New wildcard version)
 */
int
cp_cmd(string arg)
{
    string p1,p2;

    CHECK_SO_WIZ;

    if (!arg || (sscanf(arg,"%s %s",p1,p2)!=2)) {
        notify_fail("Usage: cp <from> <to>\n");
        return 0;
    }
    do_multi_file(MULTI_CP, p1, p2);
    return 1;
}

/* **************************************************************************
 * destruct - destruct an object
 */
int
destruct_ob(string str)
{
    object ob;
    int dflag;
    string a;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Destruct what ?\n");
	return 0;
    }
    if (sscanf(str, "-D %s",a) == 1)
    {
	dflag = 1;
	str = a;
    }

    ob = present(str, this_interactive());
    if (!ob)
	ob = present(str, environment(this_interactive()));

    if (!ob)
    {
	write("No " + str + " here.\n");
	return 1;
    }
    
    
    if (living(ob))
	say(QCTNAME(ob) + " is disintegrated by " + 
	    QTNAME(this_interactive()) + ".\n");
    else
    {
	a = capitalize(LANG_ASHORT(ob)) + " is disintegrated by ";
	if (member_array(this_interactive(), all_environment(ob)) != -1)
	    say(({ a + METNAME + ".\n", 
		   a + "the " + this_player()->query_nonmet_name() + ".\n", 
		   "" }) );
	else
	    say(a + QTNAME(this_interactive()) + ".\n");
    }

    if (ob != this_object())
    {
	if (dflag)
	    SECURITY->do_debug("destroy", ob);
	else
	    ob->remove_object();
    }
    else
    {
	write("Ok.\n");
	destruct();
	return 1;
    }
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * distrust - distrust an object
 */
int
distrust(string str)
{
    object ob;

    CHECK_SO_WIZ;

    if (!str)
    {
	notify_fail("Distrust what object?\n");
	return 0;
    }

    ob = present(str, this_interactive());

    if (!ob)
	ob = present(str, environment(this_interactive()));

    if (!ob)
	ob = parse_list(str);

    if (!ob) 
    {
	notify_fail("Object not found: " + str + "\n");
	return 0;
    }

    if (geteuid(ob) != geteuid(this_object()))
    {
	notify_fail("Object not trusted by you.\n");
	return 0;
    }

    /* Remove the previous euid */
    ob->set_trusted(0);

    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * donelog - list donelog
 */
int
donelog(string str) 
{ 
    return somelog(str, "done log", "/done"); 
}

/* **************************************************************************
 * du - calculate disk usage
 */
static int      xdu(string p, int af);

int
du(string str)
{
    int aflag;
    string p, flag, path;
    
    if (!str)
    {
	path = ".";
    }
    else
    {
	/* There is no getopt in CDlib... or? */
	if (sscanf(str, "%s %s", flag, path) == 2)
	{
	    if (flag != "-a")
	    {
		notify_fail("usage: du [-a] [path]\n");
		return 0;
	    }
	    else
		aflag = 1;
	}
	else
	{
	    if (str == "-a")
	    {
		aflag = 1;
		path = ".";
	    }
	    else
		path = str;
	}
    }
    p = FTPATH(this_interactive()->query_path(), path);
    
    if (p == "/")
	p = "";
    
    xdu(p, aflag);

	return 1;
}

static int
xdu(string path, int aflag)
{
    int sum, i;
    string *files, output;
    
    files = get_dir(path + "/*");
    
    sum = 0;
    
    for (i = 0; i < sizeof(files); i++)
    {
	if (files[i] == "." || files[i] == "..")
	    continue;
	
	if (aflag && file_size(path + "/" + files[i]) > -1)
	{
	    write(file_size(path + "/" + files[i]) / 1024 + "\t" +
		  path + "/" + files[i] + "\n");
	}

	if (file_size(path + "/" + files[i]) == -2)
	    sum += xdu(path + "/" + files[i], aflag);
	else
	    sum += file_size(path + "/" + files[i]);
    }
    
    write(sum / 1024 + "\t" + path + "\n");
    return sum;
}


/* **************************************************************************
 * echo - echo something in a room
 */
int
echo(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Echo what?\n");
	return 0;
    }
    str = implode(explode(str, "@@"), "#");
    say(str + "\n");
    write("You echo: " + str + "\n");
    return 1;
}

/* **************************************************************************
 * echoto - echo something to someone
 */
int
echo_to(string str)
{
    object ob;
    string who, msg;

    CHECK_SO_WIZ;

    if (!strlen(str) || sscanf(str, "%s %s", who, msg) != 2)
    {
	notify_fail("Echoto what ?\n");
	return 0;
    }
    ob = find_living(lower_case(who));
    if (!ob)
    {
	notify_fail("No player with that name.\n");
	return 0;
    }
    ob->catch_msg(msg + "\n");
    write("You echo: " + msg + "\n");
    return 1;
}

/* **************************************************************************
 * ed - edit a file
 */
int 
ed_file(string file)
{
    CHECK_SO_WIZ;

    if (!strlen(file))
    {
	ed();
	return 1;
    }
    file = FTPATH((string)this_interactive()->query_path(), file);
    ed(file);
    return 1;
}

/* **************************************************************************
 * errlog - list an error log
 */
int
errlog(string str) 
{ 
    return somelog(str, "error log", "/errors"); 
}

/* **************************************************************************
 * exec - Execute one or more lines of code
 */
int
exec_code(string str)
{
    string code;
    int error;

    if (!str)
    {
	code = this_player()->query_prop("_player_exec_text");
	if (!code || !strlen(code))
	{
	    this_player()->add_prop("_player_exec_text", "");
	    write("Enter code to execute (** to end, ~q to abort).\n]");
	    input_to("exec_edit");
	    return 1;
	}
	else
	{
	    error = exec_write(code);
	    this_player()->remove_prop("_player_exec_text");
	    return error;
	}
    }
    if (str == "clear")
    {
	this_player()->remove_prop("_player_exec_text");
	write("Ok.\n");
	return 1;
    }
    return exec_write(str);
}

/* **************************************************************************
 * force - force a player to do something
 */
int
force(string str)
{
    string who, what;
    object ob;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Force who what?\n");
	return 0;
    }

    if (sscanf(str, "%s to %s", who, what) == 2 ||
	sscanf(str, "%s %s", who, what) == 2)
    {
	ob = find_living(who);
	if (!ob)
	{
	    notify_fail("No such player.\n");
	    return 0;
	}

	if (ob->command(what))
	{
	    ob->catch_msg(QCNAME(this_interactive()) + " forced you to: " + what + "\n", this_interactive());
	    write("Ok.\n");
	}
	else
	{
	    ob->catch_msg(QCNAME(this_interactive()) + " tried to force you to: " + what + "\n", this_interactive());
	    write("You failed.\n");
	}
	return 1;
    }
    return 0;
}

/* **************************************************************************
 * idealog - read the idea log
 */
int
idealog(string str) 
{ 
    return somelog(str, "idea log", "/ideas"); 
}

/* **************************************************************************
 * indent - indent a file
 */
int
indent(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Indent what file?\n");
	return 0;
    }

    if (!cindent(FTPATH((string)this_interactive()->query_path(), str)))
	write("Indent failed.\n");
    return 1;
}

/* **************************************************************************
 * invis - become invisible
 */
int
invis()
{
    CHECK_SO_WIZ;

    if ((int)this_interactive()->query_invis())
    {
	tell_object(this_interactive(), "You are already invisible.\n");
	return 1;
    }
    write("You are now invisible.\n");
    say(QCTNAME(this_interactive()) + " " + (string)this_interactive()->query_mm_out() + "\n");
    this_interactive()->set_invis(1);
    return 1;
}

/* **************************************************************************
 * leave - leave a domain
 */
int
leave(string str)
{
    string name;

    CHECK_SO_WIZ;

    if (str != "domain")
    {
	notify_fail("Syntax: leave domain\n");	
	return 0;
    }

    name = lower_case((string)this_interactive()->query_real_name());

    write(SECURITY->leave_domain() + "\n");

    return 1;
}

/* **************************************************************************
 * load - load a file
 */
int
load(string str)
{
    object env;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Load what ?\n");
	return 0;
    }
    str = FTPATH(this_interactive()->query_path(), str);
    if (!strlen(str))
    {
	notify_fail("Invalid file name.\n");
	return 0;
    }
    
    if (file_size(str + ".c") < 0 && file_size(str) < 0)
    {
	notify_fail("No such file.\n");
	return 0;
    }

    env = environment();
    str->teleledningsanka();
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * mkdir - make a directory
 */
int
makedir(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Make what dir?\n");
	return 0;
    }
    if (mkdir(FTPATH((string)this_interactive()->query_path(), str)))
	write("Ok.\n");
    else
	write("Fail.\n");
    return 1;
}


/* **************************************************************************
 * mv - move multiple files or a sigle directory (New wildcard version)
 */
int
mv_cmd(string arg)
{
    string p1,p2;

    CHECK_SO_WIZ;

    if (!arg || (sscanf(arg,"%s %s",p1,p2)!=2)) {
        notify_fail("Usage: mv <from> <to>\n");
        return 0;
    }
    do_multi_file(MULTI_MV, p1, p2);
    return 1;
}

/* **************************************************************************
 * possess - possess a living object
 */
nomask int
possess(string arg)
{
    object victim, possob;
    string *argv, v_name, p_name;
    int argc;

    CHECK_SO_WIZ;

    argv = explode(arg, " ");
    argc = sizeof(argv);

    if (argc == 0)
    {
	notify_fail("Possess what?\n");
	return 0;
    }

    /*
     * Find the victim of this evil spell.
     */
    if (argc == 1)
	v_name = lower_case(argv[0]);
    else
	v_name = lower_case(argv[1]);

    victim = present(v_name, environment(this_interactive()));
	
    if (!victim)
	victim = find_living(v_name);

    if (!victim)
    {
	notify_fail("No living object with that name.\n");
	return 0;
    }

    p_name = (string)victim->query_possessed();

    if (!living(victim))
    {
	notify_fail("You can only possess living beings.\n");
	return 0;
    }

    if (victim == this_interactive())
    {
	notify_fail("Hey! You're already in your head. For more self-control, go see a shrink.\n");
	return 0;
    }

    if (!valid_possess(this_interactive(), victim))
    {
	notify_fail("Sorry, you're not allowed to do that.\n");
	return 0;
    }

    /*
     * Only an Arch or Keeper can break or force a possession.
     */
    if (argc == 2 && WIZ_TYPE((int)SECURITY->query_wiz_level(geteuid(this_interactive()))) >= WIZ_ARCH)
    {
	switch (argv[0])
	{
	case "force":
	    if (strlen(p_name))
	    {
		victim->command("quit");
		write("You exorcised " + capitalize(p_name) + ".\n");
		tell_object(find_player(p_name), "You were exorcised by " + capitalize((string)this_interactive()->query_real_name()) + ".\n");
	    }
	    else
		write(capitalize(v_name) + " is unpossessed.\n");
	    break;

	case "break":
	    if (strlen(p_name))
	    {
		victim->command("quit");
		write("You exorcised " + capitalize(p_name) + ".\n");
		tell_object(find_player(p_name), "You were exorcised by " + capitalize((string)this_interactive()->query_real_name()) + ".\n");
	    }
	    else
		write(capitalize(v_name) + " is unpossessed.\n");
	    return 1;
	    break;

	default:
	    notify_fail("Strange command: " + argv[0] + ".\n");
	    return 0;
	    break;
	}
    }
    else
    {
	if (strlen((string)victim->query_possessed()))
	{
	    notify_fail(capitalize(v_name) + " is already possessed by " + capitalize(p_name) + ".\n");
	    return 0;
	}
    }

    if (!query_ip_name(victim))
    {
	if ((int)SECURITY->exist_player(v_name))
	{
	    notify_fail("You can't possess an object with the same name as a player.\n");
	    return 0;
	}
    }

    /*
     * Create the control object and do the possession.
     */
    possob = clone_object("/std/possob");
    possob->move(victim, 1);
    possob->set_name(v_name);
    if (possob->possess(this_interactive(), victim))
    {
	snoop(possob, victim);
	possob->set_lock();
    }
    else
	possob->remove_object();
    
    return 1;
}

static int
valid_possess(object demon, object possessed)
{
    int efflevel;
    string dom, by, on;

    efflevel = (int)SECURITY->query_wiz_level(geteuid(demon));

    /* Arch & keeper possesses all everywhere */
    if (WIZ_TYPE(efflevel) >= WIZ_ARCH)
	return 1;

    by = geteuid(demon);
    dom = (string)SECURITY->query_wiz_dom(by);
    on = geteuid(possessed);

    if (WIZ_TYPE(efflevel) < WIZ_ARCH &&
	WIZ_TYPE((int)SECURITY->query_wiz_level(on)) > WIZ_LORD)
	return 0;

    /* Lords can possess members & member's objects everywhere */
    if (WIZ_TYPE(efflevel) == WIZ_LORD)
    {
	if ((string)SECURITY->query_domain_lord(dom) == by)
	{
	    if (dom == (string)SECURITY->query_wiz_dom(on))
		return 1;
	}
    }

    /* You can possess objects in your domain */
    if (!query_ip_number(possessed) && (dom == on || dom == SECURITY->query_wiz_dom(on)))
	return 1;

    return 0;
}

/* **************************************************************************
 * praiselog - list your praise log
 */
int
praiselog(string str) 
{ 
    return somelog(str, "praise log", "/praise"); 
}

/* **************************************************************************
 * remake - Remake an object, checks entire dependency of inherited files
 */
int 
remake_object(string str)
{
    object ob;
    string *inherits, *updatem;
    int il, upflag;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Remake what object ?\n");
	return 0;
    }
    str = FTPATH((string)this_interactive()->query_path(), str);
    if (!strlen(str))
    {
	notify_fail("Invalid file name.\n");
	return 0;
    }
    ob = find_object(str);
    if (!ob)
    {
	notify_fail("No such object loaded.\n");
	return 0;
    }
    inherits = SECURITY->do_debug("inherit_list", ob);
    for (updatem = ({}), il = sizeof(inherits) - 1; il >= 0; il--)
    {
	ob = find_object(inherits[il]);
			  
	if (ob && (file_time(inherits[il]) > object_time(ob)))
	    updatem += ({ inherits[il] });
	else if (ob &&
		 (sizeof(updatem & SECURITY->do_debug("inherit_list", ob))))
	    updatem += ({ inherits[il] });
    }
    for (il = 0; il < sizeof(updatem); il++)
    {
	write("Updating: " + updatem[il] + "\n");
	SECURITY->do_debug("destroy", find_object(updatem[il]));/* We already checked... */
    }
    write("\n-----------\nUpdated " + il + " objects.\n");
    /* call_other(str,"teleledningsanka"); */
    write("Ok.\n");
    return 1;
}

/* **************************************************************************
 * rm - remove multiple files
 */
int
rm_cmd(string arg)
{

    CHECK_SO_WIZ;

    if (!arg) {
        notify_fail("Usage: rm <files>\n");
        return 0;
    }
    do_multi_file(MULTI_RM, arg, 0);
    return 1;
}

/* **************************************************************************
 * rmdir - delete a directory
 */
int
removedir(string str)
{
    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Remove what dir?\n");
	return 0;
    }
    if (rmdir(FTPATH((string)this_interactive()->query_path(), str)))
	write("Ok.\n");
    else
	write("Fail.\n");
    return 1;
}

/* **************************************************************************
 * sdoc - extract documentation from file(s)
 */
int
sdoc(string str)
{
    string *argv, *files, *parts, path;
    int argc, i;
    mixed *ord;

    if (!str)
    {
	notify_fail("What do you want to document, see 'help sdoc'\n");
	return 0;
    }

    argv = explode(str, " "); 
    argc = sizeof(argv);

    switch (argv[0])
    {
    case "-r":
	DOCMAKER->doc_reset();
	write("Ok.\n");
	return 1;
	break;
    case "-?":
	write(DOCMAKER->doc_query_status());
	ord = DOCMAKER->doc_query_orders();
	for (i = 0; i < sizeof(ord); i++)
	{
	    write(ord[i][1] + " in " + ord[i][0] + " by: " +
		  implode(ord[i][2], ", ") + "\n");
	}
	return 1;
	break;
    default:
	if (argc < 2)
	{
	    notify_fail("Syntax error, see 'help sdoc'\n");
	    return 0;
	}

	argv[1] = FTPATH(this_player()->query_path(), argv[1]);
	if (sizeof(explode(" " + argv[1] + " ", "*")) == 1)
	{
	    files = ({ argv[1] });
	    path = "";
	}
	else
	{
	    files = get_dir(argv[1]);

	    if (sizeof(files))
		files -= ({ ".", ".." });

	    if (sizeof(parts = explode(argv[1], "/")) > 1)
	    {
		path = "/" + implode(parts[0..sizeof(parts) - 2], "/") + "/";
	    }
	    else 
		path = "/";
	}
	
	if (!sizeof(files))
	{
	    notify_fail("No such file(s): " + argv[1] + "\n");
	    return 0;
	}
		
	write("The Docscribe will be told of your request.\n");

	for (i = 0; i < sizeof(files); i++)
	{
	    if (file_size(path + files[i]) != -2)
		DOCMAKER->doc_file(argv[0], path + files[i]);
	    else
		write(path + files[i] + " is a directory.\n");
	}
	return 1;
	
	break;
    }
}

/* **************************************************************************
 * shutdown - shut the game down
 */
int
shutdown_game(string str)
{
    string *argv;
    int count;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	write("No argument to shutdown.\n");
	return 1;
    }

    argv = explode(str, " ");
    if (sizeof(argv) == 1)
    {
	if (argv[0] == "abort")
	{
	    if (remove_call_out("do_shut_down"))
	    {
		tellall("I have changed my mind. There will be no shutdown. Sorry!\n");
		return 1;
	    }
	}
	else 
	{
	    write("Unvalid argument to shutdown: " + str + "\n");
	    return 1;
	}
    }

    sscanf(argv[0], "%d", count);
    this_interactive()->add_prop(SHUT_REASON, implode(slice_array(argv, 1, sizeof(argv)), " "));

    if (count)
    {
	remove_call_out("do_shut_down");
	do_shut_down(({ this_interactive(), count }));
    }
    else if (argv[0] == "now")
    {
	remove_call_out("do_shut_down");
	do_shut_down(({ this_interactive(), 0 }));
    }
    else
	write("Unvalid argument to shutdown: " + str + "\n");
    return 1;
}

void
do_shut_down(mixed info)
{
    int il;
    object *pl;

    if (info[1])
    {
	tellall("I will shut the game down in " + info[1] + " minutes!");
	call_out("do_shut_down", 60, ({ info[0], info[1] - 1 }));
	return;
    }

    tellall("I am shutting the game down now!");

    pl = users();

    for (il = 0; il < sizeof(pl); il++) 
    {
	catch(pl[il]->save_character());
	tell_object(pl[il], "Saving your character.\n");
    }

    if (!SECURITY->master_shutdown(info[0]->query_prop(SHUT_REASON)))
	tell_object(info[0],
		    "Ooops, master do not allow us to shut down the game!\n");
}

/* **************************************************************************
 * snoop - snoop someone
 */
int
snoop_on(string str)
{
    object on, by;
    string *argv;
    int    argc;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	if (snoop(this_interactive()))
	    write("Snoop terminated.\n");
	else
	    write("Failed to terminate snoop.\n");
	return 1;
    }

    argv = explode(str, " ");

    if (sizeof(argv) == 1)
    {
	on = find_player(argv[0]);
	if (!on)
	{
	    write("No such player.\n");
	    return 1;
	}

	if ((object)SECURITY->query_snoop(on))
	{
	    write(capitalize(argv[0]) + " is already snooped.\n");
	    return 1;
	}

	if (snoop(this_interactive(), on))
	{
	    write("Snooping " + capitalize(argv[0]) + ".\n");
	    return 1;
	}
	else
	    write("Failed to snoop " + capitalize(argv[0]) + ".\n");

	return 1;
    }

    if (argv[0] == "break")
    {
	on = find_player(argv[1]);
	if (!on)
	{
	    write("No such player.\n");
	    return 1;
	}
	if (snoop(on))
	{
	    tell_object(on, capitalize((string)this_interactive()->
		query_real_name()) + " broke your snoop.\n");
	    write("Ok.\n");
	}
	else
	    write("Failed to break snoop on " + capitalize(argv[1]) + ".\n");

	return 1;
    }

    by = find_player(argv[0]);
    on = find_player(argv[1]);
    if (!on)
    {
	write("No such player: " + capitalize(argv[0]) + ".\n");
	return 1;
    }
    if (!by)
    {
	write("No such player: " + capitalize(argv[1]) + ".\n");
	return 1;
    }
    if (snoop(by, on))
    {
	tell_object(by, "You are now snooping " + capitalize(argv[1]) + ".\n");
	write("Ok.\n");
    }
    else
	write("Failed to set snoop on " + capitalize(argv[1]) + " by " +
	    capitalize(argv[0]) + ".\n");

    return 1;
}

/* **************************************************************************
 * tellall - tell something to everyone
 */
int
tellall(string str)
{
    int i;
    string name, pron;
    object *list;

    if (!strlen(str))
    {
	notify_fail("Tellall what?\n");
	return 0;
    }

    name = capitalize((string)this_player()->query_real_name());

    list = users();
    for (i = 0 ; i < sizeof(list) ; i++)
    {
	if (this_player() != list[i] && objectp(list[i]))
	{
	    if (!(((int)list[i]->query_prop(WIZARD_I_BUSY_LEVEL)) & 2))
	    {
		list[i]->add_prop("_reply_wizard", 
				  this_player()->query_real_name());

		if (list[i]->query_wiz_level())
		    tell_object(list[i], 
				capitalize(this_player()->query_real_name()) +
				" tells everyone: " + str + "\n");
		else
		{
		    pron = capitalize(this_player()->query_pronoun());
		    list[i]->catch_msg("An apparition of " + 
				       QNAME(this_player()) +
				       " appears.\n" + pron +
				       " tells everyone: " + str + "\n" +
	
			       pron + " then disappears.\n",
				       this_player());
		}
	    }
	}
    }

    write("You tell everyone: " + str + "\n");
    return 1;
}

/* **************************************************************************
 * toolsoul - affect your tool souls
 */
int
toolsoul(string str)
{
    string *tool_list;

    CHECK_SO_WIZ;

    tool_list = (string *)this_interactive()->query_tool_list();
    if (!strlen(str))
    {
	write(break_string("Current tools: " + implode(tool_list, ", ") + "\n", 76));
	return 1;
    }

    str = FTPATH((string)this_interactive()->query_path(), str);

    if (member_array(str, tool_list) >= 0)
    {
	if (this_interactive()->remove_toolsoul(str))
	    write("Removed.\n");
	else
	    write("Failed.\n");
	return 1;
    }

    if (this_interactive()->add_toolsoul(str))
	write("Added.\n");
    else
	write("Failed.\n");
    return 1;
}

/* **************************************************************************
 * trans - teleport someone somewhere
 */
int
trans(string str)
{
    object          ob;
    string *args;
    int reverse;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	notify_fail("Trans who?\n");
	return 0;
    }
    
    args = explode(str, " ");

    if (args[0] == "-r")
    {
	if (sizeof(args) == 1)
	{
	    notify_fail("Trans who back?\n");
	    return 0;
	}

	args = args[1..sizeof(args)];
	reverse = 1;
    }

    ob = find_living(args[0]);
    if (!ob)
    {
	notify_fail("No such living thing.\n");
	return 0;
    }


    if (!reverse)
    {
	tell_object(ob, "You feel yourself magically transfered.\n");
	ob->add_prop("_player_transed_from", MASTER_OB(environment(ob)));
	if (sizeof(args) > 1)
	    ob->move_living("X",FTPATH((string)this_interactive()->query_path(), 
				    args[1]));
	else
	    ob->move_living("X", environment(this_interactive()));
    }
    else
    {
	if (ob->query_prop("_player_transed_from"))
	{
	    tell_object(ob, "You feel yourself magically transfered.\n");
	    ob->move_living("X", ob->query_prop("_player_transed_from"));
	    ob->remove_prop("_player_transed_from");
	}
	else
	{
	    notify_fail(capitalize(args[0]) + " cannot be transed back.\n");
	    return 0;
	}
    }

    return 1;
}

/* **************************************************************************
 * trust - trust an object.
 */
int
trust_ob(string str)
{
    object ob;

    CHECK_SO_WIZ;

    if (!str) 
    {
	notify_fail("Trust what object?\n");
	return 0;
    }

    ob = parse_list(str);

    if (!ob) 
    {
	notify_fail("Object not found: " + str + "\n");
	return 0;
    }

    if (geteuid(ob))
    {
	notify_fail("Object already trusted by: " + geteuid(ob) + "\n");
	return 0;
    }

    /* Install the euid of this player as uid in the object */
    export_uid(ob);
    /* Activate the object */
    ob->set_trusted(1);

    write("Beware! You have just trusted: " + str + ".\n");
    return 1;
}


/* **************************************************************************
 * typolog - read the typolog
 */
int
typolog(string str) 
{ 
    return somelog(str, "typo log", "/typos"); 
}

/* **************************************************************************
 * update - update an object
 */
int
update_ob(string str)
{
    object ob, *obs;
    int kick_master, i, error;

    CHECK_SO_WIZ;

    if (!strlen(str))
    {
	ob = environment(this_player());
	str = MASTER_OB(ob);

	if (!ob)
	{
	    notify_fail("Update what object?\n");
	    return 0;
	}

	obs = filter_array(all_inventory(ob), "is_player", this_object());

	error = 0;
	for (i = 0; i < sizeof(obs); i++)
	{
	    if (obs[i]->query_default_start_location() == str)
	    {
		error = 1;	
		write("Cannot update the start location of "
		    + capitalize(obs[i]->query_real_name()) + ".\n");
	    }
	}

	if (error == 1)
	{
	    notify_fail("Update failed.\n");
	    return 0;
	}

	write("Updating environment.\n");
	say(QCTNAME(this_player()) + " updates the room.\n");

	/* Move all objects out of the room */
	for (i = 0; i<sizeof(obs); i++)
	{
	    obs[i]->move(obs[i]->query_default_start_location());
	}

	ob->remove_object();
	ob = find_object(str);
	if (ob)
	    SECURITY->do_debug("destroy", ob);

	for (i = 0; i < sizeof(obs); i++)
	    obs[i]->move(str);
	return 1;
    }
    else
    {
	str = FTPATH((string)this_interactive()->query_path(), str);
	if (!strlen(str))
	{
	    notify_fail("Invalid file name.\n");
	    return 0;
	}
	ob = find_object(str);
	if (!ob)
	{
	    notify_fail("No such object.\n");
	    return 0;
	}
    }

    if (ob == find_object(SECURITY))
	kick_master = 1;

    if (ob != this_object())
    {
	ob->remove_object();
	ob = find_object(str);
	if (ob)
	    SECURITY->do_debug("destroy", ob);

	/* When updating the master object it must be reloaded at once
	   and from within the GD
	 */
	if (kick_master)
	{
	    write(SECURITY + " was updated and reloaded.\n");
	    get_dir("/*");
	}

	else if (!ob)
	    write(str + " will be reloaded at next reference.\n");
	else
	{
	    notify_fail("Object could not be updated.\n");
	    return 0;
	}
    }
    else
    {
	write(str + " will be reloaded at next reference.\n");
	write("Destructing this object.");
	destruct();
	return 1;
    }
    return 1;
}

/* wildcard updating by Gresolle 1993 */
int dupd(string s)
{
    int i;
    string *dir, *args, path, fpath;
    
    /* catch 'update' without args */

    if (!s)
	return update_ob(s);
    
    /* check if user specifies the -d option */

    args = explode(s, " ");
    if (args[0] != "-d")
	return update_ob(s);

    /* remove '-d' and keep pattern */

    s = args[1];

    /* fix path */

    path = FTPATH(this_interactive()->query_path(), s);

    /* read directory */ 

    dir = get_dir(path);

    /* remove file pattern from end of path to expand path in update loop */

    args = explode(path, "/");
    args = exclude_array(args, sizeof(args) - 1, sizeof(args));
    path = implode(args, "/");
  
    /* move through all files returned by 'dir' */

    for (i = 0; i < sizeof(dir); i++)
    {
	/* get full filepath to objects */

	fpath = FTPATH(path, dir[i]);
	
	/* just update existing objects. skip anything names workroom.c to
	    avoid accidents */

	if (find_object(fpath) && dir[i] != "workroom.c")
        {
	    if (!update_ob(fpath)) /* standard update function */
		return 0;
        }
    }
    return 1;
}

int
is_player(object ob)
{
    if (living(ob) && !(ob->query_npc()))
	return 1;
    return 0;
}

/* **************************************************************************
 * vis - become visible
 */
int
vis()
{
    CHECK_SO_WIZ;

    if (!((int)this_interactive()->query_invis()))
    {
	write("You are not invisible.\n");
	return 1;
    }
    this_interactive()->set_invis(0);
    tell_object(this_interactive(), "You are now visible.\n");
    say(QCNAME(this_interactive()) + " " + (string)this_interactive()->query_mm_in() + "\n");
    return 1;
}

/* **************************************************************************
 * wizlist - wizard score list
 */
int
wiz_score_list(string arg)
{
    CHECK_SO_WIZ;

    if (!(this_player()->query_wiz_level() >= WIZLEV_LORD))
    {
	if (stringp(arg))
	    return this_player()->command("ranking " + arg);
	else
	    return this_player()->command("ranking");
    }
	
    if (strlen(arg))
	SECURITY->do_debug("wizlist", arg);
    else
	SECURITY->do_debug("wizlist", this_interactive()->query_real_name());
    return 1;
}

/*
 * Function name: somelog
 * Description:   Look at error logs
 */
static int
somelog(string str, string logname, string log)
{
    string file, rem;

    if (!strlen(str))
	str = (string)this_interactive()->query_real_name();
    else if (sscanf(str, "-r %s", rem) == 1)
    {
	str = rem;
    }
    else if (str == "-r")
    {
	str = (string)this_interactive()->query_real_name();
	rem = str;
    }
    else
	rem = 0;
    
    file = (string)SECURITY->query_wiz_path(str) + "/log";

    if (!strlen(file))
	return (notify_fail("No " + logname + " for: " + str + ".\n"),0);

    file += log;

    if (file_size(file) < 0)
	return (notify_fail("Can't find: " + file + ".\n"),0);

    if (!rem)
    {
	write(capitalize(logname) + ": " + file + "\n");
	this_interactive()->command("tail " + file);
    }
    else
    {
	write("Removing " + capitalize(logname) + ": " + file + "\n");
	this_interactive()->command("rm " + file);
    }
    return 1;
}
