/*
  /secure/master/fob.c

  Module to /secure/master.c
  Handles all domain, wizard and application administration in the game.
*/

#include "/sys/const.h"
#pragma strict_types

int	dom_count;	/* The next domain number to be added */
mapping	cmd_count,      /* The number of commands carried out in each domain */
        m_domains,	/* The domain mapping */
	m_wizards,	/* The wizard mapping */
	m_applications,	/* The applications mapping */
	m_global_read;	/* The global read mapping */

#define DOMAIN_MAX	9	/* Max number of members, any domain */

#define IS_MALE(x) ((x)->query_gender() == G_MALE)
#define IS_FEMALE(x) ((x)->query_gender() == G_FEMALE)

/*
 * Prototypes
 */
static void do_chwiz_lev(string wname, int nlev, string cmder);
static int remove_wizard_from_domain(string wname, string cmder);
static int remove_application(string dname, string wname);
static void remove_all_applications(string wname);
static void do_chwiz_type(string wname, int otype, int ntype, string cmder);
static void tell_domain(string dname, string wname, string wmess, string dmess);
static string add_wizard_to_domain(string dname, string wname, string cmder);

/**************************************************************************
 * Externally callable functions
 * -----------------------------
 *
 * int query_domain_number(string dname)	- Find the domain number.
 *
 * string query_domain_lord(string dname)	- Return the lord of a
 *						  domain.
 *
 * string *query_domain_members(string dname)	- Return an array containing
 *						  all members of a domain.
 *
 * string set_domain_max(int max)		- Set the maximum number of
 *						  members in a domain.
 *
 * int query_domain_max(string dname)		- Find the maximum number of
 *						  members allowed in a domain.
 *
 * string *query_domain_qexp()			- Get given quest xp.
 *
 * string *query_domain_cexp()			- Get given combat xp.
 *
 * int set_domain_sanction(string who, string sanction, string dname)
 *						- Sanction someone.
 *
 * mapping query_domain_sanction()		- Get the domain sanctions.
 *
 * int set_special_sanction(string who, string sanction, string dname, string path)
 *						- Specially sanction someone.
 *
 * mapping query_special_sanction()		- Get the special sanctions.
 *
 * string *query_domain_list()			- Find all domains.
 *
 * string make_domain(string dname, string wname)
 *						- Create a domain.
 *
 * string remove_domain(string dname)		- Remove a domain.
 *
 * string domain_add_wizard(string dname, string wname)
 *						- Add a wizard to a domain.
 *
 * string expel_wizard_from_domain(string wname)
 *						- Expel a wizard from a domain.
 *
 * string leave_domain(string wname)		- Leave a domain.
 *
 * string apply_to_domain(string dname)		- Apply to a domain.
 *
 * string accept_application(string wname)	- Accept an application.
 *
 * string deny_application(string wname)	- Deny an application.
 *
 * list_applications(string dname)		- List applications.
 *
 * string regret_application(string dname)	- Regret an application.
 *
 * int query_wiz_level(string wname)		- Find the level of a wiz.
 *
 * string query_wiz_chl(string wname)		- Find the level changer.
 *
 * string query_wiz_dom(string wname)		- Find the domain of a wiz.
 *
 * string query_wiz_chd(string wname)		- Find the domain changer.
 *
 * string *query_wiz_list(int type)		- Find all wizards of type.
 *
 * string change_wiz_level(string wname, int nlev)
 *						- Change the level of a wiz.
 *
 * string query_wiz_pretitle(mixed wiz)		- Find the correct pretitle
 *
 * string query_wiz_path(string wname)		- Find the default path.
 *
 * string *query_mage_preload()			- Get the mage preload list.
 *
 * string *query_domain_preload()		- Get the domain preload list.
 *
 * int domain_clear_xp(string dom)		- Clear the bookeeped xp
 *
 * string query_domain_madwand(string dom)	- Give the name of the madwand
 *
 * string draft_madwand(string dname, string wname)
 *						- Draft a madwand
 *
 * string expel_madwand(string dname, string wname)
 *						- Expel a madwand
 *
 * string retire_wiz(string name)
 *
 * string add_global_read(string name, string comment)
 *						- Add someone to the global
 *						  read list
 *
 * string remove_global_read(string name)	- Remove someone from the global
 *						  read list
 *
 * mapping query_global_read()			- Return the global read list.
 */

/**************************************************************************
 *
 * The 'm_domains' mapping holds the domain name as index and an array with
 * the following items as value:
 *
 * [0]	- The domain number. It is stored as tag in the player along with
 *        the quest bit set by the domain objects.
 *
 * [1]	- The name of the domain lord.
 *
 * [2]	- An array holding the names of all members, including the lord.
 *
 * [3]  - The max number of members the domain wants to accept.
 *
 * [4]  - The number of quest xp given to mortals
 *
 * [5]  - The number of combat xp given to mortals
 *
 * [6]  - The name of the 'madwand' of the domain, If one wiz in the domain
 *        is a 'madwand', i.e. a mortal that has been instawizzed by the
 *        lord of the domain.
 *
 * [7]	- The domain sanctions.
 *
 * [8]	- The domain special sanctions.
 *
 **************************************************************************
 *
 * The 'm_wizards' mapping holds the wizard name as index and an array with
 * the following items as value:
 *
 * [0]	- The wizard level.
 *
 * [1]	- The name of the wizard who last changed the level.
 *
 * [2]	- The domain the wizard is a member of.
 *
 * [3]	- The name of the wizard who last changed the domain.
 *
 **************************************************************************
 *
 * The 'm_applications' mapping holds the following information:
 *
 * Domain name : Array of applying apprentices.
 *
 */

#define FOB_DOM_NUM	0
#define FOB_DOM_LORD	1
#define FOB_DOM_MEM	2
#define FOB_DOM_MWN	3
#define FOB_DOM_QXP	4
#define FOB_DOM_CXP	5
#define FOB_DOM_MADW	6
#define FOB_DOM_SANC	7
#define FOB_DOM_SPECS	8

#define FOB_WIZ_LEV	0
#define FOB_WIZ_CHL	1
#define FOB_WIZ_DOM	2
#define FOB_WIZ_CHD	3

/* Prototypes prototypes prototypes */
string get_root_uid();

/*
 * This function is called from master.c when the KEEPERAVE file can't
 * be found. The defined values can be found in config.h
 */
int
load_fob_defaults()
{
    if (previous_object() != this_object())
	return 0;

    m_wizards = DEFAULT_WIZARDS;
    m_domains = DEFAULT_DOMAINS;
    dom_count = m_sizeof(m_domains);
    m_applications = ([]);
    return 1;
}

static string
getwho()
{
    string g1, g2;

    g1 = geteuid(this_interactive());
    g2 = geteuid(previous_object());

    if (g1 != g2)
	return "";
    else
	return g1;
}

/************************************************************************
 *
 * The domain administration code.
 *
 */

/*
 * Function name: query_domain_number
 * Description:   Find the number of a domain
 * Arguments:     dname - Name of the domain
 * Returns:	  The number if found, -1 otherwise.
 */
int
query_domain_number(string dname)
{
    dname = capitalize(dname);

    if (sizeof(m_domains[dname]))
	return m_domains[dname][FOB_DOM_NUM];
    else
	return -1;
}

/*
 * Function name: query_domain_lord
 * Description:   Find the lord of a domain
 * Arguments:     dname - Name of the domain
 * Returns:	  The domain lord if found, "" otherwise.
 */
string
query_domain_lord(string dname)
{
    dname = capitalize(dname);

    if (sizeof(m_domains[dname]))
	return m_domains[dname][FOB_DOM_LORD];
    else
	return "";
}

/*
 * Function name: query_domain_members
 * Description:   Find and return the member array of a given domain.
 * Arguments:     dname - the domain
 * Returns:	  The array if found, ({}) otherwise.
 */
string *
query_domain_members(string dname)
{
    dname = capitalize(dname);

    if (sizeof(m_domains[dname]))
	return m_domains[dname][FOB_DOM_MEM];
    else
	return ({});
}

/*
 * Function name: query_domain_list
 * Description:   Return a list of all domains.
 */
string *
query_domain_list()
{
    return m_indexes(m_domains);
}

/*
 * Function name: set_domain_max
 * Description:   Set maximum number of wizards wanted. 
 * Arguments:     max - the wanted maximum
 * 		  dname - the domain
 * Returns:	  result string
 */
string
set_domain_max(int max, string x_dname)
{
    string wname, dname;

    wname = getwho();

    if (strlen(wname) == 0)
	return "You are not allowed to do this.";

    if (strlen(x_dname) != 0)
	x_dname = capitalize(x_dname);

    switch(WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]))
    {
    case WIZ_KEEPER:
    case WIZ_ARCH:
	if (strlen(x_dname) == 0)
	    return "Set the max on which domain?";
	dname = x_dname;

	break;

    case WIZ_LORD:
	if (max > DOMAIN_MAX)
	    return "You can't set a maximum of " + max + " wizards.";

	if (strlen(x_dname) == 0)
	    dname = m_wizards[wname][FOB_WIZ_DOM];
	else
	    dname = x_dname;
	
	if (dname != x_dname)
	    return "You are not the Lord of that domain.\n";

	if (dname == "")
	    return "I can't find your domain.";

	break;

    default:
	return "You are not Lord of any domain.";
    }

    if (!sizeof(m_domains[dname]))
	return "I can't find your domain info.";
    else
    {
	if (sizeof(m_domains[dname][FOB_DOM_MEM]) > max)
	    return "You already have more members than that.";
	else
	    m_domains[dname][FOB_DOM_MWN] = max;
    }

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);
	
    return "Ok.";
}

/*
 * Function name: query_domain_max
 * Description:   Find and return the maximum number of wizards wanted. 
 * Arguments:     dname - the domain
 * Returns:	  The max number if found, -1 otherwise..
 */
int
query_domain_max(string dname)
{
    dname = capitalize(dname);

    if (sizeof(m_domains[dname]))
	return m_domains[dname][FOB_DOM_MWN];
    else
	return -1;
}

/*
 * Function name: domain_num
 * Description:   Find the domain number.
 * Arguments:     dname - the domain name
 * Returns:	  The number if found, -1 otherwise
 */
static int
domain_num(string dname)
{
    dname = capitalize(dname);

    if (sizeof(m_domains[dname]))
	return m_domains[dname][FOB_DOM_NUM];
    else
	return -1;
}

/*
 * Function name: make_domain
 * Description:   Create a new domain. Return an error string if it fails.
 * Arguments:     dname - the name of the domain
 *		  wname - the name of the lord
 * Returns:	  Result string.
 */
string
make_domain(string dname, string wname)
{
    string cmder, ret;
    object wobj;

    if (strlen(dname) == 0)
	return "Create which domain?";
    if (strlen(wname) == 0)
	return "Make which wizard lord of the domain?";

    dname = capitalize(dname);
    wname = lower_case(wname);

    cmder = getwho();
    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    /*
     * Lots of things can go wrong. Check carefully.
     */
    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "You are not allowed to do this.";

    if (sizeof(m_domains[dname]) != 0)
	return dname + " already exists.";

    if (sizeof(m_wizards[wname]) == 0)
	return capitalize(wname) + " is mortal!";

    if (m_wizards[wname][FOB_WIZ_DOM] != "")
	return capitalize(wname) + " is not an apprentice.";

    /*
     * All right, do it.
     */
    set_auth(this_object(), "root:root");

    if (file_size("/d/" + dname) != -2) 
    {
	/*
	 * Check for old discarded instances of the domain.
	 */
	if (file_size(DISCARD_DOMAIN_DIR + "/" + dname) == -2)
	{
	    rename(DISCARD_DOMAIN_DIR + "/" + dname,  "/d/" + dname);
	    ret = "Revived the old discarded " + dname + ".";
	}
	else
	{
	    if (!mkdir("/d/" + dname))
		return "Can not create the domain subdir!";
	    else
		ret = "Created the new domain " + dname + ".";
	}
    }

    m_domains += ([ dname : ({ dom_count++, "", 
			       ({}), DOMAIN_MAX, 0, 0, "", ([]), ([]) }) ]);

    do_chwiz_lev(wname, WIZMID(WIZLEV_LORD), cmder);
    add_wizard_to_domain(dname, wname, cmder);

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    if (wobj = find_player(wname))
	tell_object(wobj,"You are now Lord over the domain " + dname +".\n");

    return ret;
}

/*
 * Function name: remove_domain
 * Description:   Remove a domain, demote the wizards, move the code
 * Arguments:     dname - the domain
 * Returns:	  result string.
 */
string
remove_domain(string dname)
{
    int i;
    string cmder, *mems, ret;

    cmder = getwho();

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "You are not allowed to do this.";

    if (strlen(dname) == 0)
	return "Remove which domain?";

    dname = capitalize(dname);

    if (sizeof(m_domains[dname]) == 0)
	return "There is no such domain.";

    mems = m_domains[dname][FOB_DOM_MEM];
    for (i = 0 ; i < sizeof(mems) ; i++)
    {
	switch(WIZ_TYPE(m_wizards[mems[i]][FOB_WIZ_LEV]))
	{
	case WIZ_MAGE:
	case WIZ_ARCH:
	case WIZ_KEEPER:
	    remove_wizard_from_domain(mems[i], cmder);
	    if ((ret = add_wizard_to_domain(WIZARD_DOMAIN, mems[i], 
					    cmder)) != "Ok.")
		write(break_string("Trouble in adding " + capitalize(mems[i]) 
			+ " to " + WIZARD_DOMAIN + ": " + ret + "\n", 65));
	    break;

	default:
	    do_chwiz_lev(mems[i], WIZLEV_APPRENTICE, cmder);
	    break;
	}
    }

    m_domains = m_delete(m_domains, dname);

    rename("/d/" + dname, DISCARD_DOMAIN_DIR + "/" + dname);

    return "You have just obliterated " + dname + ". Congratulations.";
}

/*
 * Function name: draft_madwand
 * Description:   Add a mortal as an instawiz madwand to a domain.
 *		  
 * Arguments:     dname - the domain
 *		  wname - the wizard to be
 *		  cmder - who does it
 * Returns:	  result string.
 */
public string
draft_madwand(string dname, string wname)
{
    string cmder, emsg;
    int clev;
    mixed darr;
    object mpl;

    cmder = getwho();
    dname = capitalize(dname);
    wname = lower_case(wname);

    if (sizeof(m_wizards[cmder]) == 0)
	return "You are not allowed to do this.\n";

    clev = m_wizards[cmder][FOB_WIZ_LEV];

    if (WIZ_TYPE(clev) < WIZ_LORD)
	return "Only domainlords can draft Madwands.";

    if (WIZ_TYPE(clev) == WIZ_LORD &&
	m_wizards[cmder][FOB_WIZ_DOM] != dname)
	return "You can only draft a Madwand to your own domain.";

    if (sizeof(m_wizards[wname]) != 0)
	return capitalize(wname) +
	    " is a wizard. Only mortals can be made into Madwands.";
    
    if (strlen(m_domains[dname][FOB_DOM_MADW]))
	return dname + " already has a Madwand: " + 
	    capitalize(m_domains[dname][FOB_DOM_MADW]) + ".\n";

    if (sizeof(m_domains[dname][FOB_DOM_MEM]) >= m_domains[dname][FOB_DOM_MWN])
	return "The domain `" + dname + "' is already full.";

    mpl = find_player(wname);
    if (!mpl)
	return capitalize(wname) +
	    " must be logged in when promoted to Madwand."; 

    /*
	First save the mortals playerfile in a name.madwand
    */
    set_auth(this_object(), "root:root");
    rename(PLAYER_FILE(wname) + ".o", PLAYER_FILE(wname) + ".madwand");

    /*
	Try to save a copy of the file so that the player still has his .o file
    */
    if (mpl)
	mpl->save_me(1);

    /*
	Then make the mortal into an apprentice
    */
    do_chwiz_lev(wname, WIZLEV_APPRENTICE, cmder);
    
    /*
	Then draft the apprentice into the domain
    */
    emsg = add_wizard_to_domain(dname, wname, cmder);
    if (emsg != "Ok.")
	return emsg;

    /*
	Then mark the wizard as a madwand
    */	    
    m_domains[dname][FOB_DOM_MADW] = wname;
    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);
    log_file("MADWAND", ctime(time()) + " " + capitalize(wname) +
	     " drafted to " + dname + " by: " + capitalize(cmder) +"\n", -1); 
    return "Ok.";
}

/*
 * Function name: expel_madwand
 * Description:   Remove an instawiz madwand from a domain.
 *		  
 * Arguments:     dname - the domain
 *		  wname - the mortal to be
 * Returns:	  result string.
 */
public string
expel_madwand(string dname, string wname)
{
    string cmder, emsg;
    int clev;
    mixed darr;

    cmder = getwho();
    dname = capitalize(dname);
    wname = lower_case(wname);

    if (sizeof(m_wizards[cmder]) == 0)
	return "You are not allowed to do this.\n";

    clev = m_wizards[cmder][FOB_WIZ_LEV];

    if (WIZ_TYPE(clev) < WIZ_LORD)
	return "Only domainlords can expel madwands.";

    if (WIZ_TYPE(clev) == WIZ_LORD &&
	m_wizards[cmder][FOB_WIZ_DOM] != dname)
	return "You can only expel a madwand from your own domain.";
    
    if (m_domains[dname][FOB_DOM_MADW] != wname)
	return capitalize(wname) + " is not the madwand of " + dname + ".\n";

    /*
	First remove the madwand from the domain -> apprentice
    */
    remove_wizard_from_domain(wname, cmder);
    
    /*
	Mark that the domain has no madwand
    */
    m_domains[dname][FOB_DOM_MADW] = "";

    /*
	Then make the apprentice into a mortal.
	This will also remove him from the game if logged in.
    */
    do_chwiz_lev(wname, WIZLEV_MORTAL, cmder);

    /*
	Last try to restore the mortals playerfile from name.madwand
    */
    set_auth(this_object(), "root:root");
    if (file_size(PLAYER_FILE(wname) + ".madwand") > 0)
	rename(PLAYER_FILE(wname) + ".madwand", PLAYER_FILE(wname) + ".o");

    save_object(SAVEFILE);
    log_file("MADWAND", ctime(time()) + " " + capitalize(wname) +
	     " expelled from " + dname + " by: " + capitalize(cmder) +"\n", -1); 
    return "Ok.";
}


/*
 * Function name: add_wizard_to_domain
 * Description:   Add a wizards to a domain.
 * Arguments:     dname - the domain
 *		  wname - the wizard
 *		  cmder - who does it
 * Returns:	  result string.
 */
static string
add_wizard_to_domain(string dname, string wname, string cmder)
{
    string old_domain, wdir, ddir;
    int i;

    dname = capitalize(dname);
    wname = lower_case(wname);

    if (sizeof(m_domains[dname]) == 0)
	return "No such domain: " + dname + ".";

    if (sizeof(m_wizards[wname]) == 0)
	return "No such wizard: " + wname + ".";

    if (sizeof(m_domains[dname][FOB_DOM_MEM]) >= m_domains[dname][FOB_DOM_MWN])
	return "The domain `" + dname + "' is already full.";

    old_domain = m_wizards[wname][FOB_WIZ_DOM];

    switch (WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]))
    {
    case WIZ_MAGE:
    case WIZ_ARCH:
    case WIZ_KEEPER:
	if (old_domain == WIZARD_DOMAIN)
	{
	    remove_wizard_from_domain(wname, cmder);
	    break;
	}
	break;

    case WIZ_LORD:
	m_domains[dname][FOB_DOM_LORD] = wname;
	break;

    case WIZ_PILGRIM:
	return "Pilgrims can't be domain members.\n";
	break;
    default:
	if (old_domain != "")
	    return capitalize(wname) + " is already a member of the domain " + old_domain + ".";
	break;
    }

    m_wizards[wname][FOB_WIZ_DOM] = dname;
    m_wizards[wname][FOB_WIZ_CHD] = cmder;
    if (m_wizards[wname][FOB_WIZ_LEV] == WIZLEV_APPRENTICE)
	do_chwiz_lev(wname, WIZMID(WIZLEV_NORMAL), cmder);

    remove_all_applications(wname);

    m_domains[dname][FOB_DOM_MEM] += ({ wname });

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    wdir = "/d/" + WIZARD_DOMAIN + "/" + wname;
    ddir = "/d/" + dname + "/" + wname;

    if (wdir != ddir)
    {
	/*
	 * Here is one that is joining a domain. Let us just move his
         * old wizard directory if there is one.
         */
        if ((file_size(wdir) == -2) && (file_size(ddir) == -1))
	    rename(wdir, ddir);

        /*
         * This one is also joining a domain, but apperently he was a
         * member before because there is already a directory with his
         * name. First we move private. Then we put the
         * rest of his stuff in a directory called old.
         */
	else if ((file_size(wdir) == -2) && (file_size(ddir) == -2))
	{
	    if ((file_size(wdir + "/private") == -2) &&
	        (file_size(ddir + "/private") == -1))
    	        rename(wdir + "/private", ddir);

	    i = 0;
	    if (file_size(ddir + "/old") == -1)
                rename(wdir, ddir + "/old");
	    else
	    {
		while (file_size(ddir + "/old" + (++i)) != -1);
		rename(wdir, ddir + "/old" + i);
	    }
	}
    }
    else if (file_size(ddir) != -2) 
	mkdir(ddir);

    tell_domain(dname, wname, "You are now a member of the domain " + 
		dname +".\n",
		capitalize(wname) + " is now a member of the domain " + 
		dname + ".\n");

    return "Ok.";
}

/*
 * Function name: domain_add_wizard
 * Description:   Add a wizard to a domain.
 * Arguments:     dname - the domain
 *		  wname - the wizard
 * Returns:	  Result string
 */
string
domain_add_wizard(string dname, string wname)
{
    string cmder;

    if (strlen(dname) == 0)
	return "Add to which domain?";
    if (strlen(wname) == 0)
	return "Add which wizard?";

    dname = capitalize(dname);
    wname = lower_case(wname);

    cmder = getwho();
    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "You may not perform this operation.";

    /*
	Drafting a madwand
    */
    if (sizeof(m_wizards[wname]) == 0)
    {
	return draft_madwand(dname, wname);
    }

    if ((m_wizards[wname][FOB_WIZ_LEV] > WIZLEV_LORD) &&
	(m_wizards[wname][FOB_WIZ_DOM] == WIZARD_DOMAIN))
	remove_wizard_from_domain(wname, cmder);

    return add_wizard_to_domain(dname, wname, cmder);
}

/*
 * Function name: expel_wizard_from_domain
 * Description:   Expel a wizard from a domain.
 * Arguments:     wname - the wizard
 * Returns:	  Result string.
 */
string
expel_wizard_from_domain(string wname)
{
    int             type_curr, type_cmd;
    string	    cmder, dom, ret;

    if (strlen(wname) == 0)
	return "Expel which wizard?";

    wname = lower_case(wname);

    if (sizeof(m_wizards[wname]) == 0)
	return capitalize(wname) + " is mortal!";

    dom = m_wizards[wname][FOB_WIZ_DOM];

    /*
	Are we expelling the domains madwand?
    */
    if (m_domains[dom][FOB_DOM_MADW] == wname)
	return expel_madwand(dom, wname);

    cmder = getwho();
    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    type_cmd = WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]);
    type_curr = WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]);

    if (!strlen(dom) || 
	(dom != m_wizards[cmder][FOB_WIZ_DOM] && type_cmd < WIZ_ARCH))
	return capitalize(wname) + " is not a member of your domain.";
    

    switch(type_curr)
    {
    case WIZ_MAGE:
    case WIZ_ARCH:
    case WIZ_KEEPER:
        remove_wizard_from_domain(wname, cmder);
	if ((ret = add_wizard_to_domain(WIZARD_DOMAIN, wname, cmder)) != "Ok.")
	    write(break_string("Trouble in adding " + capitalize(wname) + 
		    " to " + WIZARD_DOMAIN + ": " + ret + "\n", 65));
	break;

    default:
	do_chwiz_lev(wname, WIZLEV_APPRENTICE, cmder);
	break;
    }
    return "Ok";
}

/*
 * Function name: remove_wizard_from_domain
 * Description:   Remove a wizard from a domain
 * Arguments:     wname - the wizard
 *		  cmder - who does it
 * Returns:	  Success status.
 */
static int
remove_wizard_from_domain(string wname, string cmder)
{
    string dname, wdir, ddir, *mem;
    object wiz;
    int i;

    dname = m_wizards[wname][FOB_WIZ_DOM];

    if (dname == "")
	return 0;

    m_wizards[wname][FOB_WIZ_DOM] = "";
    m_wizards[wname][FOB_WIZ_CHD] = cmder;
    
    mem = m_domains[dname][FOB_DOM_MEM];

    mem -= ({ wname });

    m_domains[dname][FOB_DOM_MEM] = mem;

    if (wiz = find_player(wname))
	tell_object(wiz, "You are removed from the domain " + dname + ".\n");

    if (m_domains[dname][FOB_DOM_LORD] != wname)
    {
	for (i = 0 ; i < sizeof(mem) ; i++)
	{
	    if (wiz = find_player(mem[i]))
		tell_object(wiz, capitalize(wname) + 
			    " is removed from " + dname + ".\n");
	}
    }
    else
    {
	m_domains[dname][FOB_DOM_LORD] = "";
	tell_domain(dname, wname, "You are no longer Lord over " + 
		    dname + ".\n",
		    capitalize(wname) + " is removed as Lord from " + 
		    dname + ".\n");
    }	    

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    /*
     * This one is leaving the domain, so we make a directory for him
     * in WIZARD_DOMAIN. If he has a private dir we move it.
     */
    wdir = "/d/" + WIZARD_DOMAIN + "/" + wname;
    ddir = "/d/" + dname + "/" + wname;

    if (wdir != ddir)
    {
	set_auth(this_object(), "root:root");

	if (file_size(wdir) != -2)
	    mkdir(wdir);

	if (file_size(ddir + "/private") == -2)
	    rename(ddir + "/private", wdir);
    }
    return 1;
}

/*
 * Function name: leave_domain
 * Description:   Someone wants to leave his domain
 * Returns:	  result string
 */
string
leave_domain()
{
    string cmder, ret;

    cmder = getwho();

    if (sizeof(m_wizards[cmder]) == 0)
	return "You are not a wizard.";

    if ((WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) >= WIZ_ARCH ||
	 WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) == WIZ_MAGE) &&
	(m_wizards[cmder][FOB_WIZ_DOM] == WIZARD_DOMAIN))
	return "You can't leave " + WIZARD_DOMAIN + " for good.";

    if (!remove_wizard_from_domain(cmder, cmder))
	return "You are not a member of any domain.";
    
    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) >= WIZ_ARCH ||
	WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) == WIZ_MAGE)
    {
	if ((ret = add_wizard_to_domain(WIZARD_DOMAIN, cmder, cmder)) != "Ok.")
	    write(break_string("Trouble in adding " + capitalize(cmder) + 
		      " to " + WIZARD_DOMAIN + ": " + ret + "\n", 65));
    }
    else
	do_chwiz_lev(cmder, WIZLEV_APPRENTICE, cmder);

    return "Ok.";
}

/*
 * Function name: tell_domain
 * Description:   Tell something to all in the domain.
 * Arguments:     dname - the domain.
 *		  wname - special message to this wiz
 *		  wmess - wizard message
 *		  dmess - domain message
 */
static void 
tell_domain(string dname, string wname, string wmess, string dmess)
{
    object wiz, dom;
    string *wlist;
    int i;

    wlist = m_domains[dname][FOB_DOM_MEM];
    wiz = find_player(wname);
    if (wiz)
	tell_object(wiz, wmess);
    for (i = 0 ; i < sizeof(wlist) ; i++)
    {
	dom = find_player(wlist[i]);
	if (dom && (!wiz || wiz != dom))
	    tell_object(dom, dmess);
    }
}

/************************************************************************
 *
 * The applications administration code.
 *
 */

/*
 * Function name: apply_to_domain
 * Description:   Apply to a domain
 * Arguments:     dname - domain the application goes to.
 * Returns:	  result string
 */
string
apply_to_domain(string dname)
{
    string wname, *appl;
    object wiz;

    if (strlen(dname) == 0)
	return "Apply to what domain?";

    dname = capitalize(dname);
    wname = getwho();
    if (strlen(wname) == 0)
	return "You are not allowed to do this.";
    
    if (sizeof(m_wizards[wname]) == 0)
	return "You are not a wizard!";

    switch(WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]))
    {
    case WIZ_MAGE:
    case WIZ_ARCH:
    case WIZ_KEEPER:
	if (m_wizards[wname][FOB_WIZ_DOM] != "" && 
	    m_wizards[wname][FOB_WIZ_DOM] != WIZARD_DOMAIN)
	    return "You are already a member of the domain: " + 
		m_wizards[wname][FOB_WIZ_DOM] + ".";
	break;

    case WIZ_PILGRIM:
	return "Pilgrims are doomed to wander the world, they may not settle down.";
	break;

    default:
	if (m_wizards[wname][FOB_WIZ_DOM] != "")
	    return "You are already a member of the domain: " + 
		m_wizards[wname][FOB_WIZ_DOM] + ".";
	break;
    }

    if (sizeof(m_domains[dname]) == 0)
	return "There is no domain with that name.";
    
    if (sizeof(m_domains[dname][FOB_DOM_MEM]) >= m_domains[dname][FOB_DOM_MWN])
	return "That domain is already full.";
    
    if (m_applications != 0)
      {
	if (sizeof(m_applications[dname]))
	  appl = m_applications[dname];
	else
	  appl = ({});
      }
    else 
      {
	m_applications = ([]);
	appl = ({});
      }
    

    if (member_array(wname, appl) >= 0)
	return "You have already filed an application to that domain!";

    appl += ({ wname });

    m_applications += ([ dname:appl ]);

    if (wiz = find_player(m_domains[dname][FOB_DOM_LORD]))
	tell_object(wiz, capitalize(wname) + 
		    " just applied for membership in " + dname + ".\n");

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    return "Ok.";
}

/*
 * Function name: accept_application
 * Description:   A lord accepts an application
 * Arguments:     wname - the wizard to accept.
 * Returns:	  result string
 */
string
accept_application(string wname)
{
    string dname, cmder, *appl;

    if (strlen(wname) == 0)
	return "Accept which wizard?";

    cmder = getwho();
    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) != WIZ_LORD)
	return "You are not a lord of any domain!";

    wname = lower_case(wname);
    dname = m_wizards[cmder][FOB_WIZ_DOM];

    appl = m_applications[dname];
    
    if (sizeof(appl) == 0)
	return "No one has applied to your domain.";

    if (member_array(wname, appl) < 0)
	return capitalize(wname) + " haven't asked to join your domain.";

    remove_all_applications(wname);
    return add_wizard_to_domain(dname, wname, cmder);
}

/*
 * Function name: deny_application
 * Description:   The lord denies an application.
 * Arguments:     wname - the wizard
 * Returns:	  result string
 */
string
deny_application(string wname)
{
    string dname, cmder, *appl;
    object wiz;
    
    cmder = getwho();
    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) != WIZ_LORD)
	return "You are not a lord of any domain!";

    wname = lower_case(wname);
    dname = m_wizards[cmder][FOB_WIZ_DOM];

    appl = m_applications[dname];

    if (sizeof(appl) == 0)
	return "No one has applied to your domain.";
    
    if (member_array(wname, appl) < 0)
	return capitalize(wname) + " haven't asked to join your domain.";

    remove_application(dname, wname);

    if (wiz = find_player(wname))
	tell_object(wiz, "Your application to the domain " + dname +
			" was denied.\n");

    return "Ok.";
}

/*
 * Function name: regret_application
 * Description:   An apprentice regrets an application to a domain.
 * Arguments:     dname - the domain
 * Returns:	  result string
 */
string
regret_application(string dname)
{
    string wname, ans;

    if (strlen(dname) == 0)
	return "Regret which application?";

    dname = capitalize(dname);

    wname = getwho();
    if (strlen(wname) == 0)
	return "You are not allowed to do this.";

    switch(remove_application(dname, wname))
    {
    case -1: /* Fall into next case */
    case 0:
	ans = "You never asked to join this domain.";
	break;

    case 1:
	ans = "Ok.";
	set_auth(this_object(), "root:root");
	save_object(SAVEFILE);
	break;
	
    default:
	ans = "Strange switch in regret_application. Please contact an archwizard.";
	break;
    }

    return ans;
}

/*
 * Function name: remove_application
 * Description:   Remove an application from the application list.
 * Arguments:     dname - the domain
 *		  wname - the wizard
 * Returns:	  result state
 */
static int
remove_application(string dname, string wname)
{
    string *appl;

    if (!sizeof((appl = m_applications[dname])))
	return -1;

    if (member_array(wname, appl) < 0)
	return 0;

    appl -= ({ wname });
    
    if (sizeof(appl) == 0)
	m_applications = m_delete(m_applications, dname);
    else
	m_applications[dname] = appl;

    return 1;
}

/*
 * Function name: rm_application
 * Description:   This way arches and keepers can remove an application by
 *		  force, perhaps when a domain has been removed and still
 *		  have someone applying to it?
 * Arguments:	  wname - The wizards name
 *		  dname - The name of the domain
 * Returns:	  The string to write
 */
string
rm_application(string wname, string dname)
{
    object wiz;
    string cmder, *appl;

    cmder = getwho();
    if (!strlen(cmder))
	return "You are not allowed to do this.";

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "Only Arches may remove applications this way.";

    wname = lower_case(wname);
    dname = capitalize(dname);
    appl = m_applications[dname];

    if (!sizeof(appl))
	return "Noone has applied to that domain.";

    if (member_array(wname, appl) < 0)
	return capitalize(wname) + " has not applied to that domain.\n";

    remove_application(dname, wname);

    if (wiz = find_player(wname))
	tell_object(wiz, "Your application to the domain " + dname + 
	    " was removed by " + capitalize(cmder) + ".\n");

    return "Ok.";
}

/*
 * Function name: list_applications
 * Description:   List applications.
 * Arguments:     dname - the domain to list.
 */
void
list_applications(string dname)
{
    string cmder;
    string *arr;
    mapping slaskmap;
    int i, cmdtype;

    dname = capitalize(dname);
    cmder = getwho();
    if (strlen(cmder) == 0)
    {
	write("You are not allowed to do this.\n");
	return;
    }
    cmdtype = WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]);

    switch (cmdtype)
    {
    case WIZ_MAGE:
	/*
	 * Mages who are not members of a specific domain 
	 * might have applications pending. Fall into next case.
	 */
	if (m_wizards[cmder][FOB_WIZ_DOM] != WIZARD_DOMAIN)
	    break;

    case WIZ_APPRENTICE:
	slaskmap = filter(m_applications, "find_appl", this_object(), cmder);
	arr = m_indexes(slaskmap);
	if (sizeof(arr) == 0)
	    write("You have no pending applications.\n");
	else
	    write(break_string("You have applications pending to: " + 
			       implode(arr, ", ") + ".\n", 65));
	break;
	
    case WIZ_LORD:
	arr = m_applications[m_wizards[cmder][FOB_WIZ_DOM]];
	if (sizeof(arr) == 0)
	    write("There are no applications pending to your domain.\n");
	else
	{
	    arr = map(arr, "cap_string", this_object());
	    write(break_string("The following wizards have applied to your domain: " + implode(arr, ", ") + ".\n", 65));
	}
	break;

    case WIZ_ARCH:
    case WIZ_KEEPER:
	if (strlen(dname) == 0)
	{
	    arr = m_indexes(m_applications);
	    if (sizeof(arr) == 0)
		write("There are no pending applications.\n");
	    else
	    {
		for (i = 0 ; i < sizeof(arr) ; i++)
		    write(break_string(arr[i] + ": " + implode(map(
		    m_applications[arr[i]], "cap_string", this_object()), 
		    ", ") + ".\n", 65));
	    }
	}
	else
	{
	    if (sizeof(m_applications[dname]) == 0)
		write("There are no pending applications to that domain.\n");
	    else
		write(break_string(dname + ": " + implode(map(
		m_applications[dname], "cap_string", this_object()), 
		", ") + ".\n", 65));
	}
	break;
	
    default:
	write("This command is of no importance to you.\n");
	break;
    }
}

int
find_appl(mixed *val, string cmder)
{
    if (member_array(cmder, val) < 0)
	return 0;
    else
	return 1;
}

string
cap_string(string wiz)
{
    return capitalize(wiz);
}

/*
 * Function name: remove_all_applications
 * Description:   Remove all application by a wizard from the application list.
 * Arguments:     wname - the wizard
 */
static void
remove_all_applications(string wname)
{
    string *domains, *wizzes;
    int i;

    domains = m_indexes(m_applications);

    for (i = 0 ; i < sizeof(domains) ; i++)
    {
	wizzes = m_applications[domains[i]];
	if (member_array(wname, wizzes) != -1)
	    if (sizeof(wizzes) == 1)
		m_applications = m_delete(m_applications, domains[i]);
	    else
		m_applications[domains[i]] = wizzes - ({ wname });
    }
}

static int lognum = 0;
/*
 * Function name: bookkeep_exp
 * Description:   Note the xp domains give to the mortals and what kind.
 *                This is not saved each time to KEEPERSAVE, we trust it
 *		  to be saved at one time or other. Exact bookkeeping is not
 *		  absolutely crucial and it would take a lot of time.
 * Arguments:     euid: The euid of the object giving xp
 *		  q_xp: Amount of quest xp. Can only be 0 or positive.
 *		  c_xp: Amount of combat xp. Can be both pos. and neg.
 */
public void
bookkeep_exp(string euid, int q_xp, int c_xp)
{
    string dname, nl;
    mixed darr;
    object pobj;

    pobj = previous_object();

    /* It should be a mortal player
    */
    if ((geteuid(pobj) != this_object()->get_bb_uid()) ||
	(!interactive(pobj)))
	return;

    /* It should not be a 'jr' wizhelper character
    */
    if (sscanf(pobj->query_real_name(), "%sjr",dname) == 1)
	return;

    if (sizeof(m_wizards[euid]))
	dname = m_wizards[euid][FOB_WIZ_DOM];
    else
	dname = euid;

    if (!sizeof(m_domains[dname]))
	return; /* Should we log this, apprentice or nonexistant euid ? */

    darr = m_domains[dname] + ({}); /* Give us a copy */

    darr[FOB_DOM_QXP] += q_xp;
    darr[FOB_DOM_CXP] += c_xp;
#ifdef LOG_BOOKKEEP
    if (q_xp > LOG_BOOKKEEP_LIMIT_Q ||
	c_xp > LOG_BOOKKEEP_LIMIT_C ||
	-q_xp > LOG_BOOKKEEP_LIMIT_Q ||
	-c_xp > LOG_BOOKKEEP_LIMIT_C)
    {
	lognum++;
	if (lognum % 5 == 0)
	    nl = "\n";
	else
	    nl = "";
	set_auth(this_object(), "root:root");
	log_file(LOG_BOOKKEEP, 
		 sprintf("%s%s (%s)\n---- %-12s(%10s) -> %-10s  q(%6d) c(%6d)\n",
			 nl, ctime(time()), file_name(environment(pobj)),
			 dname, euid, pobj->query_real_name(),
			 q_xp, c_xp), 1000000);
    }
#endif
    m_domains[dname] = darr;
}

/*
 * Function name: decay_exp
 * Description:   Let the accumulated xp / domain decay over time. This
 *                is called from check_memory which is called at regular
 *		  intervalls. check_memory also saves to KEEPERSAVE.
 *		  
 */
static void
decay_exp()
{
    m_domains = map(m_domains, "do_decay_exp", this_object());
    cmd_count = map(cmd_count, "do_decay_cmd", this_object());
}

mixed *
do_decay_exp(mixed *darr)
{
    int decay;
#ifdef DECAY_XP
    decay = DECAY_XP;
#else
    decay = 100;
#endif
    if (!decay)
	return darr;

    if (darr[FOB_DOM_QXP] >= decay || (-darr[FOB_DOM_QXP]) >= decay)
	darr[FOB_DOM_QXP] -= darr[FOB_DOM_QXP] / decay;
    else if (darr[FOB_DOM_QXP] < 0)
	darr[FOB_DOM_QXP] += 1;
    else if (darr[FOB_DOM_QXP] > 0)
	darr[FOB_DOM_QXP] -= 1;

    if (darr[FOB_DOM_CXP] >= decay || (-darr[FOB_DOM_CXP]) >= decay)
	darr[FOB_DOM_CXP] -= darr[FOB_DOM_CXP] / decay;
    else if (darr[FOB_DOM_CXP] < 0)
	darr[FOB_DOM_CXP] += 1;
    else if (darr[FOB_DOM_CXP] > 0)
	darr[FOB_DOM_CXP] -= 1;

    return darr;
}
    
int
do_decay_cmd(int count)
{
    return count - count / 100;
}

int
query_domain_commands(string dom)
{
    return cmd_count[dom];
}
/*
 *			- Get given quest xp.
 */
string *
query_domain_qexp(string dom)
{
    if (sizeof(m_domains[dom]) > FOB_DOM_QXP)
	return m_domains[dom][FOB_DOM_QXP];
    else 
	return 0;
}


/*
 *		- Get given combat xp.
 */
string *
query_domain_cexp(string dom)	
{
    return m_domains[dom][FOB_DOM_CXP];
}

/*
 * For debugging purposes. Only usable by Arch / Keepers
 */
int domain_clear_xp(string dom)
{
    int l;

    l =  SECURITY->query_wiz_level(geteuid(previous_object()));
    if (!(l >= WIZLEV_LORD))
	return 0;
    
    m_domains[dom][FOB_DOM_CXP] = 0;
    m_domains[dom][FOB_DOM_QXP] = 0;
#ifdef LOG_BOOKKEEP
    set_auth(this_object(), "root:root");
    log_file(LOG_BOOKKEEP, 
	     sprintf("%s (%s) Cleared by: %s\n",
		     ctime(time()), dom,
		     capitalize(geteuid(previous_object()))), -1);
#endif
    return 1;
}

/*
 * Give the name of the madwand of a domain
 * Returns 0 if no madwand for the domain exist
 */
public string 
query_domain_madwand(string dom)	
{
    if (sizeof(m_domains[dom]))
	return m_domains[dom][FOB_DOM_MADW];
    else
	return "";
}

/*
 * Function name: set_domain_sanction
 * Description:   Set the sanction string in a domain.
 * Arguments:     who - The domain or wizard to be sanctioned.
 *		  sanct - The sanctioning string.
 *                x_dname - The domain.
 */
public string
set_domain_sanction(string who, string sanct, string x_dname)
{
    string wname, ss, dname;
    int i;

    wname = getwho();

    if (strlen(wname) == 0)
	return "You are not allowed to do this.";

    if (strlen(x_dname) != 0)
	x_dname = capitalize(x_dname);

    switch(WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]))
    {
    case WIZ_KEEPER:
    case WIZ_ARCH:
	dname = x_dname;
	break;

    case WIZ_LORD:
	dname = m_wizards[wname][FOB_WIZ_DOM];
	
	if (dname != x_dname)
	    return "You are not the Lord of that domain.\n";

	if (dname == "")
	    return "I can't find your domain.";
	break;

    default:
	return "You are not Lord of any domain.";
    }

    if (sizeof(m_wizards[lower_case(who)]) && 
	WIZ_TYPE(m_wizards[lower_case(who)][FOB_WIZ_LEV]) == WIZ_MORTAL &&
	sizeof(m_domains[dname]) == 0)
	return "You can't sanction a mortal!";

    if (!sizeof(m_domains[dname]))
	return "I can't find your domain info.";
    else
    {
	/*
	 * Remove sanctioning.
	 */
	if (!strlen(sanct))
	{
	    if (who == "clear")
		m_domains[dname][FOB_DOM_SANC] = ([]);
	    else if (!strlen(m_domains[dname][FOB_DOM_SANC][who]))
		return "Ok. ";
	    else 
		m_domains[dname][FOB_DOM_SANC] = m_delete(m_domains[dname][FOB_DOM_SANC], who);
	}
	else
	{
	    /*
	     * Add sanctioning. Check both who and what.
	     */
	    if (!sizeof(m_domains[capitalize(who)]) && !sizeof(m_wizards[lower_case(who)]) && who != "all")
		return "There's no such player or domain to sanction.";
	    ss = "";
	    for (i = 0 ; i < strlen(sanct) ; i++)
	    {
		switch(sanct[i])
		{
		case 'r':
		case 'R':
		    ss += sanct[i..i];
		    break;

		case 'w':
		    if (m_wizards[lower_case(who)][FOB_WIZ_LEV] < WIZ_NORMAL &&
			sizeof(m_domains[capitalize(who)]) == 0)
			return "The sanctioned player must be a full wizard.";
		    ss += sanct[i..i];
		    break;

		case 'W':
		    if (sizeof(m_wizards[lower_case(who)]) &&
			m_wizards[lower_case(who)][FOB_WIZ_LEV] < WIZ_NORMAL &&
			sizeof(m_domains[capitalize(who)]) == 0)
			return "The sanctioned player must be a full wizard.";

		    if (WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]) >= WIZ_ARCH)
			ss += sanct[i..i];
		    break;
		    
		default:
		    break;
		}
	    }
	    if (strlen(ss))
		m_domains[dname][FOB_DOM_SANC] += ([ who:sanct ]);
	}
    }

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    return "Ok.";
}

/*
 * Function name: query_domain_sanction
 * Description:   Return the domain sanctioning.
 * Arguments:     dname - The domain;
 */
public string
query_domain_sanction(string dname)
{
    dname = capitalize(dname);

    return(m_domains[dname][FOB_DOM_SANC]);
}

/*
 * Function name: set_special_sanction
 * Description:   Set the sanction string in a domain.
 * Arguments:     who - The domain or wizard to be sanctioned.
 *		  sanct - The sanctioning string.
 *                x_dname - The domain.
 *		  path - The path.
 */
public string
set_special_sanction(string who, string sanct, string x_dname, string path)
{
    string wname, ss, dname;
    int i;
    mapping spec;

    wname = getwho();
    path = implode(explode(path, "/"), "/");

    if (strlen(wname) == 0)
	return "You are not allowed to do this.";

    if (strlen(x_dname) != 0)
	x_dname = capitalize(x_dname);

    switch(WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]))
    {
    case WIZ_KEEPER:
    case WIZ_ARCH:
	dname = x_dname;
	break;

    default:
	return "You are not permitted to do this.";
    }

    if (sizeof(m_wizards[lower_case(who)]) && 
	WIZ_TYPE(m_wizards[lower_case(who)][FOB_WIZ_LEV]) == WIZ_MORTAL &&
	sizeof(m_domains[dname]) == 0)
	return "You can't sanction a mortal!";

    if (!sizeof(m_domains[dname]))
	return "I can't find your domain info.";
    else
    {
	/*
	 * Remove sanctioning.
	 */
	if (path == "clear")
	{
	    if (mappingp(m_domains[dname][FOB_DOM_SPECS][lower_case(who)]))
	    {
		m_domains[dname][FOB_DOM_SPECS] = 
		    m_delete(m_domains[dname][FOB_DOM_SPECS], lower_case(who));
	    }
	    return "Ok";
	}

	/*
	 * Check for removal of sanctioning.
	 */
	spec = m_domains[dname][FOB_DOM_SPECS][lower_case(who)];

	if (mappingp(spec) &&
	    spec[path] == sanct)
	{
	    spec = m_delete(spec, path);
	    if (m_sizeof(spec))
		m_domains[dname][FOB_DOM_SPECS][lower_case(who)] = spec;
	    else
		m_domains[dname][FOB_DOM_SPECS] = m_delete(m_domains[dname][FOB_DOM_SPECS], lower_case(who));
	    return "Ok";
	}

	/*
	 * Add sanctioning. Check both who and what.
	 */
	if (!sizeof(m_domains[capitalize(who)]) && 
	    !sizeof(m_wizards[lower_case(who)]) && 
	    who != "all")
	    return "There's no such player or domain to sanction.";

	ss = "";
	for (i = 0 ; i < strlen(sanct) ; i++)
	{
	    switch(sanct[i])
	    {
	    case 'R':
		ss += sanct[i..i];
		break;

	    case 'W':
		if (sizeof(m_wizards[lower_case(who)]) &&
		    m_wizards[lower_case(who)][FOB_WIZ_LEV] < WIZ_NORMAL &&
		    sizeof(m_domains[capitalize(who)]) == 0)
		    return "The sanctioned player must be a full wizard.";
		ss += sanct[i..i];
		break;
		    
	    default:
		break;
	    }
	}
	if (strlen(ss))
	{
	    if (!mappingp(m_domains[dname][FOB_DOM_SPECS][lower_case(who)]))
		m_domains[dname][FOB_DOM_SPECS] +=
		    ([ lower_case(who): ([])]);
	    m_domains[dname][FOB_DOM_SPECS][lower_case(who)] += 
		([ path:sanct ]);
	}
    }

    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);

    return "Ok.";
}

/*
 * Function name: query_special_sanction
 * Description:   Return the domain special sanctioning.
 * Arguments:     dname - The domain;
 */
public mapping
query_special_sanction(string dname)
{
    mapping map;
    string *ind;
    int i;

    /*
     * Make this in a rather convoluted way to ensure the
     * security of the mappings.
     */
    dname = capitalize(dname);

    if (pointerp(m_domains[dname]))
    {
	map = ([]) + m_domains[dname][FOB_DOM_SPECS];
	ind = m_indexes(map);
	for (i = 0 ; i < sizeof(ind) ; i++)
	    map[ind[i]] = ([]) + map[ind[i]];
	return map;
    }
    else
	return ([]);
}


/************************************************************************
 *
 * The wizard administration code.
 *
 */

/*
 * Function name: query_wiz_level
 * Description:   Return the level of a wizard.
 * Arguments:     wname - the wizard
 */
int
query_wiz_level(string wname)
{
    wname = lower_case(wname);

    if (sizeof(m_wizards[wname]))
	return m_wizards[wname][FOB_WIZ_LEV];
    else
	return 0;
}

/*
 * Function name: query_wiz_chl
 * Description:   Return the name of the level changer.
 * Arguments:     wname - the wizard
 */
string
query_wiz_chl(string wname)
{
    wname = lower_case(wname);

    if (sizeof(m_wizards[wname]))
	return m_wizards[wname][FOB_WIZ_CHL];
    else
	return "";
}

/*
 * Function name: query_wiz_dom
 * Description:   Return the domain of a wizard.
 * Arguments:     wname - the wizard
 */
string
query_wiz_dom(string wname)
{
    wname = lower_case(wname);

    if (sizeof(m_wizards[wname]))
	return m_wizards[wname][FOB_WIZ_DOM];
    else
	return "";
}

/*
 * Function name: query_wiz_chd
 * Description:   Return the name of the domain changer.
 * Arguments:     wname - the wizard
 */
string
query_wiz_chd(string wname)
{
    wname = lower_case(wname);

    if (sizeof(m_wizards[wname]))
	return m_wizards[wname][FOB_WIZ_CHD];
    else
	return "";
}

/*
 * Function name: query_wiz_list
 * Description:   Return a list of all wizards of a given type.
 * Arguments:	  type of wizard. -1 = all.
 */
string *
query_wiz_list(int type)
{
    mapping map;

    if (type == -1)
	return m_indexes(m_wizards);
    else
    {
	map = filter(m_wizards, "find_wiz_type", this_object(), type);
	return m_indexes(map);
    }
}

int
find_wiz_type(mixed *data, int type)
{
    if (WIZ_TYPE(data[FOB_WIZ_LEV]) == type)
	return 1;
    else
	return 0;
}

/*
 * Function name: do_chwiz_lev
 * Description:   Change the level (and type) of a wizard.
 *		  This routine assumes that the wizard _does_ exist in the
 *		  m_wizards array. This must have been checked before calling.
 * Arguments:     wname - the wizard
 *		  nlev  - the new level
 * 		  cmder - who orders this
 */
static void
do_chwiz_lev(string wname, int nlev, string cmder)
{
    int olev, otype, ntype;
    object wiz;
    string what;

    if (sizeof(m_wizards[wname]) != 0)
	olev = m_wizards[wname][FOB_WIZ_LEV];
    else
	olev = WIZ_MORTAL;

    if (olev == nlev)
	return;

    otype = WIZ_TYPE(olev);
    ntype = WIZ_TYPE(nlev);

    if (otype != ntype)
	do_chwiz_type(wname, otype, ntype, cmder);

    if (ntype == WIZ_MORTAL)
	return;

    m_wizards[wname][FOB_WIZ_LEV] = nlev;
    m_wizards[wname][FOB_WIZ_CHL] = cmder;

    log_file("LEVEL", ctime(time()) + " " + capitalize(wname) + ": " + 
	     olev + "->" + nlev + " by " + capitalize(cmder) + ".\n", -1);
    set_auth(this_object(), "root:root");
    save_object(SAVEFILE);
    
    if (nlev > olev)
	what = "promoted";
    else
	what = "demoted";

    wiz = find_player(wname);
    if (wiz)
    {
	wiz->open_player();
	set_auth(this_object(), "#:" + wname);
	export_uid(wiz);
	set_auth(this_object(), "root:root");
	wiz->set_trusted(1);
	wiz->set_wiz_level(nlev);
	wiz->save_me(1);
	tell_object(wiz, "You were " + what + " to level " + nlev + " (" +
		    capitalize(WIZ_LEV_NAME(nlev)) + ") by " +
		    capitalize(cmder) + ".\n");
    }
}

/*
 * Function name: do_chwiz_type
 * Description:   Change the type of a wizard.
 *		  This routine assumes that the wizard _does_ exist in the
 *		  m_wizards array. This must have been checked before calling.
 *
 *		  Possible operations:
 *
 *		  Mortal->Apprentice.
 *		  Mortal->Pilgrim.
 *                Apprentice->Retiree
 *                Retiree->Apprentice
 *		  Pilgrim->Any above, Pilgrim->Any below
 *		  Wizard->Any above, Wizard->Any below, but Wizard->Lord will
 *			make the old Lord->Wizard
 *		  Mage as Wizard.
 *		  Lord->Any above, Lord->Any below.
 *		  Arch->Any above, Arch->Any below.
 *		  Keeper->Any below.
 *
 * Arguments:     wname - the wizard
 *		  otype - the old type
 *		  ntype - the new type
 *		  cmder - the command giver
 */
static void
do_chwiz_type(string wname, int otype, int ntype, string cmder)
{
    object wiz, scroll;
    string dname, olord;

    /* 
     * Should be possible, but...
     */
    if (otype == ntype)
	return;

    wiz = find_player(wname);
    set_auth(this_object(), "root:root");

    /*
     * Create a new apprentice.
     */
    if (otype == WIZ_MORTAL && ntype == WIZ_APPRENTICE)
    {
	m_wizards += ([ wname : ({ WIZLEV_APPRENTICE, cmder, "", "" }) ]);
	if (wiz)
	{
	    if (catch(scroll = clone_object(WIZ_APPRENTICE_SCROLL)))
	    {
		write("Error getting the apprentice scroll.\n");
		tell_object(wiz, "Error cloneing the apprentice scroll.\n");
	    } else {
	    	scroll->move(wiz);
	    	tell_object(wiz, "You have been given a scroll containing " +
				 "valuable information. Read it now!\n");
	    }
	    wiz->set_default_start_location(WIZ_ROOM);
	    wiz->save_me(1);
	}
    }

    /*
     * Create a new pilgrim from a mortal
     */
    if (otype == WIZ_MORTAL && ntype == WIZ_PILGRIM)
    {
	m_wizards += ([ wname : ({ WIZLEV_PILGRIM, cmder, "", "" }) ]);
	if (wiz)
	{
	    wiz->set_default_start_location(WIZ_ROOM);
	    wiz->save_me(1);
	}
    }
    
    /*
     * Demote a wizard out of a domain. His directory is moved automatically.
     * Ignore the return value since the objective is filled anyway.
     */
    if (otype > WIZ_RETIRED && ntype < WIZ_NORMAL)
	remove_wizard_from_domain(wname, cmder);

    /*
     * Demote a wizard back to mortality. Just to make damn sure, 
     * remove his old character file all together.
     */
    if (ntype == WIZ_MORTAL)
    {
	if (objectp(SECURITY->finger_player(wname)))
	{
	    rename ("/players/" + extract(wname, 0, 0) + "/" + wname + ".o",
		    "/players/" + extract(wname, 0, 0) + "/" + wname + ".o.org"); 
	}
	m_wizards = m_delete(m_wizards, wname);
	if (wiz)
	    wiz->remove_object();
    }

    /*
     * Promote a wizard in a domain to lord. If neccesary, demote the
     * old lord to wizard status in the same domain. The new lord must
     * have been a member of the domain before this happends.
     */
    if (otype > WIZ_RETIRED && ntype == WIZ_LORD)
    {
	dname = m_wizards[wname][FOB_WIZ_DOM];
	if (strlen((olord = m_domains[dname][FOB_DOM_LORD])) != 0)
	    do_chwiz_lev(olord, WIZMID(WIZLEV_NORMAL), cmder);
	m_domains[dname][FOB_DOM_LORD] = wname;
	tell_domain(dname, wname, "You are now Lord of " + dname + ".\n", 
		    capitalize(wname) + " is the new Lord of " + 
		    dname + ".\n");
    }

    /*
     * Take care of old lords if this isn't already done.
     */
    if (otype == WIZ_LORD && ntype > WIZ_RETIRED)
    {
	dname = m_wizards[wname][FOB_WIZ_DOM];
	m_domains[dname][FOB_DOM_LORD] = "";
	tell_domain(dname, wname, "You are no longer Lord of " + dname + ".\n",
		    capitalize(wname) + "is no longer Lord of " + 
		    dname + ".\n");
    }

    /*
     * Update the command hooks.
     */
    if (wiz)
    {
	wiz->update_hooks();
	tell_object(wiz, "Your command set has changed.\n");
    }
}

/*
 * Function name: change_wiz_level
 * Description:   Change the level (and type) of a wizard.
 * Arguments:     wname - the wizard name
 *		  nlev - the new level
 * Returns:	  Result string
 */
string
change_wiz_level(string wname, int nlev)
{
    string cmder;
    int olev, otype, ntype, cmdtype;

    wname = lower_case(wname);

    cmder = getwho();

    if (strlen(cmder) == 0)
	return "You are not allowed to do this.";

    cmdtype = WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]);

    if (sizeof(m_wizards[wname]) != 0)
	otype = WIZ_TYPE(m_wizards[wname][FOB_WIZ_LEV]);
    else
	otype = WIZ_MORTAL;

    ntype = WIZ_TYPE(nlev);

    if (sizeof(m_wizards[wname]) != 0)
	olev = m_wizards[wname][FOB_WIZ_LEV];
    else
	olev = WIZ_MORTAL;
    
    /*
     * Lords can draft a mortal with 'promote' into the domain as a Madwand
     * Arches and keepers must use 'draft' to accomplish the same thing.
     */
    if (cmdtype == WIZ_LORD && otype == WIZ_MORTAL && ntype > WIZ_MORTAL)
	return draft_madwand(m_wizards[cmder][FOB_WIZ_DOM], wname);

    /*
     * Lords and higher can expel a Madwand from his domain
     */
    if (cmdtype >= WIZ_LORD && otype >= WIZ_NORMAL && ntype < WIZ_NORMAL &&
	m_domains[m_wizards[wname][FOB_WIZ_DOM]][FOB_DOM_MADW] == wname)
	return expel_madwand(m_wizards[wname][FOB_WIZ_DOM], wname);

    /*
     * An apprentice must join a domain before promotion is possible.
     * 'Lone' wizards have a special domain.
     */
    if (otype < WIZ_NORMAL && ntype > WIZ_RETIRED)
	return capitalize(wname) + " is not a member of a domain.";

    /*
     * Lords can only raised from wizard level. They must also be
     * members of the domain they are to become lord in, but this
     * follows automatically.
     */
    if (ntype == WIZ_LORD && otype > WIZ_MAGE && otype != WIZ_LORD)
	return capitalize(wname) + " is too highly ranked to become Lord.";

    if (ntype == WIZ_LORD && otype < WIZ_NORMAL && otype != WIZ_LORD)
	return capitalize(wname) + " is too lowly ranked to become Lord.";

    switch (cmdtype)
    {
    case WIZ_KEEPER:
	/*
	 * Keepers can change any level up to and including Arch.
	 */
	if (otype < WIZ_KEEPER && ntype < WIZ_KEEPER)
	{
	    do_chwiz_lev(wname, nlev, cmder);
	    return "Ok.";
	}
	else
	    return "You can only promote/demote below Keeper level.";
	break;

    case WIZ_ARCH:
	/*
	 * Archwizards can change any level up to but not including Arch.
	 */
	if (otype < WIZ_ARCH && ntype < WIZ_ARCH)
	{
	    do_chwiz_lev(wname, nlev, cmder);
	    return "Ok.";
	}
	else
	    return "You can only promote/demote below Arch level.";
	break;

    case WIZ_LORD:
	/*
	 * Lords can change the levels of their subjects.
	 */
	if (otype < WIZ_NORMAL ||
	    m_wizards[wname][FOB_WIZ_DOM] != m_wizards[cmder][FOB_WIZ_DOM])
	    return capitalize(wname) + " is not a member of your domain.";
	if (ntype > otype || 
	    ntype < WIZ_NORMAL ||
	    (otype == WIZ_MAGE && ntype != WIZ_MAGE) || 
	    (ntype == WIZ_MAGE && otype != WIZ_MAGE) ||
	    otype > WIZ_LORD ||
	    otype == WIZ_LORD && nlev > olev)
	    return "You are not allowed to do that.";
	do_chwiz_lev(wname, nlev, cmder);
	return "Ok.";
	break;

    case WIZ_APPRENTICE:
	/*
	 * Apprentices may retire
	 */
	if (ntype != WIZ_RETIRED || wname != cmder)
	    return "You are not allowed to do that.";
	do_chwiz_lev(wname, nlev, cmder);
	return "Ok.";
	break;

    default:
	/*
	 * At other levels, you can not set levels of wizards.
	 */
	return "You are not allowed to do that.";
    }
}


/*
 * Function name:  create_wizard
 * Description:    Allows certain objects in the game to promote a mortal
 *                 to wizardhood
 * Arguments:      name: the name of the mortal to be promoted
 */
void
create_wizard(string name)
{
    if (member_array(function_exists("make_wiz", previous_object()),
		     WIZ_MAKER) < 0)
	return;

    do_chwiz_lev(name, 1, get_root_uid());

    set_auth(this_object(), "root:root");
    log_file("NEW_WIZ", extract(ctime(time()), 4, 9) + "   " +
	capitalize(name) + "\t avg (" +
	this_interactive()->query_average_stat() + ")\t age (" +
	this_interactive()->query_age() / 43200 + " d)\n", -1);
}

/*
 * Function name:  query_wiz_pretitle
 * Description:    Gives a pretitle for a specific wizard
 * Arguments:      wiz: The wizard player object
 * Returns:        A pretitle for the wizard
 */
string
query_wiz_pretitle(mixed wiz)
{
    int             l;

    if (stringp(wiz))
	wiz = find_player(wiz);
    if (!objectp(wiz))
	return "";

    l = WIZ_TYPE(wiz->query_wiz_level());

    if (l == WIZ_APPRENTICE)
	return (IS_MALE(wiz)) ? "Squire" : ((IS_FEMALE(wiz)) ? "Maid" : "Some");
    if (l == WIZ_NORMAL)
	return (IS_MALE(wiz)) ? "Sir" : ((IS_FEMALE(wiz)) ? "Dame" : "It");
    if (l == WIZ_PILGRIM)
	return (IS_FEMALE(wiz)) ? "Sister" : "Brother";
    if (l == WIZ_MAGE)
	return "Honorable";
    if (l == WIZ_RETIRED)
	return (IS_MALE(wiz)) ? "Mister" : ((IS_FEMALE(wiz)) ? "Miss" : "It");
    if (l == WIZ_LORD)
	return (IS_FEMALE(wiz)) ? "Lady" : "Lord";
    if (l == WIZ_ARCH)
	return "Proctor";
    if (l == WIZ_KEEPER)
	return "Master";

    return "";
}

/*
 * Function name:  query_wiz_path
 * Description:    Gives a default path for a wizard
 * Arguments:      wname - the wizard name
 * Returns:        A default path for the wizard
 */
string
query_wiz_path(string wname)
{
    string path, dname;

    /*
     * A domain path.
     */
    if (sizeof(m_domains[wname]) != 0)
	return "/d/" + wname;

    wname = lower_case(wname);

    /*
     * Root.
     */
    if (wname == get_root_uid())
	return "/syslog";

    if (sizeof(m_wizards[wname]) != 0)
	dname = m_wizards[wname][FOB_WIZ_DOM];
    else
	return "";
    
    /*
     * A wizard who is a domain member.
     */
    if (strlen(dname) != 0)
	return "/d/" + dname + "/" + wname;
    
    /*
     * An Apprentice or Pilgrim.
     */
    if (sizeof(m_wizards[wname]) != 0)
	return "/doc";
    
    /*
     * Just about anyone else.
     */
    return "";
}

/*
 * Function name:  query_mage_preload
 * Description:    Finds all WIZARD_DOMAIN mages and makes a preload list
 * Returns:	   List of files to preload.
 */
static string *
query_mage_preload()
{
    string *prefiles, *wizzes;
    int i;

    wizzes = m_domains[WIZARD_DOMAIN][FOB_DOM_MEM];

    prefiles = ({ });
    for (i = 0 ; i < sizeof(wizzes) ; i++)
	prefiles += ({ "/d/" + WIZARD_DOMAIN + "/" + 
			   wizzes[i] + "/" + PRELOAD_WIZARD });

    return prefiles;
}

/*
 * Function name:  query_domain_preload
 * Description:    Make a list of domainfiles to preload.
 * Returns	    Returns a list of the files to preload.
 */
static string *
query_domain_preload()
{
    int i;
    string *prefiles, *domains, lord;

    prefiles = ({});
    domains = m_indexes(m_domains);
    
    for (i = 0 ; i < sizeof(domains) ; i++) 
	prefiles += ({ "/d/" + domains[i] + "/" + PRELOAD_DOMAIN });

    return prefiles;
}

/*
 * Function name: retire_wiz
 * Description:   Change wizard level to the standard retired level.
 *              
 * Arguments:     name - the retiree
 * Returns:     result string.
 */
public string
retire_wiz(string name)
{
    string cmder;
    int clev;

    cmder = getwho();
    name = lower_case(name);

    if (sizeof(m_wizards[cmder]) == 0)
      return "You are not allowed to do this.\n";

    clev = m_wizards[cmder][FOB_WIZ_LEV];

    if ((WIZ_TYPE(clev) < WIZ_ARCH) && (cmder != name))
      return "You may only retire yourself.";

    change_wiz_level(name, WIZLEV_APPRENTICE);
    change_wiz_level(name, WIZLEV_RETIRED);

    log_file("RETIRED", ctime(time()) + " " + capitalize(name) +
           " retired by: " + capitalize(cmder) + "\n", -1); 
    return "Ok.";
}

/*
 * Function name: add_global_read
 * Description:   Add a wizard to the global read list.
 * Arguments:     name - The name of the wiz to be added.
 *		  comment - A suitable comment to store along with the name.
 * Returns:	  Result string.
 */
string 
add_global_read(string name, string comment)
{
    string cmder;
    int *wizzes;

    cmder = getwho();

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "You are not allowed to do this.";

    if (WIZ_TYPE(m_wizards[name][FOB_WIZ_LEV]) < WIZ_NORMAL ||
    	WIZ_TYPE(m_wizards[name][FOB_WIZ_LEV]) > WIZ_MAGE)
	return "That person is either not a true wizard or an arch.";

    if (!mappingp(m_global_read))
	m_global_read = ([]);

    if (m_global_read[name])
	return "That wizard already has global access.";

    m_global_read += ([ name: ({ cmder, comment }) ]);
    save_object(SAVEFILE);
    return "Added.";
}

/*
 * Function name: remove_global_read
 * Description:   Remove a wizard from the global access list.
 * Arguments:     name - The wizard to remove.
 * Returns:	  Result string.
 */
string 
remove_global_read(string name)
{
    string cmder;

    cmder = getwho();

    if (WIZ_TYPE(m_wizards[cmder][FOB_WIZ_LEV]) < WIZ_ARCH)
	return "You are not allowed to do this.";

    if (!mappingp(m_global_read))
	m_global_read = ([]);

    if (!m_global_read[name])
	return "There is no such wizard registered.";

    m_global_read = m_delete(m_global_read, name);
    save_object(SAVEFILE);
    return "Removed.";
}

/*
 * Function name: query_global_read
 * Returns:	  The global read list.
 */
mapping 
query_global_read()
{
    if (!mappingp(m_global_read))
	m_global_read = ([]);

    return m_global_read;
}

/*
 * Function name: 
 * Description:   
 * Arguments:     
 * Returns:	  
 */
