/*
  armour.c

   Contains all routines relating to armours of any kind.

   Defined functions and variables:

   set_ac(int)		Sets the armour class.

   set_at(type)		Set the armour type.
			If not defined, set to default from /sys/formulas.h

   set_am(list)		Set armour modifier vs weapon damage type.
			If not defined, set to default from /sys/formulas.h

   set_af(obj)		Sets the name of the object that contains the function
			to call for extra defined wear() and remove()
			functions.

   wear_arm(string)	Wear the armour defined in string.

   remove_arm(string)	Unwear the armour defined in string.

   query_worn()         Ask if this object is worn, returns the wearer

 */

/*
   In order to make the armour recoverable you must add the following code:

   query_recover() { return "path:myargs" + query_arm_recover(); }
   init_recover(arg) { ...my recover code...; init_arm_recover(arg); }
*/

#pragma save_binary

inherit "/std/object";

#include <wa_types.h>
#include <formulas.h>
#include <stdproperties.h>
#include <macros.h>
#include <cmdparse.h>

/*
 * Prototypes
 */
	string	arm_condition_desc();
	void	update_prop_settings();
public	int	*query_slots();

/*
 * Variables
 */
static int      arm_ac,		/* Armour class */
                arm_at,		/* Armour type */
                *arm_mods,	/* Armour modifiers */
                worn,		/* Worn or not */
		arm_shield,     /* Bodypart(s) protected by shield */
		worn_on_part,	/* On which part is the armour worn */
		max_value,	/* The value of armour when not worn down */
		hits,		/* No of hits on armour without worse cond */
		condition,	/* How correded/worn down the armour has been */
		likely_break,	/* How likely the armour is to break */
		likely_cond,	/* Likely cond of armour will worsen */
		repair;		/* How much of this has been repaired */
static object   wearer,		/* Who is holding it */
                wear_func;	/* Object that defines extra wear/remove */
static string   gFail;          /* Acc error messages when wearing */

/*
 * Function name: create_armour
 * Description:   Reset the armour
 */
void
create_armour() 
{
    ::create_object();
}

/*
 * Function name: create_object
 * Description:   Create the armour. (constructor)
 */
public nomask void
create_object()
{
    arm_at = F_ARMOUR_DEFAULT_AT;
    arm_mods = allocate(W_NO_DT);
    likely_break = 2;
    likely_cond = 3;
    set_name("armour");
    worn = 0;
    add_prop(OBJ_I_VALUE, "@@query_value");
    add_prop(OBJ_I_VOLUME, 1000);
    add_prop(OBJ_I_WEIGHT, 500);

    create_armour();

    update_prop_settings();
}

/*
 * Function name: reset_armour
 * Description:   Reset the armour (default)
 */
void
reset_armour() { ::reset_object(); }

/*
 * Function name: reset_object
 * Description:   Reset the armour. 
 */
nomask void
reset_object() { reset_armour(); }

/*
 * Function name: init
 * Description:   Initialize some armour related actions.
 */
void
init()
{
    ::init();
    add_action("wear_arm", "wear");
    add_action("remove_arm", "remove");
}

/*
 * Function name: short, pshort, long
 * Description:   We change the short, plural short and long description of
 *                the armour if it is broken.
 * Returns:       The description.
 */
public varargs string
short(object for_obj)
{
    string str;

    str = "";
    if (query_prop(OBJ_I_BROKEN))
        str = "broken ";

    return str + ::short(for_obj);
}

public varargs string
plural_short(object for_obj)
{
    string str;

    str = "";
    if (::plural_short(for_obj))
    {
        if (query_prop(OBJ_I_BROKEN))
            str = "broken ";
        return str + ::plural_short(for_obj);
    }

    return ::plural_short(for_obj);
}

public varargs string
long(mixed for_obj)
{
    string str;

    str = ::long(for_obj);
    if (objectp(for_obj))
	str += arm_condition_desc();
    return str;
}

/*
 * Function name: wear_arm
 * Description:   wear the armour.
 * Arguments:	  what: what to wear.
 */
nomask int
wear_arm(string what)
{
    object 	*a;
    
    notify_fail(capitalize(query_verb()) + " what?\n");  /* access failure */
    
    /*
     * This is an exception from the rule that we should do 'notify_fail'
     * and return 0, all the time. If we do then all armours will run
     * this code, which is somewhat wastefull.
     */
    gFail = "";
    a = CMDPARSE_ONE_ITEM(what, "wear_one_thing", "wear_access");
    if (!sizeof(a))
    {
	if (!strlen(gFail))
	    return 0;
	else
	    write(gFail);
    }
    gFail = 0;
    return 1;
}

/*
 * Check that what we try to get is an armour and that we carry it
 */
public nomask
wear_access(what)
{
    if (function_exists("create_object", what) != ARMOUR_FILE)
	return 0;
    if (environment(what) == this_player())
	return 1;
    else
	return 0;
}
    
nomask
wear_one_thing(what)
{
    string fail;

    /*
     * Is it this armour?
     */
    if (this_object() != what)
	fail = what->wear_me();
    else
	fail = wear_me();
    if (stringp(fail))
    {
	gFail += fail;
	return 0;
    }
    return 1;
}

/*
 * Wear this armour
 * Return error message, write success message. Return 1 if success.
 */
wear_me()
{
    int wret, il;
    string wfail, what, how;

    what = short();
    if (worn)
	return "You already wear the " + what + ".\n";
    else if (this_player() != environment(this_object()))
	return "You must carry " + what + " in order to wear it.\n";
    else if (query_prop(OBJ_I_BROKEN))
	return "You can't wear broken armours.\n";

    wearer = this_player();
    worn_on_part = arm_at;

    if (worn_on_part == A_SHIELD)
    {
	if (wearer->query_armour(W_LEFT) == 0)
	    worn_on_part = W_LEFT;
	else if (wearer->query_armour(W_RIGHT) == 0)
	    worn_on_part = W_RIGHT;
	else
	    return "Your arms are already busy with other assignments.\n";
    } else if (worn_on_part == A_ANY_FINGER)
    {
	if (wearer->query_armour(A_L_FINGER) == 0)
	    worn_on_part = A_L_FINGER;
	else if (wearer->query_armour(A_R_FINGER) == 0)
	    worn_on_part = A_R_FINGER;
	else
	    return "There is no place left on your fingers.\n";
    } else
	for (il = TS_TORSO; il <= worn_on_part; il *= 2)
	    if ((worn_on_part & il) && wearer->query_armour(il))
	    	return "The " + wearer->query_armour(il)->short() +
			" is in the way.\n";

    if (stringp(wfail = wearer->wear_arm(this_object())))
	return wfail;

    wret = 0;

    /*
     * A wear function in another object.
     */
    if ((!wear_func) || (!(wret = wear_func->wear(this_object()))))
    {
	how = wear_how(worn_on_part);
	write("You wear the " + what + how + ".\n");
	say(QCTNAME(this_player()) + " wears the " + what + ".\n");
    }

    /*
     * If the wearfunc returned a value < 0 then we can not wear the armour
     */
    if (intp(wret) && wret >= 0)
    {
	worn = 1;
	return 1;
    } 
    else
    {
	wearer->remove_arm(this_object());
	worn = 0;
	if (stringp(wret))
	    return wret;
	else
	    return "You can not wear " + short(this_player()) + ".\n";
    }
}

/*
 * Function name: wear_how
 * Description:   Generate the string that is shown when the armour is worn
 * Arguments:	  location: the location(s) covered by the armour
 */
public string
wear_how(int location)
{
    string how, pos;

    if (this_player() == wearer)
	pos = "your";
    else
	pos = wearer->query_possessive();

    switch (location)
    {
	case A_TORSO:
	    how = " over " + pos + " shoulders"; break;
	case A_HEAD:
	    how = " on " + pos + " head"; break;
	case A_LEGS:
	    how = " on " + pos + " legs"; break;
	case W_RIGHT:
	case A_R_ARM:
	    how = " on " + pos + " right arm"; break;
	case W_LEFT:
	case A_L_ARM:
	    how = " on " + pos + " left arm"; break;
	case A_FEET:
	    how = " on " + pos + " feet"; break;
	case A_ROBE:
	    how = " around " + pos + " body"; break;
	case A_NECK:
	    how = " around " + pos + " neck"; break;
	case A_WAIST:
	    how = " around " + pos + " waist"; break;
	case A_R_FINGER:
	    how = " on " + pos + " right finger"; break;
	case A_L_FINGER:
	    how = " on " + pos + " left finger"; break;
	case A_HANDS:
	    how = " on both hands"; break;
	case A_R_HAND:
	    how = " on " + pos + " right hand"; break;
	case A_L_HAND:
	    how = " on " + pos + " left hand"; break;
	case A_R_FOOT:
	    how = " on " + pos + " right foot"; break;
	case A_L_FOOT:
	    how = " on " + pos + " left foot"; break;
	case A_ARMS:
	    how = " on both arms"; break;
	default: 
	    how = "";
    }

    return how;
}

/*
 * Function name: remove_arm
 * Description:   Unwear the armour.
 * Arguments:	  what: What to remove.
 */
remove_arm(what)
{
    object 	*a;
    string	prep, loc;
    int il;
    
    notify_fail(capitalize(query_verb()) + " what?\n");  /* access failure */
    if (!what)
	return 0;

    /*
     * This is an exception from the rule that we should do 'notify_fail'
     * and return 0, all the time. If we do then all armours will run
     * this code, which is somewhat wastefull.
     */
    if (!parse_command(what, all_inventory(this_player()), "%i %p %s",
		       a, prep, loc))
    {
	prep = 0; loc = 0;
	if (!parse_command(what, all_inventory(this_player()), "%i", a))
	    return 0;
    }
    il = a[0];
    a = (a & this_player()->query_armour(-1));
    if (prep)
	a = filter(a, "find_arm", this_object(), loc);
    a = CMDPARSE_STD->normal_access( ({ il }) + a, 0, 0, 1);

    if (!sizeof(a))
    {
	if (prep)
	    notify_fail("No such armour worn " + prep + " " + loc + ".\n");
	else
	    notify_fail("No such armour worn.\n");
	return 0;
    }

    for (il = 0; il < sizeof(a); il++)
	a[il]->remove_me();

    return 1;
}

/*
 * Remove the armour, return 0 if failure
 */
nomask int
remove_me()
{
    int wret;

    if (!worn || !wearer)
	return 0;
	
    /*
     * A remove function in another object.
     */
    if ((!wear_func) || (!(wret = wear_func->remove(this_object()))))
    {
	if (check_seen(this_player()))
	    write("You remove the " + short() + ".\n");
	else
	    write("You remove something.\n");
	say(QCTNAME(this_player()) + " removes " + QSHORT(this_object()) + ".\n");
    }
    if (wret >= 0)
    {
	wearer->remove_arm(this_object());
	worn = 0;
	return 1;
    }
    else
	return 0;
}

static int
find_arm(object arm, string ploc)
{
    int where;
    string a, b;

    ploc = "d " + ploc + " d";
    where = arm->query_at();

    if (sscanf(ploc, "%sleft%s", a, b) && (where & A_R_ARM))
	return 0;

    if (sscanf(ploc, "%sright%s", a, b) && (where & A_L_ARM))
	return 0;

    if (sscanf(ploc, "%shand%s", a, b) && (where & A_LEGS))
	return 0;

    if (sscanf(ploc, "%sfoot%s", a, b) &&
	((where & A_L_ARM) ||
	 (where & A_R_ARM)))
	return 0;
    return 1;
}

/*
 * Function name: leave_env
 * Description:   The armour is moved from the inventory
 * Arguments:	  from - Where from
 */
void
leave_env(object from, object to)
{
    ::leave_env(from, to);

    if (!worn)
	return;

    if ((!wear_func) || (!wear_func->remove(this_object())))
	tell_object(wearer, "You remove the " + short() + ".\n");

    wearer->remove_arm(this_object());
    worn = 0;
}

/* Define set/query functions for the ordinary variables in an object */

void
set_name(string name)
{
    if (query_lock())
	return;			/* All changes have been locked out */

    ::set_name(name);
}

/*
 * Function name: set_ac
 * Description:   Set the armour class of the object
 */
void
set_ac(int class)
{
    if (query_lock())
	return;			/* All changes have been locked out */

    arm_ac = class;
}

/*
 * Description: Give the ac for a specific hitlocation that is protected
 * Arguments:   hid: Hitlocation id
 */
varargs int
query_ac(int hid)
{
    return arm_ac - condition + repair;
}

/*
 * Function name: set_at
 * Description:   Set armour type
 */
void
set_at(int type)
{
    if (query_lock())
	return;			/* All changes have been locked out */

    arm_at = type;
}

/*
 * Description: Give the armour type for this armour.
 *		Armour type is a combination of the tool slots that this
 *		armour takes up and possibly the magic flag.
 *		(see /sys/wa_types.h)
 * Arguments:   hid: Hitlocation id
 */
int
query_at()
{
    return arm_at;
}

/*
 * Function name: set_am
 * Description:   Set the modifyers for different attacks
 */
void
set_am(int *list)
{
    if (query_lock())
	return;			/* All changes have been locked out */

    if (F_LEGAL_AM(list))
	arm_mods = list;
}

/*
 * Description: Give the ac modifier for a specific hitlocation
 * Arguments:   hid: Hitlocation id
 */
int *
query_am(int hid)
{
    return arm_mods + ({});
}

/*
 * Function name: set_condition
 * Description:   Use this to increases the corroded status on armour. If the
 *		  armour gets dented or anything that turns it into worse
 *		  condition, call this function.
 * Arguments:     cond - The new condition we want (can only be raised)
 * Returns:       1 if new condition accepted
 */
int
set_condition(int cond)
{
    if (cond > condition)
    {
	condition = cond;
	if (F_ARMOUR_BREAK(condition - repair, likely_break))
	    call_out("remove_broken", 1);
	if (worn && wearer)
	    wearer->update_armour(this_object());
	return 1;
    }
    return 0;
}

/*
 * Function name: query_condition
 * Description:   Return the general condition modifier. It indicates how
 *		  many times the condition of this armour has been worsened.
 *		  The tru condition of the armour is:
 *		   	base_ac - condition + repairs
 * Returns:	  The general condition modifier
 */
int query_condition() { return condition; }

/*
 * Function name: set_likely_cond
 * Description:   Set how likely the armour will get worn down if hit
 * Arguments:     i - how likely [0, 30] recommended
 */
void set_likely_cond(int i) { likely_cond = i; }

/*
 * Function name: query_likely_cond
 * Description:   How likely is this armour to wear down with use (a relative
 *		  number)
 * Returns:	  The likliness
 */
int query_likely_cond() { return likely_cond; }

/*
 * Function name: set_likely_break
 * Description:   Set how likely the armour is to break if you use it.
 * Argument:      i - How likely, [0, 20] recommended
 */
void set_likely_break(int i) { likely_break = i; }

/*
 * Function name: query_likely_break
 * Description:   How likely this piece of armour is to break (a relative number)
 * Returns:       How liely it is
 */
int query_likely_break() { return likely_break; }

/*
 * Function name: remove_broken
 * Description:   The armour got broken
 */
remove_broken()
{
    add_prop(OBJ_I_BROKEN, 1);
    if (!worn || !wearer)
	return;

    tell_object(wearer, "The " + short(wearer) + " breaks!!!\n");
    say("The " + QSHORT(this_object()) + " worn by " +
	QTNAME(wearer) + " breaks!!!\n",
	wearer);

    wearer->remove_arm(this_object());
    worn = 0;
}

/*
 * Function name: set_repair
 * Description:   When trying to repair the armour, call this function. Repairs
 *		  can only increase the repair factor.
 * Arguments:	  rep - The new repair number
 * Returns:	  1 if new repair status accepted
 */
int
set_repair(int rep)
{
    if (rep > repair && F_LEGAL_ARMOUR_REPAIR(rep, condition))
    {
	repair = rep;
	if (worn && wearer)
	    wearer->update_armour(this_object());
	return 1;
    }
    return 0;
}

/*
 * Function name: query_repair
 * Description:   How many times this weapon has been repaired. Actual ac is:
 *			base_ac - condition + repairs
 * Returns:	  How many times...
 */
int query_repair() { return repair; }

/*
 * Function name: set_armour_hits
 * Description:   By setting the hits counter you will have influence over how
 *		  likely the armour is to get in a worse condition. The hits 
 *		  variable keeps track of how many times this piece of armour
 *		  has been hit.
 * Argument:	  new_hits - integer
 */
public void
set_armour_hits(int new_hits) { hits = new_hits; }

/*
 * Function name: query_armour_hits
 * Description:   This function returns how many times this armour has been hit
 * 		  since last time it got degenerated. The lower this number is
 *		  the less likely the armour will degenerate (strange isn't it?)
 * Returns:	  The hits variable
 */
public int
query_armour_hits() { return hits; }

/*
 * Function name: add_prop_obj_i_value
 * Description:   Someone is adding the value prop to this object.
 * Arguments:	  val - The new value (mixed)
 * Returns:	  1 if not to let the val variable through to the prop
 */
int
add_prop_obj_i_value(mixed val)
{
    if (!max_value)
    {
	max_value = 10;
	return 0;
    }

    if (intp(val) && val)
	max_value = val;

    return 1;
}

/*
 * Function name: query_value
 * Description:   Qhat's the value of this armour
 */
int
query_value()
{
    if (query_prop(OBJ_I_BROKEN))
	return 0;

    return max_value * F_ARMOUR_VALUE_REDUCE(condition - repair) / 100;
}

/*
 * Sets the object to call wear/remove_arm in when this occurs.
 * Those functions can return:
 *		0 - No affect the armour can be worn / removed
 *		1 - It can be worn / removed but no text should be printed
 *		    (it was done in the function)
 *		-1  It can not be worn / removed default failmsg will be 
 *		    written
 *             string  It can not be worn / removed 'string' is the 
 *		       fail message to print
 */
void
set_af(object obj)
{
    if (query_lock())
	return;			/* All changes have been locked out */

    wear_func = obj;
}

object
query_af()
{
    return wear_func;
}

/*
 * Function name: got_hit
 * Description:   Notes that the defender has been hit. It can be used
 *		  to reduce the ac for this hitlocation for each hit.
 * Arguments:     hid:   The hitloc id, ie the bodypart hit.
 *                ph:    The %hurt
 *                att:   Attacker
 *		  aid:   The attack id
 *                dt:    The damagetype
 *		  dam:   The damage done to us in hit points
 */
varargs int
got_hit(int hid, int ph, object att, int dt, int dam)
{
    if (dam <= 0)
	return 0;

    hits++;
    if (F_ARMOUR_CONDITION_WORSE(hits, arm_ac, likely_cond))
    {
	hits = 0;
	set_condition(query_condition() + 1);
    }

    return 0;
}

/*
 * Function name: check_armour
 * Description:   Check file for security.
 */
int
check_armour()
{
    return 1;
}

/*
 * Function name: set_default_armour
 * Description:
 * Arguments:
 * Returns:
 */
public varargs void
set_default_armour(int ac, int at, int *am, object af)
{
    /* Sets the armour class.
    */
    if (ac) set_ac(ac); 
    else set_ac(1);

    /* Set the armour type.
    */
    if (at) set_at(at);
    else set_at(A_BODY);
    
    /* Set armour modifier vs weapon damage type.
    */
    if (am) set_am(am);
    else set_am(A_NAKED_MOD);

    
    /* Sets the name of the object that contains the function
       to call for extra defined wear_arm() and remove_arm()
       functions.
    */
    if (af) set_af(af);
}
    

/*
 * Function name: set_shield_slot
 * Description:   Set the hitlocation(s) protected by the shield or magic
 *		  armour
 * Arguments:     hids: Hitlocation(s)
 */
public void
set_shield_slot(mixed hids) { arm_shield = hids; }

public nomask int *
query_protects();

/*
 * Function name: query_shield_slots
 * Description:   Give a bodypart protected by a shield or magic armour
 */
public int *
query_shield_slots()
{
    int *ss;

    ss = check_call(arm_shield);
    if (ss) return ss;
    if (worn_on_part & W_LEFT) return ({ A_L_ARM });
    if (worn_on_part & W_RIGHT) return ({ A_R_ARM });
    if (query_prop(OBJ_I_IS_MAGIC_ARMOUR)) return query_protects();
    return ({ });
}

/*
 * Description: Return the slots this armour allocates
 */
public int *
query_slots() 
{
    int abit, *hids;

    for (hids = ({}), abit = 2; abit <= worn_on_part; abit *= 2)
    {
	if (worn_on_part & abit)
	{
	    hids = hids + ({ abit });
	}
    }
    return hids;
}

/*
 * Function name: query_protects
 * Description:   Give an array of the bodyparts protected by this armour.
 */
public nomask int *
query_protects()
{
    int abit, *hids, *chid;
    /*
       Table of armour slots <-> Hitlocations

             TORSO HEAD LEGS R_ARM L_ARM ROBE SHIELD

    TORSO      X      
    HEAD	    X
    LEGS	         X
    R_ARM	               X
    L_ARM	                     X
    ROBE       X         X                
    SHIELD                     X     X                          
    MAGIC      X    X    X     X     X   	

	Max ac is 100 for all hitlocations. If the sum of the ac for a
	given hitlocation is > 100 then it is set to 100.
    */
    for (hids = ({}), abit = 1; abit <= worn_on_part; abit *= 2)
    {
	if (worn_on_part & abit)
	{
	    switch (abit)
	    {
	    case A_TORSO:
	    case A_HEAD:	
	    case A_LEGS:
	    case A_R_ARM:
	    case A_L_ARM:
		chid = ({ abit });
		break;
	    case A_ROBE:  
		chid = ({ A_TORSO, A_LEGS });
		break;
	    case W_LEFT:
	    case W_RIGHT:
		chid = query_shield_slots();
		break;
	    default:
		chid = ({});
	    }
	    hids = (hids - chid) + chid;
	}
    }
    return hids;
}
 
/*
 * Function name: update_prop_settings
 * Description:   Will uppdate weight and value of this object to be legal
 */
nomask void
update_prop_settings()
{
#if 0
write("Val: " + query_prop(OBJ_I_VALUE) + " Weight: " + query_prop(OBJ_I_WEIGHT) + ".\n");
#endif

    if (query_prop(OBJ_I_VALUE) < F_VALUE_ARMOUR(arm_ac) &&
			!query_prop(OBJ_I_IS_MAGIC_ARMOUR))
	add_prop(OBJ_I_VALUE, F_VALUE_ARMOUR(arm_ac));
 
    if (F_WEIGHT_FAULT_ARMOUR(query_prop(OBJ_I_WEIGHT), arm_ac, arm_at) &&
			!query_prop(OBJ_I_IS_MAGIC_ARMOUR))
	add_prop(OBJ_I_WEIGHT, F_WEIGHT_DEFAULT_ARMOUR(arm_ac, arm_at));

#if 0
write("Val: " + query_prop(OBJ_I_VALUE) + " Weight: " + query_prop(OBJ_I_WEIGHT) + ".\n");
#endif
}

/*
 * Function name: query_worn
 * Description:   If this object is worn or not
 * Returns:       The object who wears this object if this object is worn
 */
object
query_worn()
{
    if (worn) return wearer;
}

/*
 * Function name: stat_object
 * Description:   This function is called when a wizard wants to get more
 *                information about an object.
 * Returns:       str - The string to write..
 */
string
stat_object()
{
    string str;

    str = ::stat_object();

    str += "AC: " + arm_ac + "\t";
    if (sizeof(arm_mods))
	str += "Mods: " + arm_mods[0] + ", " + arm_mods[1] + ", " +
		arm_mods[2] + "\t";

    str += "Type: " + arm_at + "\n";
    if (!query_worn())
	wearer = this_player();
    if (arm_at == A_SHIELD)
	str += "It's a shield.\n";
    else if (arm_at == A_ANY_FINGER)
	str += "It's a ring.\n";
    else
    	str += "It can be worn" + wear_how(arm_at) + ".\n";

    str += "Hits: " + hits + "  Condition: " + condition + "  Repairs: " + repair;

    return str + "\n";
}

/*
 * Function name: arm_condition_desc
 * Description:   Returns the description of this armour
 */
string
arm_condition_desc()
{
    string str;

    if (query_prop(OBJ_I_BROKEN))
	return "It is broken.\n";

    switch(condition - repair)
    {
	case 0:
	    str = "in prime condition"; break;
	case 1:
	case 2:
	    str = "a little worn down"; break;
	case 3:
	case 4:
	    str = "in a very bad shape"; break;
	case 5:
	case 6:
	case 7:
	    str = "in big need for repairs"; break;
	default:
	    str = "going to break any second"; break;
    }

    return "It looks like it is " + str + ".\n";
}

/*
 * Function name: appraise_object
 * Description:   Called when someone appraises this object
 */
void
appraise_object(int num)
{
    ::appraise_object(num);

    /* 
     * Condition in long description now
    write("\n" + arm_condition_desc());
     */
}

/*
 * Function name: query_arm_recover
 * Description:   Return the recover strings for changing armour variables.
 * Returns:	  Part of the recoder string
 */
string
query_arm_recover()
{
    return "#a_h#" + hits + "#a_c#" + condition + "#a_r#" + repair +
	   "#a_b#" + query_prop(OBJ_I_BROKEN) + "#";
}

/*
 * Function name: init_arm_recover
 * Description:   Initialize the armour variables at recover.
 * Arguments:	  arg - The recover string as recieved from query_arm_recover()
 */
void
init_arm_recover(string arg)
{
    string foobar;
    int tmp;

    sscanf(arg, "%s#a_h#%d#%s", foobar, hits, foobar);
    sscanf(arg, "%s#a_c#%d#%s", foobar, condition, foobar);
    sscanf(arg, "%s#a_r#%d#%s", foobar, repair, foobar);
    sscanf(arg, "%s#a_b#%d#%s", foobar, tmp, foobar);
    if (tmp)
	add_prop(OBJ_I_BROKEN, tmp);
}
