/*
  /std/combat/ctool.c

  This is the externalized combat routines for monsters and other mobiles
  that are able to use tools for aid in combat. Typical tools are weapons
  and armours.

  This object is cloned and linked to a specific individual when
  engaged in combat. The actual object resides in 'limbo'.

*/
#pragma save_binary

inherit "/std/combat/cplain";

#include <wa_types.h>
#include <composite.h>
#include <stdproperties.h>
#include <ss_types.h>
#include <macros.h>
#include <filter_funs.h>

/* Prototypes */
public void cb_remove_arm(object wep);
static void adjust_ac(int hid, object arm, int rm);
public mixed cb_query_armour(int which);

static	mapping	aid_attack,  	    /* The 'weapon' aiding the attacks */ 
		aid_hitloc,         /* The 'armours' protecting the hitlocs */
                tool_slots; 	    /* The object occupying a certain slot' */ 

/* 
   NOTE
 
     There is a limited number of tool slots and one slot can only be
     occupied by one object at a time. One object may occupy many slots though.
    
     Armours can protect one or more hit locations. What hit locations
     a given armour protects is given by the function 'query_protects' in
     the armour. 

     Weapons can only aid one attack. The attack id is given by the
     function 'query_attack_id' in the weapon.

     The tool slots are used to ensure that the use of a weapon and an armour
     do not conflict. The slots that a weapon or armour occupies are given
     by the function 'query_slots' in weapons and armours.

     Observe that what attacks and hitlocation a weapon and armour is defined
     to aid is independant of what slots it allocates.

     The combat system makes no checks on that relevant tool slots aids
     relevant attacks and hit locations. This is taken care of in
     /std/armour.c and /std/weapon.c

     Tool slots are made as defined by the objects. The only thing that the
     combat system does is to ensure that two tools do not use the same 
     slot.

   MAGICAL armours

     Magical armours can protect one or more hitlocations without allocating
     a tool slot. If the property OBJ_I_IS_MAGIC_ARMOUR is defined the armour
     is considered to be a magic one. The magic armour can of course allocate
     any number of tool slots, just like a normal armour.

   MAGICAL weapons

     Magical weapons work just like normal weapons. A magical 'weapon' that
     allocates no combat slot is not a 'weapon' it is an independant magic
     object and must cause damage onto the enemy on its own. Such magic
     attacks are not supported in the combat system. 

*/

/*
 * Function name: create_cplain
 * Description:   Reset the combat functions
 */
public nomask void
create_cplain()
{
    if (me)
	return;
    this_object()->create_ctool();
}

/*
 * Function name: cb_configure
 * Description:   Configure humanoid attacks and hitlocations.
 * Returns:       True if hit, otherwise 0.
 */
public void
cb_configure()
{
    ::cb_configure();
    aid_attack = ([]);
    aid_hitloc = ([]);
    tool_slots = ([]);

    qme()->add_subloc(SUBLOC_WIELD);
    qme()->add_subloc(SUBLOC_WORNA);
}

/*
 * Function name: cb_wield_weapon
 * Description:   Wield a weapon. Here 'weapon' can be any object.
 * Arguments:	  wep - The weapon to wield.
 * Returns:       True if wielded.
 */
public mixed
cb_wield_weapon(object wep)
{
    int il, aid, *slots, *prot, extra;
    object *obs;

    aid = (int) wep->query_attack_id();
    slots = (int*) wep->query_slots();
    prot = (int *) wep->query_protects();

    /* Can we use this weapon ?
     */
    if (!query_attack(aid))
	return "It doesn't seem to fit your body very well.\n";

    /*
     * Are all the slots it needs free.
     */
    for (il = 0; il < sizeof(slots); il++)
	if (tool_slots[slots[il]])
	    return "Something is in the way.\n";

    if (sizeof(prot) && (sizeof(query_hitloc_id() & prot) < sizeof(prot)))
	return "It seems not fit for you to wield.";

    if (wep->move(qme(), SUBLOC_WIELD))
	return "You cannot wield this weapon for some strange reason.\n";

    for (il = 0; il < sizeof(slots); il++)
	tool_slots[slots[il]] = wep;
    
    aid_attack[aid] = wep; 

    add_attack(wep->query_hit(), wep->query_modified_pen(), wep->query_dt(), 
	       wep->query_procuse(), aid);

    for (il = 0; il < sizeof(prot); il++)
    {
	/* Add this weapon to the ones protecting this hitlocation
        */
	obs = aid_hitloc[prot[il]];
	if (pointerp(obs))
	    obs = obs + ({ wep });
	else
	    obs = ({ wep });

	aid_hitloc[prot[il]] = obs;   
	adjust_ac(prot[il], wep, 0);  /* Fix one hitlocs acc ac */
    }

    /*
     * If more than two weapons wielded check the 2H combat, only with 2H
     * skill > 20 will it be profitable to wield 2 weapons.
     * Attack use range from 80 to 150
     */
    if (sizeof(cb_query_weapon(-1)) > 1)
    {
	extra = qme()->query_skill(SS_2H_COMBAT);
	extra = extra > 20 ? extra / 2 : extra - 20;
	this_object()->cb_set_attackuse(100 + extra);
    }

    return 1;
}

/*
 * Function name: cb_show_wielded
 * Description:   Describe the currently wielded weapons.
 * Argumensts:    ob: The object to give the description
 * Returns:       Description string.
 */
public string 
cb_show_wielded(object ob)
{
    mixed *a;
    int il;
    string str, p, pr;

    a = m_values(aid_attack) - ({ 0 });

    if (!sizeof(a))
	return "";
    
    if (ob != qme())
    {
	p = qme()->query_possessive();
	pr = capitalize(qme()->query_pronoun()) + " is";
    }
    else
    {
	p = "your";
	pr = "You are";
    }

    for (il = 0; il < sizeof(a); il++) 
    {
	if (objectp(a[il]))
	    a[il] = a[il]->query_wield_desc(p);
	else
	    a[il] = 0;
    }

    a = a - ({ 0 });					   

    if (sizeof(a) < 2)
	str = a[0];
    else
	str = implode(a[0 .. sizeof(a) - 2], ", ") + " and " + a[sizeof(a) - 1];

    return break_string(pr + " wielding " + str, 76) + ".\n";
}

/*
 * Function name: unwield
 * Description:   Unwield a weapon.
 * Arguments:	  wep - The weapon to unwield.
 * Returns:       None.
 */
public void
cb_unwield(object wep)
{
    int aid;

    aid = (int)wep->query_attack_id();

    /* Are we using it?
     */
    if (aid_attack[aid] != wep)
	return;

    /* Take it away from the wield subloc
    */
    if (environment(wep) == qme())
	wep->move(qme());

    aid_attack[aid] = 0;
    cb_remove_arm(wep); 
    
    me->cr_reset_attack(aid);

   /* If we wield no more than 1 weapon our % use should go back to 100 */

    if (sizeof(cb_query_weapon(-1)) < 2)
        this_object()->cb_set_attackuse(100);
}

static int empty_slot(object ob, object curw) { return (ob != curw); }
static int weapons_out(object ob) { return (ob && ob->check_armour()); }

/*
 * Function name: cb_query_weapon
 * Description:   Returns the weapon held in a specified location.
 *                Illegal location gives a list of all.
 * Arguments:	  which: A numeric label describing a weapon
 *                       location. On humanoids this is W_RIGHT etc.
 * Returns:       The corresponding weapon.
 */
public mixed
cb_query_weapon(int which)
{
    if (which >= 0)
    {
	if (member_array(which, m_indexes(aid_attack)) >= 0)
	    return aid_attack[which];
	else
	    return 0;
    }
    else
	return m_values(aid_attack) - ({0});
}

/*
 * Function name: adjust_ac
 * Description:   Adjust relevant hitlocations for a given armour slot
 *                when we wear an armour or remove an armour.
 * Arguments:	  hid:   The hitlocation id as given in /sys/wa_types.h
 *                arm:   The armour.
 *                rm:    True if we remove armour
 */
static void
adjust_ac(int hid, object arm, int rm)
{
    int il, ac, *am;
    mixed *oldloc;

    ac = (int)arm->query_ac(hid);
    am = (int*)arm->query_am(hid);

    for (il = 0; il < sizeof(am); il++) am[il] += ac; 
    
    oldloc = query_hitloc(hid);

    for (il = 0; il < sizeof(am) && il < sizeof(oldloc[0]); il++)
    {
	if (rm)
	    oldloc[0][il] -= am[il];
	else
	    oldloc[0][il] += am[il];
    }
    add_hitloc(oldloc[0], oldloc[1], oldloc[2], hid);
}

/*
 * Function name: cb_wear_arm
 * Description:   Wear an armour
 * Arguments:	  arm - The armour.
 * Returns:       True if worn, text if fail
 */
public mixed
cb_wear_arm(object arm)
{
    int *hid, il, *slots;
    object *obs;

    if (arm->query_prop(OBJ_I_IS_MAGIC_ARMOUR))
	hid = arm->query_shield_slots();
    else
	hid = arm->query_protects();   /* The hitlocations */
    slots = arm->query_slots();  /* The needed tool slot */

    /* Can we use this armour ? We must define all the hitlocs it protects
     */
    if (sizeof(hid) && (sizeof(query_hitloc_id() & hid) < sizeof(hid)))
	return "It seems not to fit your body very well.";

    /*
     * Are all the slots it needs free ?
     */
    for (il = 0; il < sizeof(slots); il++)
	if (tool_slots[slots[il]])
	    return "Something seems to be in the way.";

    if (arm->move(qme(), SUBLOC_WORNA))
	return "Not possible to wear for some reason.";

    for (il = 0; il < sizeof(slots); il++)
	tool_slots[slots[il]] = arm;

    for (il = 0; il < sizeof(hid); il++)
    {
	/* Add this armour to the ones protecting this hitlocation
        */
	obs = aid_hitloc[hid[il]];
	if (pointerp(obs))
	    obs = obs + ({ arm });
	else
	    obs = ({ arm });

	aid_hitloc[hid[il]] = obs;   
	adjust_ac(hid[il], arm, 0);  /* Fix one hitlocs acc ac */
    }
    return 1;
}

/*
 * Function name: cb_show_worn
 * Description:   Describe the currently worn armours
 * Arguments:     ob: The object that the description is for
 * Returns:       Description string.
 */
public string
cb_show_worn(object ob)
{
    mixed *a, *b, *c;
    string str, p;
    int il;

    b = m_values(aid_hitloc); /* All protecting objects */
    for (c = ({}), il = 0; il < sizeof(b); il++)
	c += pointerp(b[il]) ? b[il] : ({});

    c += m_values(tool_slots); /* All objects used */

    a = ({});
    while(sizeof(c))
    {
	a += ({ c[0] });
	c -= ({ c[0] });
    }

    a = filter(a, "weapons_out", this_object());
    a = filter(a, "filter_shown", FILTER_FILE);

    if (!sizeof(a))
	return ""; 

    if (ob != qme())
	p = capitalize(qme()->query_pronoun()) + " is";
    else
	p = "You are";

    str = (string)COMPOSITE_FILE->desc_dead(a);

    return break_string(p + " wearing " + str, 76) + ".\n";
}

/*
 * Function name: cb_remove_arm
 * Description:   Remove an armour
 * Arguments:	  arm - The armour.
 */
public void
cb_remove_arm(object arm) 
{
    int il, pos, *hids;
    mixed *a, *b;

    /* Are we wearing it ?
     */
    if (member_array(arm, m_values(tool_slots)) < 0)
    {
	a = m_values(aid_hitloc); /* All protecting objects */
	for (b = ({}), il = 0; il < sizeof(a); il++)
	    b += pointerp(a[il]) ? a[il] : ({});
	if (member_array(arm, b) < 0)
	    return 0;
    }

    if (environment(arm) == qme())
	arm->move(qme());
    
    tool_slots = filter(tool_slots, "empty_slot", this_object(), arm);

    hids = query_hitloc_id();

    /*
     * Remove the armours effect for each hit location it protects
     */
    for (il = 0; il < sizeof(hids); il++)
    {
	if (pointerp(aid_hitloc[hids[il]]) &&
	    (pos = member_array(arm, aid_hitloc[hids[il]])) >= 0)
	{
	    adjust_ac(hids[il], arm, 1);
	    if (!sizeof(aid_hitloc[hids[il]] = 
		exclude_array(aid_hitloc[hids[il]], pos, pos)))
	    {
		qme()->cr_reset_hitloc(hids[il]);
	    }
	}
    }
}

/*
 * Function name: cb_query_armour
 * Description:   Returns the armour of a given armour slot.
 *		  A list of all if illegal argument is given.
 * Arguments:	  which: A numeric label describing an armour
 *                       location. On humanoids this is TS_HEAD etc.
 * Returns:       The corresponding armour
 * BUGS/features:
 *         If an object can be both weapon and armour it will not show up
 *         here as an armour if wielded too.
 */
public mixed
cb_query_armour(int which)
{
    mixed *a, *b;
    int il;

    if (which >=0)
    {
	if (member_array(which, m_indexes(tool_slots)) >= 0)
	    return tool_slots[which];
	else
	    return 0;
    }

    a = m_values(aid_hitloc);
    for (b = ({}), il = 0; il < sizeof(a); il++)
	b += pointerp(a[il]) ? a[il] : ({});
    b += m_values(tool_slots);
    a = ({});
    while(sizeof(b))
    {
	a += ({ b[0] });
	b -= ({ b[0] });
    }

    return a - m_values(aid_attack) - ({0});
}

/*
 * Function name: cb_attack_desc
 * Description:   Gives the description of a certain attack slot.
 * Arguments:     aid:   The attack id
 * Returns:       string holding description on VBFC form. Do not use write()
 */
public string
cb_attack_desc(int aid)
{
    if (objectp(aid_attack[aid]))
    {
	return QSHORT(aid_attack[aid]);
    }
    else
    {
	return qme()->cr_attack_desc(aid);
    }
}

/*
 * Function name: cb_try_hit
 * Description:   Decide if a certain attack fails because of something
 *                related to the attack itself, ie specific weapon that only
 *		  works some of the time. 
 * Arguments:     aid:   The attack id
 * Returns:       True if hit, otherwise 0.
 */
public int
cb_try_hit(int aid)
{
    if (objectp(aid_attack[aid]))
    {
	return aid_attack[aid]->try_hit(cb_query_attack());
    }
    else
    {
	return (int)qme()->cr_try_hit(aid);
    }
}

/*
 * Function name: cb_did_hit
 * Description:   Tells us that we hit something. Should produce combat
 *                messages to all relevant parties. 
 * Arguments:     aid:   The attack id
 *                hdesc: The hitlocation description.
 *                phurt: The %hurt made on the enemy
 *                enemy: The enemy who got hit
 *		  dt:	 The current damagetype
 *		  phit:  The %success that we made with our weapon
 *		  dam:   The damamge made in hit points
 */
public varargs void
cb_did_hit(int aid, string hdesc, int phurt, object enemy, int dt,
	   int phit, int dam)
{
    if ((!enemy) || (!qme()))
	return;

    if (objectp(aid_attack[aid]) && 
	aid_attack[aid]->did_hit(aid, hdesc, phurt, enemy, dt, phit, dam))
    {
	/*
	 * Adjust our panic level
	 */
	if (phurt >= 0)
	    cb_add_panic(-3 - phurt / 5);
	else
	    cb_add_panic(1);
	return;
    }
    else
	::cb_did_hit(aid, hdesc, phurt, enemy, dt, phit, dam);
}

/*
 * Function name: cb_got_hit
 * Description:   Tells us that we got hit. It can be used to reduce the ac
 *                for a given hitlocation for each hit.
 * Arguments:     hid:   The hitloc id
 *                ph:    The %hurt
 *                att:   Attacker
 *		  aid:   The attack id
 *                dt:    The damagetype
 *		  dam:   The damage done in hitpoints
 */
public varargs void
cb_got_hit(int hid, int ph, object att, int aid, int dt, int dam)
{
    int il;
    object *arms;

    /* 
     * Many armours may help to cover the specific bodypart: hid
     */
    arms = aid_hitloc[hid];
    if (pointerp(arms))
    {
	for (il = 0; il < sizeof(arms); il++)
	    arms[il]->got_hit(hid, ph, att, aid, dt, dam);
    }
    qme()->cr_got_hit(hid, ph, att, aid, dt, dam);
}

/*
 * Function namn: cb_update_armour
 * Description:   Call this function if the ac of a shielding object has changed
 * Arguments:     obj - the object which ac has changed
 */
public void
cb_update_armour(object obj)
{
    int *index, i, j;
    object *arms;
    mixed *oldloc;

    if (!obj)
	return;

    index = m_indexes(aid_hitloc);
    for (i = 0; i < sizeof(index); i++)
    {
	arms = aid_hitloc[index[i]];
	if (pointerp(arms) && obj && (member_array(obj, arms) > - 1))
	{
	    me->cr_reset_hitloc(index[i]);
	    for (j = 0; j < sizeof(arms); j++)
		adjust_ac(index[i], arms[j], 0);
	}
    }
}
	    
/*
 * Function namn: cb_update_weapon
 * Description:   Call this function when something has caused the weapon
 *		  stats to change, skill raise or sharpening the weapon or so.
 * Arguments:     wep - The weapon
 */
public void
cb_update_weapon(object wep)
{
    if (!wep)
	return;

    add_attack(wep->query_hit(), wep->query_modified_pen(), wep->query_dt(),
		wep->query_procuse(), wep->query_attack_id(),
		me->query_skill(wep->query_wt()));
    cb_update_armour(wep);
}

/*
 * Function namn:
 * Description:
 * Arguments:
 * Returns:
 */

