/*
   /std/drink.c
   
   This is the standard object used for any form of drinkable stuff.

 Usage:

 	inherit "/std/drink";

	create_drink()
	{
	    set_soft_amount(amount_in_millilitres);
	    set_alco_amount(amount_in_millilitres);
	    set_name("name of drink");
	    set_short(...);
	    set_long(....);
	}

	An imperial pint is 540 millilitres.
*/
#pragma save_binary

inherit "/std/heap";

#include <cmdparse.h>
#include <composite.h>
#include <macros.h>
#include <stdproperties.h>

static	int	soft_amount,
		alco_amount;
static	object	*gFail;

/*
 * Prototypes
 */
void 	consume_them(object *arr);

public void
create_drink()
{
    set_name("drink");
}

public nomask void
create_heap()
{
    set_heap_size(1);

    create_drink();
}

public nomask void
reset_heap() { this_object()->reset_drink(); }

public void
set_long(string long)
{
    ::set_long(long);

    if (!query_prop(HEAP_S_UNIQUE_ID))
	add_prop(HEAP_S_UNIQUE_ID, long);
}

/*
 * Function name:	set_soft_amount
 * Description:		sets the amount of liquid in the drink
 * Arguments:		a: The amount of liquid
 */
public void
set_soft_amount(int a) { soft_amount = a; }

/*
 * Function name:	query_soft_amount
 * Description:		Gives the amount of liquid in the drink
 * Returns:		Amount as int
 */
public int
query_soft_amount() { return soft_amount; }

/*
 * Function name:	set_alco_amount
 * Description:		sets the amount of alcohol in the drink
 * Arguments:		a: The amount of alcohol
 */
public void
set_alco_amount(int a) { alco_amount = a; }

/*
 * Function name:	query_alco_amount
 * Description:		Gives the amount of alcohol in the drink
 * Returns:		Amount as int
 */
public int
query_alco_amount() { return alco_amount; }

void
init()
{
    add_action("drink_it", "drink");
}

/*
 * Function name: consume_text
 * Description:   Here the drink message is written. You may redefine it if
 *                you wish.
 * Arguments:     arr - Objects being consumed
 *                vb  - The verb player used to consume them
 */
void
consume_text(object *arr, string vb)
{
    string str;

    write("You " + vb + " " + (str = COMPOSITE_DEAD(arr)) + ".\n");
    say(QCTNAME(this_player()) + " " + vb + "s " + str + ".\n");
}

/*
 * Function name:	drink_it
 * Description:		Drinks the objects described as parameter to 'drink'.
 *			It uses command parsing to find which objects to drink.
 * Arguments:		str: The trailing command after 'drink ...'
 * Returns:		True if command successful
 */
public int
drink_it(string str)
{
    int	headache, i;
    object *a;
    string vb;
    
    if (this_player()->query_prop(TEMP_STDDRINK_CHECKED))
	return 0;

    gFail = ({});
    vb = query_verb(); 
    headache = (int) this_player()->query_headache();
    
    notify_fail(capitalize(vb) + " what?\n");  /* access failure */
    
    if (!str)
	return 0;
    a = CMDPARSE_ONE_ITEM(str, "drink_one_thing", "drink_access");
    if (sizeof(a) > 0) 
    {
	consume_text(a, vb);
	if (headache && (!this_player()->query_headache()))
	    write("Your headache disappears.\n");
	for (i = 0; i < sizeof(a); i++)
	    a[i]->destruct_object();

	return 1;
    }
    else
    {
	call_out("tp_remove_temp", 1, this_player());
	this_player()->add_prop(TEMP_STDDRINK_CHECKED, 1);
	if (sizeof(gFail))
	    notify_fail("@@drink_fail:" + file_name(this_object()));
	return 0;
    }
}

void
tp_remove_temp(object tp)
{
    tp->remove_prop(TEMP_STDDRINK_CHECKED);
    remove_call_out("tp_remove_temp");
}

string
drink_fail()
{
    string str;

    str = "You try to drink " + COMPOSITE_DEAD(gFail) + " but fail.\n";
    say(QCTNAME(this_player()) + " tries to drink " + QCOMPDEAD +
	     " but fails.\n");
    tp_remove_temp(this_player());
    return str;
}

int
drink_access(object ob)
{ 
    if ((environment(ob) == this_player()) &&
	(function_exists("create_heap", ob) == "/std/drink") &&
	(ob->short()))
	return 1;
    else
	return 0;
}

int
drink_one_thing(object ob)
{
    int am1, am2, pstuff, num, i;

    am1 = (int) ob->query_soft_amount();
    am2 = (int) ob->query_alco_amount();
    num = (int) ob->num_heap();

    for (i = 1; i <= num; i++)
    {
	if (!this_player()->drink_soft(am1))
	{
	    if (i == 1)
	    {
		ob->split_heap(1);
	    	write(capitalize(ob->short()) + " is too much for you.\n");
	    	gFail += ({ ob });
	    	return 0;
	    }
	    ob->split_heap(i - 1);
	    return 1;
	}
	if (!this_player()->drink_alco(am2))
	{
	    this_player()->drink_soft(-am1);
	    if (i == 1)
	    {
		ob->split_heap(1);
		write(capitalize(ob->short()) + " is too strong for you.\n");
	        gFail += ({ ob });
	        return 0;
	    }
	    ob->split_heap(i - 1);
	    return 1;
	}
    }
    return 1;
}

void
destruct_object()
{
    if (leave_behind > 0)
	set_heap_size(leave_behind);
    else
	call_out("remove_object", 1);
}

/*
 * Function name: stat_object
 * Description:   This function is called when a wizard wants to get more
 *                information about an object.
 * Returns:       str - The string to write..
 */
string
stat_object()
{
    string str;

    str = ::stat_object();

    str += "Soft: " + soft_amount + "\n";
    str += "Alco: " + alco_amount + "\n";

    return str;
}

