/*
 *  /std/living/combat.c
 *
 *  This is a subpart of living.c
 *  All internal combat routines are coded here.
 *
 *  This file is included into living.c
 *
 *  Ver 2.0 JnA: 911220
 *
 *  Most of the functionality is moved to the external combat object,
 *  see /std/combat/cbase
 *
 *  Revision history:
 *  23 Sept 1993: Mercade: Remove poisons if you die.
 */

#include "/sys/stdproperties.h"
#include "/sys/wa_types.h"
#include "/secure/std.h"

static int      time_to_heal;   /* Healing counter */
static int	quickness;	/* how quick are we? */

static string   whimpy_dir;	/* Direction to wimpy in if needed */

static object   my_leader,	/* Pointer to team leader if exist */
                *my_team,	/* Array of team members if leader */
		combat_extern;  /* The external combat object */

static mixed    leftover_list;  /* The list of leftovers */

#define CEX if (!combat_extern) combat_reload()
varargs public mixed query_leftover(string organ);
public int remove_leftover(string organ);

/*
 * Function name:   query_combat_file
 * Description:     Gives the name of the file to use for combat.
 * Returns:         The name
 */
public string
query_combat_file() { return COMBAT_FILE; }

/*
 * Function name:   query_combat_object
 * Description:     Gives the object that is currently acting as external
 *                  combat object
 * Returns:         the combat object
 */
public object
query_combat_object() { return combat_extern; }

static void
combat_reload()
{
    PROFILE("combat_reload");
    if (combat_extern)
	return;

    combat_extern = clone_object(query_combat_file());

    /*
     * We can define our own combat object, but it must inherit
     * the original.
     */
    if (function_exists("create_object", combat_extern) != COMBAT_FILE)
    {
	write("ILLEGAL COMBAT OBJECT: " +
	      function_exists("create_object", combat_extern) +
	      " should be: " + COMBAT_FILE + "\n");
	destruct();
    }

    combat_extern->cb_link(); /* Link me to the combat object */

    /* 
     * Configure this living object. For humanoids this includes adding
     * the hand attacks and head, torso etc hitlocations.
     */
    combat_extern->cb_configure(); 
}

/*
 * Function name:   combat_reset
 * Description:     Reset the combat functions of the living object.
 * Arguments:       arg: Reset argument.
 */
static nomask void
combat_reset()
{
    PROFILE("combat_reset");
    my_team = ({});
    add_subloc(SUBLOC_WORNA, this_object());
    add_subloc(SUBLOC_WIELD, this_object());
    combat_reload();
}

/*
 * Function name:   do_die
 * Description:     Called from enemy combat object when it thinks we died.
 * Arguments:       killer: The enemy that caused our death.
 */
public void
do_die(object killer)
{
    object corpse;

    PROFILE("do_die");
    /* Did I die ?
    */
    if (query_hp() > 0 || query_wiz_level())
        return;

    if (!killer)
	killer = previous_object();

    CEX; combat_extern->cb_death_occured(killer);

    killer->notify_you_killed_me(this_object());
    
    /*
     * Fix the corpse and possibly the ghost
     */
    if (this_object()->query_prop(LIVE_I_NO_CORPSE))
    {
	set_ghost(1);
	move_all_to(environment(this_object()));
    }
    else    if (!(corpse = (object) this_object()->make_corpse()))
    {
	corpse = clone_object("/std/corpse");
	corpse->set_name(query_name());
	corpse->change_prop(CONT_I_WEIGHT, query_prop(CONT_I_WEIGHT));
	corpse->change_prop(CONT_I_VOLUME, query_prop(CONT_I_VOLUME));
	corpse->add_prop(CORPSE_S_RACE, query_race_name());
	corpse->add_prop(CONT_I_TRANSP, 1);
	corpse->change_prop(CONT_I_MAX_WEIGHT,
			    query_prop(CONT_I_MAX_WEIGHT));
	corpse->change_prop(CONT_I_MAX_VOLUME,
			    query_prop(CONT_I_MAX_VOLUME));
	corpse->add_leftover(query_leftover());
        set_ghost(1);
        corpse->move(environment(this_object()), 1);
        move_all_to(corpse);
    }
    else
    {
    	set_ghost(1);
    	corpse->move(environment(this_object()), 1);
    	move_all_to(corpse);
    }
    
    if (!this_object()->second_life(killer))
    {
	this_object()->remove_object();
    }
    else
    {
	log_file("KILLS", ctime(time()) + " " + this_object()->query_name() +
		 " (" + this_object()->query_average_stat() + ") by " +
		 killer->query_name() + " (" + creator(killer) + ")\n", -1);
    }
}

/*
 * Function name:   move_all_to
 * Description:     Move the entire inventory of this_object to dest
 * Arguments:       dest: destination of the inventory
 */
static nomask void
move_all_to(object dest)
{
    object *oblist;
    int i, ret;

    PROFILE("move_all_to");
    oblist = all_inventory(this_object());
    if (oblist && sizeof(oblist) > 0)
    {
	for (i = sizeof(oblist) - 1; i >= 0; i--)
        {
            /* remove all poisons.. you are dead, so they wo not bother you
             * in your new body.
             */
            if (function_exists("create_object", oblist[i]) ==
                "/std/poison_effect")
            {
                oblist[i]->remove_object();
                continue;
            }
	    if (catch(ret = oblist[i]->move(dest)))
		log_file("DIE_ERR", ctime(time()) + " " +
		    this_object()->query_name() + " (" +
		    file_name(oblist[i]) + ")\n");
	    else if (ret)
		oblist[i]->move(environment(this_object()));
        }
    }
}


/*************************************************
 *
 * Whimpy routines
 *
 */

/*
 * Function name:   set_whimpy_dir
 * Description:     Sets the favourite direction of the whimpy escape routine
 * Arguments:       str: the direction string
 */
public void
set_whimpy_dir(string str)
{
    whimpy_dir = str;
}

/*
 * Function name:   query_whimpy_dir
 * Description:     Gives the current favourite whimpy escape direction
 * Returns:         The direction string
 */
public string
query_whimpy_dir()
{
    return whimpy_dir;
}


/*************************************************
 *
 * Team routines
 *
 */

/*
 * Function name:    set_leader
 * Description:      Sets this living as a member in a team
 *                   It will fail if this living is a leader itself
 * Arguments:	     leader: The objectpointer to the leader of the team
 * Returns:	     True if successfull
 */
public int
set_leader(object leader)
{
    if (sizeof(my_team))
	return 0;			/* We can't be both leader and led */

    my_leader = leader;

    return 1;
}

/*
 * Function name:   query_leader
 * Description:     Gives the object of the living who is the leader of the
 *                  team that we are in.
 * Returns:         The object with the leader, or 0 if we're not in a team
 */
public object
query_leader()
{
    return my_leader;
}

/*
 * Function name:   team_join
 * Description:     Sets this living as the leader of another
 *                  Fails if we have a leader, then we can't lead others.
 * Arguments:	    member: The objectpointer to the new member of my team
 * Returns:         True if member accepted in the team
 */
public int
team_join(object member)
{
    PROFILE("team_join");
    if (my_leader)
	return 0;

    if (!member->set_leader(this_object()))
	return 0;

    if (member_array(member, query_team()) >= 0)
	return 1;		/* Already member */
    if (!my_team)
	my_team = ({ member });
    else
	my_team = my_team + ({ member });
    return 1;
}

/*
 * Function name:   query_team
 * Description:     Gives an array with all objects of team members in it
 * Returns:         The array with team members
 */
public object *
query_team()
{
    int             i;
    PROFILE("query_team");
    if (my_team == 0)
	return ({});
    while ((i = member_array(0, my_team)) >= 0)
	my_team = exclude_array(my_team, i, i);
    return my_team;
}

/*
 * Function name:   team_leave
 * Description:     Removes this living as the leader of another
 * Arguments:	    member: The objectpointer to the member leaving my team
 */
public void 
team_leave(object member)
{
    int             a;

    PROFILE("team_leave");
    a = member_array(member, my_team);
    if (a >= 0)
    {
	my_team[a]->set_leader(0);
	my_team = exclude_array(my_team, a, a);
    }
}

/*
 * Function name:   query_team_others
 * Description:     Gives all members/leader that we are joined up with
 * Returns:         The array with all other members
 */
public /* object * */
query_team_others()
{
    PROFILE("query_team_others");
    if (my_leader)
	return (object*)my_leader->query_team() + 
	    ({ my_leader }) - ({ this_object() });
    else
	return query_team();
}


/************************************************************
 * 
 * Redirected functions to the external combat object
 *
 */

/*
 * Function name:   hit_me
 * Description:     Called to make damage on this object. The actually
 *		    made damage is returned and will be used to change
 *		    the score of the aggressor.
 * Arguments:	    wcpen 	- The wc-penetration
 *		    dt    	- damagetype, use MAGIC_DT if ac will not
 *				  help against this attack.
 *		    attacker 	- Object hurting us
 *		    attack_id 	- Special id saying what attack hit us. If you
 *		    		  have made a special attack, let the id be -1
 * Returns:         The hitresult as given by the external combat object.
 */
public mixed
hit_me(int wcpen, int dt, object attacker, int attack_id)
{
    mixed hres;
    int wi;
    PROFILE("hit_me");

    /*
     * Start nonplayers when attacked
     */
    start_heart();

    CEX;
    hres = (mixed)combat_extern->cb_hit_me(wcpen, dt, attacker, attack_id);

    if (!(wi = query_whimpy()))
	return hres;

    if (((100 * query_hp()) / query_max_hp()) < wi)
    {
	call_out("run_away", 1);
    }

    return hres;
}

/*
 * Function name:   attack_object
 * Description:     Start attacking, the actual attack is done in heart_beat
 * Arguments:	    The object to attack
 */
public void
attack_object(object ob)
{
    PROFILE("attack_object");
    /* 
     * For monsters, start the heart beat
     */
    start_heart();

    CEX; combat_extern->cb_attack(ob);
}

/*
 * Function name:   attacked_by
 * Description:     This routine is called when we are attacked.
 * Arguments:	    ob: The attacker
 */
public void
attacked_by(object ob)
{
    PROFILE("attacked_by");
    CEX; combat_extern->cb_attacked_by(ob);
}

/*
 * Function name:   combat_init
 * Description:     Notes when players are introduced into our environment
 *                  Used to attack known enemies on sight.
 */
static nomask void
combat_init()
{
    PROFILE("combat_init");
    /*
     * Is this_player() in list of known enemies ?
     * Use attacked_by() so that not forced to swap current enemy
     */
    CEX;

    /* Can't attack people you can't see */
    if (!CAN_SEE(this_player(), this_object()))
	return;

    if (member_array(this_player(), this_object()->query_enemy(-1)) >= 0)
	this_object()->attacked_by(this_player());

}

/*
 * Function name:   heal_living
 * Description:     Heals the living object
 * Arguments:	    num: Should be 0, otherwise >0 for lost heart_beats
 */
static nomask void
heal_living(int num)
{
    PROFILE("heal_living");
    time_to_heal += num + 1;
    CEX;
    time_to_heal = (int)combat_extern->cb_heal(time_to_heal);
}

/*
 * Function name: run_away
 * Description:   Runs away from the fight
 */
public void
run_away()
{
    object          here;
    int             i,
                    j;

    PROFILE("run_away");
    while (remove_call_out("run_away") != -1);  
    CEX; combat_extern->cb_run_away(whimpy_dir);
}

/*
 * Function name:   stop_fight
 * Description:     Stops the current fight
 */
public void
stop_fight(mixed elist) { CEX; combat_extern->cb_stop_fight(elist); }

/*
 * Function name:   query_enemy
 * Description:     Gives information of recorded enemies. If you want to
 *		    know currently fought enemy (if any) call query_attack()
 * Arguments:       arg: Enemy number (-1 == all enemies)
 * Returns:         Object pointer to the enemy
 */
public mixed
query_enemy(int arg) 
{
    PROFILE("query_enemy");
    CEX; return combat_extern->cb_query_enemy(arg); 
}

/*
 * Function name:   query_attack
 * Description:     Return the attacked object.
 * Returns:         The attacked object.
 */
public object
query_attack() { CEX; return (object)combat_extern->cb_query_attack(); }

/*******************************************
 *
 * Weapon and Armour routines. 
 *
 * These are merely registration routines for objects used in combat.
 * The actual management of their function is done in the external combat
 * object. The terminology of weapons, armours, wield and wear remain only
 * for backwards compatibility and confusion.
 */

/*
 * Function name:   wield
 * Description:     Wield a weapon.
 * Arguments:	    wep - The weapon to wield.
 * Returns:         True if wielded.
 */
public int
wield(object wep) { CEX; return (int) combat_extern->cb_wield_weapon(wep); }

/*
 * Function name:   unwield
 * Description:     Unwield a weapon.
 * Arguments:	    wep: The weapon to unwield.
 * Returns:         None.
 */
public void
unwield(object wep) { CEX; combat_extern->cb_unwield(wep); }

/*
 * Function name:   query_weapon
 * Description:     Returns the weapon held in a specified location.
 *		    A list of all if illegal argument is given.
 * Arguments:	    which: A numeric label describing a weapon
 *                         location. On humanoids this is W_RIGHT etc.
 * Returns:         The corresponding weapon.
 */
varargs public mixed
query_weapon(int which) { CEX; return combat_extern->cb_query_weapon(which); }

/*
 * Function name: update_weapon
 * Description:   Call this function if the stats or skills of a weapon has
 *		  changed.
 * Arguments:     wep - the weapon 
 */
public void
update_weapon(object wep) { CEX; combat_extern->cb_update_weapon(wep); }

/*
 * Function name:   wear_arm
 * Description:     Wear an armour
 * Arguments:	    arm: The armour.
 */
public int
wear_arm(object arm) { return (int) combat_extern->cb_wear_arm(arm); }

/*
 * Function name:   remove_arm
 * Description:     Remove an armour
 * Arguments:	    arm - The armour.
 */
public void
remove_arm(object arm) { CEX; combat_extern->cb_remove_arm(arm); }

/*
 * Function name:   query_armour
 * Description:     Returns the armour of a given position.
 *		    A list of all if no argument is given.
 * Arguments:	    which: A numeric label describing an armour
 *                         location. On humanoids this is TS_HEAD etc.
 * Returns:         The corresponding armour
 */
varargs public mixed
query_armour(int which) 
{
    CEX; return (int)combat_extern->cb_query_armour(which); 
}

/*
 * Function name: update_armour
 * Description:   Call this function when the ac of an armour has changed
 * Arguments:     arm - the armour
 */
public void
update_armour(object arm) { CEX; combat_extern->cb_update_armour(arm); }

/*
 * Function name:   adjust_combat_on_move
 * Description:     Called to let movement affect the ongoing fight. This
 *                  is used to print hunting messages.
 * Arguments:	    True if leaving else arriving
 */
public void
adjust_combat_on_move(int leave)
{
    CEX; combat_extern->cb_adjust_combat_on_move(leave);
}

/*
 * Function name:   adjust_combat_on_intox
 * Description:     Called to let intoxication affect combat. This
 *                  is used to do nasty drunk type things *laugh*
 * Arguments:       pintox: %intoxicated      
 */
public void
adjust_combat_on_intox(int pintox)
{
    CEX; combat_extern->cb_adjust_combat_on_intox(pintox);
}

/*
 * Function name:   add_panic
 * Description:     Adjust the panic level.
 * Arguments:       dpan: The panic increase/decrease
 */
public void
add_panic(int dpan) { CEX; combat_extern->cb_add_panic(dpan); }

/*
 * Function name:   query_panic
 * Description:     Give panic value
 * Returns:         The panic value
 */
public int
query_panic() { CEX; return (int)combat_extern->cb_query_panic(); }

/*
 * Function name:   combat_status
 * Description:     Let the combat object describe the combat status
 * Returns:         Description as string
 */
public string
combat_status() 
{
    CEX; return combat_extern->cb_status(); 
}

/*
 * Function name:   combat_data
 * Description:	    Let the combat object describe some combat data
 * Returns:	    Description as string
 */
public string
combat_data()
{
    CEX; return combat_extern->cb_data();
}

/*
 * Function name:   tell_watcher
 * Description:     Send a string to people who wants to see fights
 * Arguments:	    str   - The string to send
 * 		    enemy - The enemy we fought
 *		    arr   - Array of objects not to send this message to
 *			    If not used, message sent to all spectators who
 *			    wants to see blood.
 */
varargs void
tell_watcher(string str, object enemy, mixed arr)
{
    CEX; combat_extern->tell_watcher(str, enemy, arr);
}

/*
 * Function name: add_leftover
 * Description:   Add leftovers to the body.
 * Arguments:	  obj - The path to the object to leave behind.
 *		  organ - The actual leftover name
 *		  nitems - Number of leftovers of this kind. (-1 = infinite)
 *		  vbfc - VBFC to check.
 *		  hard - Hard remains (left after total decay).
 *		  cut - If this has to be "cut" loose or if "tear" is enough.
 */
varargs public void
add_leftover(string obj, string organ, int nitems, string vbfc,
	     int hard, int cut)
{
    PROFILE("add_leftover");
    if (!sizeof(leftover_list))
	leftover_list = ({ });

    remove_leftover(organ);

    leftover_list += ({ ({ obj, organ, nitems, vbfc, hard, cut }) });
}

/*
 * Function name: query_leftover
 * Description:   Return the leftover list. If an organ is specified, that
 *		  actual entry is looked for, otherwise, return the entire
 *		  list.
 *		  The returned list contains the following entries:
 *		  ({ objpath, organ, nitems, vbfc, hard })
 * Arguments:	  organ - The organ to search for.
 */
varargs public mixed
query_leftover(string organ)
{
    int i;

    PROFILE("query_leftover");
    if (!sizeof(leftover_list))
	return ({ });

    if (!strlen(organ))
	return leftover_list;

    for (i = 0 ; i < sizeof(leftover_list) ; i++)
	if (leftover_list[i][1] == organ)
	    return leftover_list[i];
}

/*
 * Function name: remove_leftover
 * Description:   Remove a leftover entry from a body.
 * Arguments:	  organ - Which entry to remove.
 * Returns:       1 - Ok, removed, 0 - Not found.
 */
public int
remove_leftover(string organ)
{
    int i;

    PROFILE("remove_leftover");
    if (!sizeof(leftover_list))
	return 0;

    for (i = 0 ; i < sizeof(leftover_list) ; i++)
	if (leftover_list[i][1] == organ)
	    leftover_list = leftover_list[0..(i - 1)] +
		leftover_list[(i + 1)..(sizeof(leftover_list))];
}
