/*
    /std/living/description.c

    This is a subpart of living.c
    All description relevant routines are defined here.

    This file is included into living.c

    NOTE
      There is some calls of the type: this_object()->function()
      in a number of places. The reason for this is to allow those
      functions to be shadowed as internal function calls are not
      possible to shadow.
*/

#include <wa_types.h> 		/* Sublocations for used weapons / armour */
#include <stdproperties.h>
#include <macros.h>
#include <formulas.h>
#include <state_desc.h>

#ifdef DAY_AND_NIGHT
#include "/sys/mudtime.h"
#endif

#ifdef STATUE_WHEN_LINKDEAD
#include "/sys/language.h"

static int is_linkdead;

set_linkdead(int i)
{
  is_linkdead = i;
}

query_linkdead()
{
  return is_linkdead;
}
#endif

static int appearance_offset;

/*
 * Function name:   query_met
 * Description:     Tells if we know a certain living's name.
 * Arguments:       name: Name of living or objectp of living
 * Returns:         True if we know this name otherwise false.
 */
public int
query_met(mixed name)
{
    return 1;  /* All NPC's know everybody */
}

/*
 * Function name:   notmet_me
 * Description:     Finds out if obj is considered to have met me. Players
 *                  must have been introduced to me. All others don't have to
 *                  be introduced to know me.
 * Arguments:       obj: Object in question, has it met me?
 * Returns:         True if object has not met me.
 */
public int
notmet_me(object obj)
{
    if (obj && query_ip_number(obj))
	return !obj->query_met(this_object());
    return !this_player()->query_met(this_object());
}

/*
 * Function name:   query_real_name
 * Description:     Returns the lowercase name of this living.
 *                  E.g.: "fatty".
 * Returns:         The name
 */
public string
query_real_name()
{
    return lower_case(::query_name());
}

/*
 * Function name:   query_name
 * Description:     Returns the capitalized name of this living.
 *                  E.g.: "Fatty".
 * Returns:         The name
 */
public string
query_name()
{
    return capitalize(::query_name());
}

/*
 * Function name:   query_met_name
 * Description:     Returns the name of this living, or "ghost of name" if
 *                  this living is not quite so living. E.g.: "Fatty".
 * Returns:         The name
 */
public string 
query_met_name()
{    
    if (this_object()->query_ghost() & 1)
	return "ghost of " + query_name();
    else
	return query_name();
}

/*
* Function name:   query_nonmet_name
* Description:     Gives the name of this living to players who have not met
*                  this living. E.g.: "big fat gnome wizard"
* Returns:         The name of the living when unmet.
*/
public string
query_nonmet_name()
{
    string *adj, str;

    if (sizeof((adj = this_object()->query_adj(1))) > 0)
	str = implode(adj, " ") + " " +
	    this_object()->query_race_name() +
	    (query_wiz_level() ? (" " + LD_WIZARD) : "");
    else
	str = (string)this_object()->query_race_name() +
	    (query_wiz_level() ? (" " + LD_WIZARD) : "");

    if (query_ghost() & 1)
	str += " " + LD_GHOST;

    return str;
}

/*
 * Function name:   query_Met_name
 * Description:     Returns the capitalized name of this living, prepended
 *                  with "Ghost of" if the living is not that living at all.
 *                  E.g.: "Ghost of Fatty".
 * Returns:         The capitalized name of the living when met.
 */
public string 
query_Met_name() 
{
    return capitalize(query_met_name());
}

/*
 * Function name:   query_art_name
 * Description:	    Gives the name with a prefix article when the object
 *                  has not met this living.
 *                  E.g.:
 *                        when met:   "Fatty",
 *                        when unmet: "a big fat gnome wizard".
 * Arguments:       pobj: The object that wants to know the name.
 *                  If pobj is undefined, it is assumed that the call has
 *                  been done through a protected vbfc
 * Returns:         Name prefixed with article.
 */
public varargs string 
query_art_name(object pobj)
{
    string pre, aft;

    pre = ""; aft = "";
    if (!objectp(pobj))
	pobj = previous_object(-1);
    if (!CAN_SEE(pobj, this_object()) || !CAN_SEE_IN_ROOM(pobj))
	return LD_SOMEONE;
    if (this_object()->query_prop(OBJ_I_HIDE))
    {
	pre = "[";
	aft = "]";
    }
    if (this_object()->query_prop(OBJ_I_INVIS))
    {
	pre = "(";
	aft = ")";
    }

    if (notmet_me(pobj))
	return pre +  LANG_ADDART(query_nonmet_name()) + aft;
    else
	return pre + query_met_name() + aft;
}

/*
 * Function name:   query_Art_name
 * Description:	    Gives the name with a capitalized prefix article when the
 *                  calling object has not met this living.
 *                  E.g.:
 *                        when met:   "Fatty",
 *                        when unmet: "A big fat gnome wizard".
 * Arguments:       pobj: The object that wants to know the name.
 *                  If pobj is undefined, it is assumed that the call has
 *                  been done through a protected vbfc
 * Returns:         Name prefixed with capitalized article.
 */
public varargs string
query_Art_name(object pobj) 
{
    return capitalize(query_art_name(pobj));
}

/*
 * Function name:   query_the_name
 * Description:	    Gives the name preceded by "the" when the object
 *                  has not met this living.
 *                  E.g.:
 *                        when met:   "Fatty",
 *                        when unmet: "the big fat gnome wizard".
 * Arguments:       pobj: The object that wants to know the name.
 *                  If pobj is undefined, it is assumed that the call has
 *                  been done through a protected vbfc
 * Returns:         Name prefixed with "the".
 */
public varargs string
query_the_name(object pobj)
{
    string pre = "", aft = "";

    if (!objectp(pobj))
	pobj = previous_object(-1);

    if (!CAN_SEE(pobj, this_object()) || !CAN_SEE_IN_ROOM(pobj))
	return LD_SOMEONE;
    if (query_prop(OBJ_I_HIDE))
    {
	pre = "[";
	aft = "]";
    }
    if (query_prop(OBJ_I_INVIS))
    {
	pre = "(";
	aft = ")";
    }

    if (notmet_me(pobj))
	return pre + LD_THE + " " + query_nonmet_name() + aft;
    else
	return pre + query_met_name() + aft;
}

/*
 * Function name:   query_The_name
 * Description:	    Gives the name preceded by "The" when the calling object
 *                  has not met this living.
 *                  E.g.:
 *                        when met:   "Fatty",
 *                        when unmet: "The big fat gnome wizard".
 * Argument:        pobj: The object that wants to know the name.
 *                  If pobj is undefined, it is assumed that the call has
 *                  been done through a protected vbfc
 * Returns:         Name prefixed with "The".
 */
public varargs string
query_The_name(object pobj) 
{
    return capitalize(query_the_name(pobj));
}

/*
 * Function name:   query_exp_title
 * Description:	    Returns "wizard" if this living is a wizard, or else
 *                  tries to calculate a title from the stats
 * Returns:         The title.
 */
public string
query_exp_title()
{
    int a,i,s;
    string *t;

    if (query_wiz_level()) 
	return LD_WIZARD;
    
    a = this_object()->query_average_stat();
    if (stringp(SD_AV_TITLES))
	t = explode(SD_AV_TITLES + " "," ");
    else
	t = SD_AV_TITLES;

    s = sizeof(t);
    
    for (i = 0; i < s; i++) 
    {
	if (a < ((100 * i) / s)) 
	    return t[i];
    }
    return t[s - 1];
}

/*
 * Function:    query_presentation
 * Description: Gives a presentation of the living in one line. Including
 *              Name, Race, Guild titles, Alignment and Experience level
 *              This should only be displayed to met players.
 *              E.g.: "Fatty the donut-fan, wizard, male gnome (eating)"
 * Returns:     The presentation string
 */
public string
query_presentation()
{
    string a, b, c;
    
    a = this_object()->query_title(); 
    b = this_object()->query_exp_title(); 
#ifndef NO_ALIGN_TITLE
    c = this_object()->query_al_title();
#endif

    return query_name() + 
	(strlen(a) ? (" " + a + ",") : "") +
	(strlen(b) ? (" " + b + ",") : "") + " " +
	this_object()->query_gender_string() + " " + 
        this_object()->query_race_name() + 
	(strlen(c) ? (" (" + c + ")") : "");
}

public string
query_format_presentation()
{
    string a, b, c;
    
    a = this_object()->query_title(); 
    b = this_object()->query_exp_title(); 
#ifndef NO_ALIGN_TITLE
    c = this_object()->query_al_title();
#endif

    return SECURITY->query_wiz_pretitle(this_object()) + " " +
	query_name() + "\n        " +
	(strlen(a) ? (a + ",") : "") +"\n        " +
	(strlen(b) ? (b + ",") : "") + " " +
	(string)this_object()->query_gender_string() + " " + 
        (string)this_object()->query_race_name() + 
	(strlen(c) ? (" (" + c + ")") : "");
}

/*
 * Function name:   short
 * Description:     Returns the short-description of this living, for the
 *                  object given. Handles invisibility and met/nonmet.
 * Returns:         The short string.
 */
public varargs string 
short(object for_obj)
{
    string desc;

    /*
     * If a specific short is set with set_short, use that description
     */
    if (strlen(query_short()))
	return ::short(for_obj);

    if (!for_obj)
	for_obj = this_player();

    if (notmet_me(for_obj))
    {
	desc = this_object()->query_nonmet_name();
#ifdef STATUE_WHEN_LINKDEAD
	if (!this_object()->query_npc() && !query_ip_number(this_object()))
	{
	    set_linkdead(1);
	    desc = STATUE_DESC + " " + LANG_ADDART(desc);
	}
#endif
    }
    else
    {
	desc = this_object()->query_met_name();
#ifdef STATUE_WHEN_LINKDEAD
	if (!this_object()->query_npc() && !query_ip_number(this_object()))
	{
	    set_linkdead(1);
	    desc = STATUE_DESC + " " + desc;
	}
#endif
    }
    if (query_prop(LIVE_S_EXTRA_SHORT))
	desc = desc + query_prop(LIVE_S_EXTRA_SHORT);

    return desc;
}

/*
 * Function name: vbfc_short
 * Description:   Gives short as seen by previous object
 * Returns:	  string holding short()
 */
public string
vbfc_short()
{
    object for_obj;

    for_obj = previous_object(-1);
    if (!this_object()->check_seen(for_obj))
	return LD_SOMEONE;

    return this_object()->short(for_obj);
}

/*
 * Function name:   show_scar
 * Description:     Finds out which scars this living has.
 * Returns:         A string with the scars.
 */
string
show_scar(mixed for_obj)
{
    int num;

    if (!objectp(for_obj))
	for_obj = this_player();

    num = this_object()->num_scar();

    if (num > 0)
    {
  	if (for_obj == this_object())
	    return 
		break_string(LD_YOUR_SCARS(num, this_object()->desc_scar()) +
				".\n", 75);

	else
	    return 
		break_string(capitalize(this_object()->query_pronoun()) +
			     LD_HAS_SCARS(num) + " " +
			     this_object()->query_possessive() + " " +
			     this_object()->desc_scar() + ".\n", 75);
    }

    return "";
}

/*
 * Function name:   long
 * Description:     Returns the long-description of this living, and shows also
 *                  the inventory. Handles invisibility and met/nonmet. Note
 *                  that the function does not do any writes! It only returns
 *                  the correct string.
 * Arguments:       string for_obj: return an item-description
 *                  object for_obj: return the living-description
 * Returns:         The description string
 */
public varargs string 
long(mixed for_obj)
{
    string          cap_pronoun, res;
    object 	    eob;
    
    if (stringp(for_obj))   /* Items */
	return ::long(for_obj);

    if (!objectp(for_obj))
	for_obj = this_player();

    if (for_obj == this_object())
	res = LD_PRESENT_YOU(this_object());
    else
    {		
	if (!strlen(res = query_long()))
	{
	    cap_pronoun = capitalize((string)this_object()->query_pronoun());
	    if (!notmet_me(for_obj))
	    {
		res = LD_PRESENT_TO(this_object());
	    }
	    else if (!(this_object()->short(for_obj)))
		return "";
	    else
		res = cap_pronoun + " is " +
			LANG_ADDART(this_object()->short(for_obj)) + ".\n";
    
	    if (this_object()->query_ghost() & 1) 
	    {
		if (notmet_me(for_obj))
		    return LD_NONMET_GHOST(this_object());
		else
		    return LD_MET_GHOST(this_object());
	    }
	}
	else
	{
	    res = check_call(res);
	}
    }
    res += this_object()->show_scar(for_obj);

    return res + this_object()->show_sublocs(for_obj);
}    

    
/*
 * Function name:   do_glance
 * Description:     This is the routine describing rooms to livings
 * Arguments:       brief: if true, do write the short-description,
 *                  else write the long-description.
 * Returns:         always 1.
 */
public int
do_glance(int brief)
{
    object          ob,
                    *ob_list,
                    *lv,
                    *dd;
    string          tmp,
                    item,
                    it,
                    *ctants;
    int             max;

    if (query_wiz_level())
    {
	tmp = file_name(environment());
	if (strlen(environment()->query_prop(OBJ_S_WIZINFO)))
	    tmp = "Wizinfo " + tmp;

	tell_object(this_object(), tmp + "\n");
    }

    /*
     * It is dark.
     */
    if (!CAN_SEE_IN_ROOM(this_object()))
    {
	write(LD_DARK_LONG);
	return 1;
    }

    /*
     * Describe the room and its contents.
     */
    ob = environment(this_object());

    if (!ob->check_seen(this_object()))
	write(LD_CANT_SEE);
    else
    {
#ifdef DAY_AND_NIGHT
        if (!ob->query_prop(ROOM_I_INSIDE) && 
	    (HOUR>21 || HOUR<5) && (ob->query_prop(OBJ_I_LIGHT) +
		 this_object()->query_prop(LIVE_I_SEE_DARK)) < 2)
	{
            write(LD_IS_NIGHT(ob));
        }
	else
#endif
	if (brief)
  	{
	    write(ob->short() + ".\n");
	    if (!ob->query_noshow_obvious())
		write(ob->exits_description());
	} else
	    write(ob->long());
    }

    ob_list = all_inventory(environment(this_object()));
    lv = filter(ob_list, "filter_other_living", FILTER_FILE);
    dd = filter(ob_list, "filter_dead", FILTER_FILE);
    dd = filter(dd, "filter_shown", FILTER_FILE);

    item = (string) COMPOSITE_FILE->desc_dead(dd, 1);
    if (item)
	write(break_string(capitalize(item) + ".", 76) + "\n");

    item = (string) COMPOSITE_FILE->desc_live(lv, 1);
    if (item)
	write(break_string(capitalize(item) + ".", 76) + "\n");

    /*
     * Describe possible combats going on.
     */
    for (max = 0; max < sizeof(lv); max++)
    {
	ctants = map(lv[max]->query_enemy(-1), "fights", 
		    this_object(), lv[max]);
	if (sizeof(ctants))
	    ctants -= ({0});

	if (sizeof(ctants) == 1)
	    item = LD_FIGHT1(ctants[0]);
	else if (sizeof(ctants) > 1)
	    item = LD_FIGHT_MANY(ctants);
	else
	    item = 0;
	if (item)
	    write(LD_FIGHT_DESC(item, lv[max]));

    }
    return 1;
}

/*
 * Function name:   fights
 * Description:	    Find out who fights who.
 * Arguments:       me, enemy: The fighting folks
 * Returns:         "you" if the enemy is fighting me, or else his name
 */
static mixed
fights(object me, object enemy)
{
    if (!objectp(me))
	return 0;

    if (me->query_enemy(0) == enemy)
    {
	if (me == this_object())
	    return "you";
	else
	    return me->query_the_name(this_object());
    }
    else
	return 0;
}


void
set_appearance_offset(int off)
{
    appearance_offset = off;
}

int
query_appearance_offset()
{
    return appearance_offset;
}

/*
 * Function name: my_opinion
 * Description:   Return my opinion about the appearance of an other
 *                living creature. -1 if not living, 0 - 100 if living.
 *                0 = perfectly in taste, 100 = the most ugly you can imagine
 * Arguments:     ob: the object to give an opinion about
 * Returns:       The opinion as int.
 */
public int
my_opinion(object ob)
{
    int op, op_offset;

    if (!living(ob))
	return -1;
    op = query_opinion() - ob->query_appearance();
    op = op > 0 ? op : - op;
    op = op < 50 ? op : 100 - op;

    op_offset = ob->query_appearance_offset();
    if (op_offset)
	op += op_offset;
    else
    {
	if (query_race_name() != ob->query_race_name())
	    op += 10;
     }
    if (query_race_name() == ob->query_race_name() &&
	query_gender() != ob->query_gender())
	op -= 5;
    if (query_intoxicated() > 0)
	op -= query_intoxicated() / 4;
    op += query_headache();
    op *= 2;

    return (op < 0 ? 0 : (op > 99 ? 99 : op));
}

/*
 * Function name: appraise_object
 * Description:   This function is called when someone tries to appraise a 
 *		  living object.
 * Arguments:     num - use this number instead of the skill.
 */
public void
appraise_object(int num)
{
    write("\n" + this_object()->long(this_player()) + "\n");
    write(break_string(LD_APPRAISE(appraise_weight(num), appraise_volume(num)),
	75) + "\n");
}

/*
 * Function name: query_align_text
 * Description:	  Returns the alignment text for this object.
 * Returns:	  The alignment text.
 */
string
query_align_text()
{
    int a, prc;
    string t, *names;

    a = this_object()->query_alignment();

    if (a < 0)
    {
	prc = (100 * (-a)) / (F_KILL_NEUTRAL_ALIGNMENT * 100);
	names = SD_EVIL_ALIGN;
    }
    else
    {
	prc = (100 * a) / (F_KILL_NEUTRAL_ALIGNMENT * 100);
	names = SD_GOOD_ALIGN;
    }

    a = (sizeof(names) * prc) / 100;    

    if (a >= sizeof(names))
	a = sizeof(names) - 1;

    return names[a];
}

