/*
  /std/mobile.c

  This is the base for all nonplayer living objects. This file holds the
  code that is common to both non humanoid creatures and humanoid npc's.

*/
#pragma save_binary

inherit "/std/living";
inherit "/std/act/seqaction" ;
inherit "/std/act/trigaction";

#include <ss_types.h>
#include <stdproperties.h>
#include <macros.h>
#include <std.h>

private	object	mobile_link;	       /* The linked master if controlled */
private int     mobile_exp_factor;     /* The xp modifier to give to killer */
private	int	no_accept;	       /* Flag when mobile stops accepting */

/*
 * Prototypes
 */
varargs void default_config_mobile(int lvl);

void
create_mobile()
{
    /* 
     * Observe that we do not continue the 'default' calls here
     * All default values are already set.
     */
    set_name("mobile");
}

nomask void
create_living()
{
    add_prop(CONT_I_HEIGHT, 160); /* Default height for monsters, 160 cm */
    add_prop(NPC_M_NO_ACCEPT_GIVE, "@@mobile_deny_objects");
    mobile_exp_factor = 100;
    this_object()->seq_reset();
    default_config_mobile();
    create_mobile();
    stats_to_acc_exp();
}

void
reset_mobile()
{
    ::reset_living();
}

nomask void
reset_living() { reset_mobile(); }

/*
 * Function name: team_join
 * Description:   Sets this living as the leader of another
 * Arguments:	  member: The objectpointer to the new member of my team
 * Returns:       True if member accepted in the team
 */
int
team_join(object member)
{
    if (query_ip_number(member))
	return 0;		/* mobile leaders overplayers */
    return ::team_join(member);
}

static int   
test_live_here(object ob)
{
    return (living(ob) && interactive(ob) && (ob != this_object()));
}

/*
 * Function name: test_if_any_here
 * Description:   Turn of heart_beat if we are alone.
 * Returns:
 */
void
test_if_any_here()
{
    if (environment(this_object()) &&
	(!sizeof(filter(all_inventory(environment(this_object())), 
			      "test_live_here", this_object()))))
	stop_heart();
    else
	call_out("test_if_any_here", 50);
}

/* 
 * start_heart
 * Description:  When a mobile has an active heartbeat we must test
 *		 now and then if we can turn it off.
 */
static void
start_heart()
{
    ::start_heart();
    remove_call_out("test_if_any_here");
    call_out("test_if_any_here", 50);
}

public string
query_race_name()
{
    mixed str;

    str = ::query_race_name();

    if (!str)
	return (string) DEFAULT_PLAYER->query_race();
    else 
	return str;
}

/*
 * Function name:  default_config_mobile
 * Description:    Sets all neccessary values for this mobile to function
 */
varargs void
default_config_mobile(int lvl)
{
    int i;

    for (i = 0 ; i < SS_NO_STATS - 3 ; i++)
	set_base_stat(i, (lvl ? lvl : 5));

    set_hp(500 * 500); /* Will adjust for CON above */

    stats_to_acc_exp();

    add_prop(CONT_I_WEIGHT, 70 * 1000);
    add_prop(CONT_I_VOLUME, 70 * 1000);

    mobile_exp_factor = 100;
    set_fatigue(query_max_fatigue());
}

/*
 * Function name: set_link_remote
 * Description:   Links a player to the output of the mobile
 * Arguments:	  player: Player to link mobile to
 */
void
set_link_remote(object player)
{
    object control;

    if (!player)
	player = this_player();

    if (!living(player) || mobile_link)
	return;

    control = clone_object("/std/remote_npc"); /* Get the remote control */
    control->set_npc(this_object());
    mobile_link = control;
    control->move(player);
}

/*
 * Description: Return the pointer to the current controller
 */
query_link_remote() { return mobile_link; }

/*
 *  Description: For monster link purposes
 */
void
catch_tell(string str)
{
    if (mobile_link)
	mobile_link->link_intext(str);
    ::catch_tell(str);
}

/*
 * Function name: catch_msg
 * Description:   This function is called for every normal message sent
 *                to this living.
 * Arguments:     str:       Message to tell the living
 *                from_player: The object that generated the message
 *			     This is only valid if the message is on the
 *			     form ({ "met message", "unmet message",
 *				     "unseen message" })
 */
public void 
catch_msg(mixed str, object from_player)
{
    if(!query_ip_number(this_object()) && !query_tell_active())
	return;

    if (pointerp(str))
    {
	if (!from_player)
	    from_player = this_player();
	if ((sizeof(str) > 2) && (!CAN_SEE_IN_ROOM(this_object()) ||
		!CAN_SEE(this_object(), from_player)))
	    catch_tell(str[2]);
	else if (this_object()->query_met(from_player))
	    catch_tell(str[0]);
	else 
	    catch_tell(str[1]);
    }
    else
	catch_tell(process_string(str, 1));
}

/*
 * Function name:  set_stats
 * Description:    Set the basic stats (see /sys/ss_types.h)
 */
public void
set_stats(int *stats)
{
    int il, max;

    max = (sizeof(stats) > SS_NO_STATS ? SS_NO_STATS : sizeof(stats));

    for (il = 0; il < max; il++)
	set_base_stat(il, stats[il]);
}


/*
 * Function name:  set_exp_factor
 * Description:    Set the xp factor (50% - 150%). This is the amount of
 *		   the base xp to give to the killer.
 *		   Base xp is:   3 * averagestat^2
 */
public void
set_exp_factor(int proc_xp)
{
    if (proc_xp > 49 && proc_xp < 151)
	mobile_exp_factor = proc_xp;
}

/*
 * Function name:  query_exp_factor
 * Description:    Give the xp factor (50% - 150%). This is the amount of
 *		   the base xp to give to the killer.
 *		   Base xp is:   3 * averagestat^2
 */
public int
query_exp_factor() { return mobile_exp_factor; }
   
/*
 * Function name: init_living
 * Description:   A patch for the automatic attack if this mobile can do that
 */
public void
init_living() { this_object()->init_attack(); }

/*
 * Function name: special_attack
 * Description:   Called from the external combat object's heart_beat
 *                routine. By redefining this, monsters can easily attack
 *                with spells or take special actions when engaged
 *                in combat.
 * Arguments:     victim (the one we are fighting right now)
 * Returns:       0 - if we want the round to continue
 *                1 - if we are done with this round
 * Note:          By always returning 1 the mobile is unable
 *                to do an ordinary attack.
 */
public int
special_attack(object victim)
{
    return 0;
}
 
/*
 * Function name: mobile_deny_objects
 * Description:   This function is called from VBFC and NPC_M_NO_ACCEPT_GIVE
 *		  prop checking. If this mobile is intelligent enough he might
 *		  recognize that someone is trying to load him down before
 *		  attacking and then he doesn't accept any more objects. This
 *		  is merely default behaviour. Feel free to code something
 *		  different
 * Returns:       A message that will be printed to the player or 0
 */
mixed
mobile_deny_objects()
{
    string str;

    if (no_accept || random(100) < query_stat(SS_INT))
    {
	str = " doesn't accept any gifts from you.\n";
	no_accept = 1;
	return str;
    }
    return 0;
}

/*
 * Function name: refresh_mobile
 * Description:   Mobiles not configured with function default_config_mobile
 *		  might come out not fully healed and with little mana and
 *		  tired. Call three functions and that's fixed, or you the
 *		  lazy can call this function. 
 */
void
refresh_mobile()
{
    heal_hp(query_max_hp());
    add_mana(query_max_mana());
    add_fatigue(query_max_fatigue());
}

