/*
 * possob.c
 *
 * This object is the object that controls the release mechanism during
 * possessions as well as the possessed player in case of player possessions.
 */
#pragma save_binary

#include <macros.h>
#include <stdproperties.h>

inherit "/std/object";

object demon, possessed;
int is_player;

/*
 * Function name: create_object
 * Description:	  Initialize the object
 */
create_object()
{
    set_name("possob");
    set_short("possessed player");
    set_long("This is the sadly abused soul of a possessed player. " +
	"However, you shouldn't be\n" + 
	"able to see this, so something bad must've happened. Contact " +
	"an Arch or Keeper\n" +
	"immediately!\n");
    set_no_show();
    is_player = 0;
}

/*
 * Function name: init
 * Description:	  Initialize some commands.
 */
init()
{
    if (environment(this_object()) != this_player())
	return;

    add_action("release", "quit");
    add_action("think", "think");
}

/*
 * Function name: release
 * Description:	  Stop possessing.
 */
int
release()
{
    /*
     * Someone called directly. Baaaaaaaad.
     */
    if (this_interactive() != possessed && previous_object() != possessed)
	return 0;

    /*
     * Get rid of the snoop.
     */
    if (is_player)
	snoop(this_object());

    exec(demon, possessed);
    /*
     * If the possessed object is a player, move him back.
     */
    if (is_player)
	exec(possessed, this_object());

    tell_object(possessed, "You feel the alien presence withdraw and " +
	"you regain control of your body.\n");
    tell_object(demon, "You release " +
	capitalize(possessed->query_real_name()) + ".\n");
    possessed->set_possessed("");
    remove_object();

    return 1;
}

/*
 * Function name: possess a player
 * Description:	  Set up the possession of a player complete with 
 *		  possess object and messages to both parties.
 * Arguments:	  ob1 - The demon,
 *		  ob2 - The possessed.
 */
int
possess(object ob1, object ob2)
{
    if (obj_no_change)
	return 0;

    if (previous_object() != find_object("/cmd/wiz_cmd_normal"))
	return 0;

    demon = ob1;
    possessed = ob2;

    /*
     * If the possessed object is a player, move him here.
     */
    if (interactive(possessed))
    {
	is_player = 1;
	exec(this_object(), possessed);
    }
    exec(possessed, demon);
    possessed->set_possessed((string)demon->query_real_name());

    tell_object(this_object(), "You feel your very soul being forced " +
	"together and crammed into a remote\n" +
	"corner of your brain. Something alien and powerful takes command " +
	"of your\n" +
	"body. You can only watch from a distance in helpless terror. " +
	"All you seem\n" +
	"to be able to do is to 'think' unhappy thoughts.\n");
    tell_object(possessed, "You are now possessing " +
	capitalize((string)possessed->query_real_name()) + ".\n");

    /*
     * Give the possessed the ability to 'think'.
     */
    enable_commands();
    add_action("think", "think");

    /*
     * Start a periodic environment checker.
     */
    call_out("check_env", 30);

    return 1;
}

/*
 * Function name: check_env
 * Description:	  Periodically check that the object that is possessed
 *		  hasn't been destructed or the possession object moved.
 *		  If this happends, stop possessing.
 */
void
check_env()
{
    /*
     * Do this once a minute.
     */
    call_out("check_env", 30);

    /*
     * In this case the possessed body has been destroyed which means
     * that the demon has been thrown out as well. Better remove his
     * body as well in that case.
     */
    if (!possessed && demon)
	demon->remove_object();

    /*
     * This object has been moved out of the possessed body. 
     * Stop possessing.
     */
    if (environment(this_object()) != possessed)
	release();
}

/*
 * Function name: think
 * Description:	  Allows the sadly possessed to think, the only way to
 *		  communicated with the demon in charge of his body.
 * Arguments:	  arg - the thought
 */
int
think(string arg)
{
    if (strlen(arg) == 0)
	tell_object(possessed, "Cogito ergo sum.\n");
    else
	tell_object(possessed, "You think: " + arg + "\n");
    return 1;
}


public void
catch_tell(string msg)
{
    write_socket(msg);
}

/*
 * Function name: query_prevent_shadow
 * Description:	  Prevents shadowing. 
 */
query_prevent_shadow()
{
    return 1;
}

/*
 * Function name: query_possessed
 * Description:	  Return the possessed object.
 */
string
query_possessed()
{
    return (string)possessed->query_real_name();
}

/*
 * Function name: query_demon
 * Description:	  Return the possessing object.
 */
string
query_demon()
{
    return (string)demon->query_real_name();
}

string
query_real_name()
{
    return geteuid(this_object());
}

string
modify_command(string cmd)
{
    return cmd;
}

/*
 * Function name:
 * Description:
 * Arguments:
 * Returns:
 */
