/* 

   /std/room/description.c

   Room with description adding and obvious exits.
   ----------------------------------------------------------------------
   Defined functions and variables:

   add_my_desc(string)           

   remove_my_desc()              Removes added description(s). No argument
                                 is needed the callers added description(s)
                                 will be removed.

   set_noshow_obvious(int)       True if obvoius exits should not be told in
                                 in the description of the room.
   ----------------------------------------------------------------------
*/

#include "/sys/language.h"

static  mixed     room_descs;       /* Extra longs added to the rooms own */
static  int       room_no_obvious;  /* True if exits should be shown */
static  int     searched;         /* Times this room has been searched */
static  string  *herbs;           /* WHat herbs grows in this room? */

/*
 * Function name: 	add_my_desc
 * Description:   	Add a description printed after the normal
 *                      longdescription.
 * Arguments:	  	str: Description as a string
 *			cobj: Object responsible for the description
 *			      Default: previous_object()
 */
varargs public void
add_my_desc(string str, object cobj)
{
    if (query_prop(ROOM_I_NO_EXTRA_DESC))
	return;

    if (!cobj)
	cobj = previous_object();
    
    if (!str)
	return;

    if (!room_descs)
	room_descs = ({ cobj, str });
    else
	room_descs = room_descs + ({ cobj, str });
}

/*
 * Function name:       change_my_desc
 * Description:         Change a description printed after the normal
 *                      longdescription. NOTE: if an object has more than
 * 			one extra description only one will change.
 * Arguments:           str: New description as a string
 *                      cobj: Object responsible for the description
 *                            Default: previous_object()
 */
varargs public void
change_my_desc(string str, object cobj)
{
    int i;
    mixed tmp_descs;

    if (query_prop(ROOM_I_NO_EXTRA_DESC))
	return;

    if (!cobj)
        cobj = previous_object();

    if (!str || !cobj)
        return;

    i = member_array(cobj, room_descs);

    if (i < 0)
	add_my_desc(str, cobj);
    else
	room_descs[i + 1] = str;
}

/*
 * Function name: 	remove_my_desc
 * Description:   	Removes an earlier added  description printed after
 *                      the normal longdescription.
 * Arguments:	  	cobj: Object responsible for the description
 *			      Default: previous_object()
 */
varargs public void
remove_my_desc(object cobj)
{
    int i, sf;

    if (query_prop(ROOM_I_NO_EXTRA_DESC))
	return;

    if (!cobj)
	cobj = previous_object();

    sf = objectp(cobj);

    i = member_array(cobj, room_descs);
    while (i >= 0)
    {
	if (sf)
	    room_descs = exclude_array(room_descs, i, i + 1);
	else
	    room_descs = exclude_array(room_descs, i - 1, i);
	i = member_array(cobj, room_descs);
    }
}

/*
 * Function name: 	query_desc
 * Description:   	Gives a list of all added descriptions to this room
 * Returns:       	Array on the form:
 *			    desc1, obj1, desc2, obj2, ..... descN, objN
 */
public mixed
query_desc() { return slice_array(room_descs, 0, sizeof(room_descs)); }

/*
 * Function name:
 * Description:     Set to true if the obvious exits should not be shown.
 */
public void
set_noshow_obvious(int obv) { room_no_obvious = obv; }

/*
 * Function name: query_noshow_obvious
 * Description:	  Query the noshow status
 */
public int
query_noshow_obvious() { return room_no_obvious; }

/*
 * Function name: exits_description
 * Description:   This function will return the obvious exits described
 *		  in a njice way.
 * Returns:	  The obvious exits
 */
public string
exits_description()
{
    string *exits, ex;
    int i;

    exits = query_exit();
    if (!exits || !sizeof(exits))
	return "";

    if (sizeof(exits) < 4)
	return "There is one obvious exit: " + exits[1] + ".\n";

    ex = "There are " + LANG_WNUM(sizeof(exits) / 3) +
	" obvious exits: " + exits[1];
    for (i = 4; i < sizeof(exits); i += 3)
	ex += ", " + exits[i];

    return break_string(ex, 76) + ".\n";
}

/*
 * Function name: long
 * Description:   Describe the room and possibly the exits
 * Arguments:	  str: name of item or 0
 * Returns:       A string holding the long description of the room.
 */
varargs public mixed
long(string str)
{
    int i;
    mixed lg;

    lg = ::long(str);
    if (str)
	return lg;
    if (!lg)
	lg = "";

    while ((i = member_array(0, room_descs)) >= 0)
	room_descs = exclude_array(room_descs, i, i + 1);

    if (room_descs) 
	for (i = 1; i < sizeof(room_descs); i += 2)
	    lg = lg + check_call(room_descs[i]);

    if (room_no_obvious)
	return lg;

    return lg + this_object()->exits_description();
}

/*
 * Function name: calc_pros
 * Description:   Finding formula, positive side effects
 * Arguments:     player - The herbalist
 * Returns:       A number
 */
int
calc_pros(object player)
{
    int p;

    /*
     * Players that are members of certain types of guilds get better chances.
     */
    if (player->query_guild_style_occ() == "cleric") p = p + 15;
    if (player->query_guild_style_lay() == "cleric") p = p + 10;
    if (player->query_guild_style_occ() == "ranger") p = p + 10;
    if (player->query_guild_style_lay() == "ranger") p = p + 5;
    p = p + player->query_skill(SS_HERBALISM);

    /* Penalty will be given if no skill. */
    /* To add a certain element of luck  - wise players might get lucky. */

    p = p + random(player->query_stat(SS_WIS) / 3);

    return p;
}

/*
 * Function name: calc_cons
 * Description:   Finding formula, negative side effects
 * Arguments:     player - The herbalist
 * Returns:       A number ;-)
 */
int
calc_cons(object player)
{
    int p;

    /* If no herbalism skill, players will really have problems. */
    if (!player->query_skill(SS_HERBALISM))
        p = p + 15;

    /* Extra penalty if the player cannot see in the room or is blind */
    if (!CAN_SEE_IN_ROOM(player) || !CAN_SEE(player, this_object()))
        p = p + 45;

    /* Penalty increases for each time the room has been searched.
     * This will of course be sad for those who comes to the room after
     * a non-skilled searcher has tried to search some times, but this
     * could indicate that the first person trampled the herbs underfoot :-)
     */
    p = p + searched * searched * 5;

    /* If we have good luck, we can also have bad luck....
     * ....and stupid players with low intelligence have more bad luck ;-)
     */
    p = p + random((100 - player->query_stat(SS_INT)) / 3);

    return p;
}

/*
 * Function name: no_find
 * Description:   Called when the herbalist didn't find any herbs
 * Returns:       string to write
 */
string
no_find()
{
    return "You search everywhere, but find no herbs.\n";
}

/*
 * Function name: search_for_herb
 * Description:   The herbalist has searched the room, now let's see if and
 *                what he has found.
 * Arguments:	  herbalist - The player searching for herbs
 * Returns:       The message to write
 */
string
search_for_herbs(object herbalist)
{
    object herb;
    string herb_file;

    seteuid(getuid(this_object()));

    if (!herbs || !sizeof(herbs))
        return no_find();

    herb_file = herbs[random(sizeof(herbs))];
    if (!herb_file)
        return no_find();

    if ((calc_pros(herbalist) - calc_cons(herbalist) -
                (herb_file->query_find_diff() * 10 - 50)) <= 0)
    {
        searched += 1;
        return no_find();
    }

    herb = clone_object(herb_file);
    tell_room(environment(herbalist), QCTNAME(herbalist) +
        " finds some herbs.\n", herbalist);
    searched += 2;

    if (herb->move(herbalist)) {
	if (herb->move(this_object()))
	    herb->move(this_object(), 1);
	herb->start_decay();
	return "You find " + LANG_ADDART(herb->short()) + ", but are unable "+
	       "to carry it with you!\n";
    }
    else
	return "You find " + LANG_ADDART(herb->short()) + "!\n";
}

/*
 * Function name: add_herb_file
 * Description:   Add a herb file to our array
 * Arguments:     file - The filename to our herb
 * Returns:       1 if added
 */
int
add_herb_file(string file)
{
    if (!file)
        return 0;

    if (!herbs)
        herbs = ({ file });
    else
        herbs += ({ file });
    return 1;
}

/*
 * Function name: query_herb_files
 * Description:   Query the herb files
 * Returns:       The herb array or 0.
 */
mixed query_herb_files() { return herbs; }

/*
 * Function name: set_searched
 * Description:   Set the searched times number, perhaps there grow some new
 *                herbs at reset?
 * Arguments:     i - The new search number
 */
void set_searched(int i) { searched = i; }

/*
 * Function name: query_searched
 * Description:   Query for the search number
 * Returns:       The search number
 */
int query_searched() { return searched; }

