/*
   weapon.c

   Contains all routines relating to weapons of any kind.

   Defined functions and variables:

   set_hit(int)		Sets the weapon "to hit" value.

   set_pen(int)		Sets the weapon penetration value.

   set_pm(int *)	Sets some modifiers for the penetration on the weapon.
			only useful if weapon is of multiple damage type.

   set_wt(type)		Set the weapon type.
			If not defined, set to default.

   set_dt(type)		Set the damage type.
			If not defined, set to default.

   set_hands(which)	Set the hand(s) used to wield the weapon.
			If not defined, set to default.

   set_wf(obj)		Sets the name of the object that contains the function
			to call for extra defined wield() and unwield()
			functions.

   wield_wep(string)	Wield the weapon defined in string.

   unwield_wep(string)	Unwield the weapon defined in string.

   query_wielded()      Returns the wielder if this object is wielded

*/

/*
   In order to make the weapon recoverable you must add the following code:

   query_recover() { return "path:myargs" + query_wep_recover(); }
   init_recover(arg) { ...my recover code...; init_wep_recover(arg); }
*/

#pragma save_binary

inherit "/std/object";

#include <wa_types.h>
#include <ss_types.h>
#include <formulas.h>
#include <stdproperties.h>
#include <macros.h>
#include <cmdparse.h>
#include <language.h>

static int      wep_hit,	/* Weapon to hit */
                wep_pen,	/* Weapon penetration */
                wep_wt,		/* Weapon type */
                wep_dt,		/* Damage type */
                wep_hands,	/* How many hands the weapon takes */
                wielded,	/* Wielded or not */
		wielded_in_hand,/* Wielded in which hand */
		*m_pen,		/* Modifiers for the wep_pen */
		hits,		/* No of hits the weapon have made */
		dull,		/* How dull the weapon has become */
		corroded,	/* Corrotion on the weapon */
		repair_dull,	/* How much dullness we have repaired */
		repair_corr,	/* How much corrosion we have repaired */
		likely_corr,	/* How likely will this weapon corrode */
		likely_break,	/* How likely will this weapon break? */
		likely_dull,	/* How likely will it be dulled by battle? */
		max_value;	/* Value of weapon at prime condition */
static object   wielder,	/* Who is holding it */
                wield_func;	/* Object that defines extra wield/unwield */
static string   gFail;          /* Acc error messages when wielding */

/*
 * Prototypes
 */
string	wep_condition_desc();
void	update_prop_settings();

/*
 * Function name: create
 * Description:   Create the weapon. (constructor)
 */
public nomask
create_object(arg)
{
    wep_wt = F_WEAPON_DEFAULT_WT;
    wep_dt = F_WEAPON_DEFAULT_DT;
    likely_dull = 10;
    likely_corr = 10;
    likely_break = 10;
    wep_hands = F_WEAPON_DEFAULT_HANDS;
    set_name("weapon");
    wielded = 0;
    add_prop(OBJ_I_VALUE, "@@query_value");
    add_prop(OBJ_I_VOLUME, 500);
    add_prop(OBJ_I_WEIGHT, 200);
    m_pen = ({ 0, 0, 0 });

    create_weapon();

    update_prop_settings();
}

/*
 * Function name: create_weapon
 * Description:   create the weapon. (standard)
 */
public create_weapon() { ::create_object(); }

/*
 * Function name: reset
 * Description:   Reset the weapon. 
 */
public nomask
reset_object() { reset_weapon(); }

/*
 * Function name: reset_weapon
 * Description:   reset the weapon. (standard)
 */
public reset_weapon() { ::reset_object(); }

/*
 * Function name: init
 * Description:   Initialize some weapon related actions.
 */
public void
init()
{
    ::init();
    add_action("wield_wep", "wield");
    add_action("unwield_wep", "unwield");
}

/*
 * Function name: short, pshort, long
 * Description:   We change the short, plural short and long description of
 *                the weapon if it is broken.
 * Returns:       The description.
 */
public varargs string
short(object for_obj)
{
    string str;

    str = "";
    if (query_prop(OBJ_I_BROKEN))
        str = "broken ";

    return str + ::short(for_obj);
}

public varargs string
plural_short(object for_obj)
{
    string str;

    str = "";
    if (::plural_short(for_obj))
    {
        if (query_prop(OBJ_I_BROKEN))
            str = "broken ";
        return str + ::plural_short(for_obj);
    }

    return ::plural_short(for_obj);
}

public varargs string
long(object for_obj)
{
    return ::long(for_obj) + break_string(wep_condition_desc(), 75);
}

/*
 * Function name: wield_wep
 * Description:   wield the weapon.
 * Arguments:	  what: what to wield.
 */
public nomask int
wield_wep(string str)
{
    object 	*a;
    
    notify_fail(capitalize(query_verb()) + " what?\n");  /* access failure */
    
    /*
     * This is an exception from the rule that we should do 'notify_fail'
     * and return 0, all the time. If we do then all weapons will run
     * this code, which is somewhat wastefull.
     */
    gFail = "";
    a = CMDPARSE_ONE_ITEM(str, "wield_one_thing", "wield_access");
    if (!sizeof(a))
    {
	if (!strlen(gFail))
	    return 0;
	else
	    write(break_string(gFail, 70));
    }
    gFail = 0;
    return 1;
}

/*
 * Check that what we try to get is a weapon and that we carry it
 */
public nomask
wield_access(object what)
{
    if (function_exists("create_object", what) != WEAPON_FILE)
	return 0;
    if (environment(what) == this_player())
	return 1;
    else
	return 0;
}
    
nomask int
wield_one_thing(object what)
{
    string fail;

    /*
     * Is it this weapon?
     */
    if (this_object() != what)
	fail = what->wield_me();
    else
	fail = this_object()->wield_me();
    if (stringp(fail))
    {
	gFail += fail;
	return 0;
    }
    return 1;
}

/*
 * Wield this weapon
 * Return error message, write success message. Return 1 if success.
 */
nomask mixed
wield_me()
{
    int wret, skill, pen;
    string penuse, wfail;

    if (wielded)
	return "You already wield the " + short() + ".\n";
    else if (this_player() != environment(this_object()))
	return "You must carry the " + short() + " to be able to wield it.\n";
    else if (query_prop(OBJ_I_BROKEN))
	return "The " + short() + " is broken and not wieldable.\n";

    wielder = this_player();

    /* 
     * Check for a hand to wield the weapon in.
     */
    wielded_in_hand = wep_hands;
    if (wep_hands != W_ANYH)
    {
	/* 
         * Anything in both hands
         */
	if (wielder->query_weapon(W_BOTH) && wep_hands < W_FOOTR)
	    return "Your hands seem busy with another weapon.\n";

	/* 
         * Anything in the specified hand
         */
	if (wielder->query_weapon(wep_hands))
	    return "You already wield " + 
		   wielder->query_weapon(wep_hands)->short() + ".\n";
    
	if (wep_hands == W_BOTH &&
	    (wielder->query_weapon(W_RIGHT) || 
	     wielder->query_weapon(W_LEFT)))
	    return "You need both hands to wield it.\n";
    }
    else if (wielder->query_weapon(W_BOTH) == 0 && 
	     wielder->query_weapon(W_RIGHT) == 0)
	wielded_in_hand = W_RIGHT;

    else if (wielder->query_weapon(W_BOTH) == 0 && 
	     wielder->query_weapon(W_LEFT) == 0)
	wielded_in_hand = W_LEFT;

    else
	return "You have no hand left to wield that weapon.\n";

    if (stringp(wfail = wielder->wield(this_object())))
	return wfail;

    wret = 0;

    /*
     * A wield function in another object.
     */
    if ((!wield_func) || (!(wret=wield_func->wield(this_object()))))
    {
	if (wielded_in_hand == W_BOTH)
	    write("You wield the " + short() + " in both your hands.\n");
	else if (wielded_in_hand < W_FOOTR)
	    write("You wield the " + short() + " in your " +
		  (wielded_in_hand == W_RIGHT ? "right" : "left") + 
		  " hand.\n");
	else
	    write("You wield the " + short() + " on your " +
		  (wielded_in_hand == W_FOOTR ? "right" : "left") + 
		  " foot.\n");

	say(QCTNAME(this_player()) + " wields " + QSHORT(this_object()) + ".\n");
    }
    if (intp(wret) && wret >= 0)
    {
	wielded = 1;
	return 1;
    }

    if (stringp(wfail = wielder->unwield(this_object())))
    {
	/* This is serious and almost fatal, please panic! */
	wielded = 1;
	return 1;
    }


    /*
     * If the wieldfunc returned a value <0 the we can not wield
     * likewise if it returned a string, but then we use that string
     * as error message.
     */
    if (stringp(wret))
	return wret;
    else 
	return "You cant wield " + short() + "\n";

#if 0 /* This should be move to appraise somewhere */

    skill = wielder->query_skill(SS_WEP_FIRST + query_wt() - W_FIRST);
    wret = 100 * skill / (wep_pen ? wep_pen : 1);

    if (wret == 0)
	penuse = "You feel yourself unskilled with the " + short() + ".";
    else if (wret > 99)
	penuse = "You feel you can use the " + short() + " to its full capability.";
    else
    {
	switch(wret)
	{
	case 1..10: 
	    penuse = "inadeqaute";
	    break;
	case 11..50:
	    penuse = "insufficient";
	    break;
	case 51..75:
	    penuse = "fair";
	    break;
	case 76..90:
	    penuse = "rather good";
	    break;
	case 91..100:
	    penuse = "almost excellent";
	    break;
	}
        penuse = "You feel your skill with the " + short() + " is " +
	    penuse + ".\n";
    }
    tell_object(wielder,penuse + "\n");
#endif
}

/*
 * Function name: unwield_wep
 * Description:   Unwield the weapon.
 * Arguments:	  what: What to unwield.
 */
unwield_wep(what)
{
    object 	*a;
    string	prep, loc;
    int il;
    
    notify_fail(capitalize(query_verb()) + " what?\n");  /* access failure */
    if (!what)
	return 0;

    /*
     * This is an exception from the rule that we should do 'notify_fail'
     * and return 0, all the time. If we do then all weapons will run
     * this code, which is somewhat wastefull.
     */
    if (!parse_command(what, all_inventory(this_player()), "%i %p %s",
		       a, prep, loc))
    {
	prep = 0; loc = 0;
	if (!parse_command(what, all_inventory(this_player()), "%i", a))
	    return 0;
    }
    il = a[0];
    a = (a & this_player()->query_weapon(-1));
    if (prep)
	a = filter(a, "find_wep", this_object(), loc);
    a = CMDPARSE_STD->normal_access( ({ il }) + a, 0, 0, 1);

    if (!sizeof(a))
    {
	if (prep)
	    notify_fail("No such weapon wielded " + prep + " " + loc + ".\n");
	else
	    notify_fail("No such weapon wielded.\n");
	return 0;
    }

    for (il = 0; il < sizeof(a); il++)
	a[il]->unwield_me();

    return 1;
}

/*
 * Unwield the weapon
 */
nomask int
unwield_me()
{
    int wret;

    if (!wielded || !wielder)
	return 0;

    wret = 0;

    /*
     * A unwield function in another object.
     */
    if ((!wield_func) || (!(wret = wield_func->unwield(this_object()))))
    {
	if (check_seen(this_player()))
	    write("You stop wielding the " + short() + ".\n");
	else
	    write("You stop wielding something.\n");
	say(QCTNAME(this_player()) + " stops wielding " + 
	    QSHORT(this_object()) + ".\n");
    }
    /*
     * If the wieldfunc returned a value < 0 then we can not unwield
     */
    if (wret >= 0)
    {
	wielder->unwield(this_object());
	wielded = 0;
	wielded_in_hand = wep_hands;
	return 1;
    }
    else
	return 0;
}

static int
find_wep(object wep, string ploc)
{
    int where;
    string a, b;

    ploc = "d " + ploc + " d";
    where = wep->query_attack_id();

    if (sscanf(ploc, "%sleft%s", a, b) &&
	((where == W_RIGHT) ||
	 (where == W_FOOTR)))
	return 0;
    if (sscanf(ploc, "%sright%s", a, b) &&
	((where == W_LEFT) ||
	 (where == W_FOOTL)))
	return 0;
    if (sscanf(ploc, "%shand%s", a, b) &&
	((where == W_FOOTL) ||
	 (where == W_FOOTR)))
	return 0;
    if (sscanf(ploc, "%sfoot%s", a, b) &&
	((where == W_ANYH) ||
	 (where == W_LEFT) ||
	 (where == W_RIGHT) ||
	 (where == W_BOTH)))
	return 0;
    return 1;
}

/*
 * Function name: leave_env
 * Description:   The weapon is moved from the inventory
 * Arguments:	  from - Where from
 */
void
leave_env(object from, object dest)
{
    ::leave_env(from, dest);

    if (!wielded)
	return;

    if ((!wield_func || !wield_func->unwield(this_object())) && wielder)
	tell_object(wielder, "You stop wielding the " + short() + ".\n");

    wielder->unwield(this_object());
    wielded = 0;
}

void
set_name(string name)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    ::set_name(name);
}

/*
 * Function name: set_hit
 * Description:   Set the to hit value in the weapon
 */
void
set_hit(int class)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    wep_hit = class;
}

/*
 * Function name: query_hit
 * Description:   Query the to hit value in the weapon
 */
int query_hit() { return wep_hit; }

/*
 * Function name: set_pen
 * Description:   Set penetration of the weapon
 */
void
set_pen(int class)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    wep_pen = class;
}

/*
 * Function name: query_pen
 * Description:   Query penetration of the weapon
 */
int query_pen() { return wep_pen; }

/*
 * Function name: set_pm
 * Description:   Set the modifiers for damage types.
 * Arguments:	  list, an array of modifiers like ({ impale, slash, bludgeon })
 *		  The sum of the modifiers should be 0 and a modifier for
 *		  a damage type that is not used should also be 0.
 */
void
set_pm(int *list)
{
    if (query_lock())
	return;

    if (F_LEGAL_AM(list))
	m_pen = list;
}

/*
 * Function name: query_pm
 * Description:   Query the modifiers of damage type
 */
int *query_pm() { return m_pen + ({}); }

/*
 * Function name; query_modified_pen
 * Description:   Query for pen modified for different damage types
 */
int *
query_modified_pen()
{
    int *m_pent, i, du, co, pen;

    du = dull - repair_dull;
    co = corroded - repair_corr;
    pen = this_object()->query_pen();

    m_pent = allocate(W_NO_DT);

    for (i = 0; i < W_NO_DT; i++)
    {
	if (!pointerp(m_pen))
	    m_pent[i] = pen;
	else if (i >= sizeof(m_pen))
	    m_pent[i] = pen + (i ? m_pen[0] : 0);
	else
	    m_pent[i] = pen + m_pen[i];
    }

    return ({ m_pent[0] - 2 * (du + co) / 3, m_pent[1] - du - co,
		m_pent[2] - (du + co) / 3 });
}

/*
 * Function name: set_wt
 * Description:   Set the weapon type
 */
void
set_wt(int type)
{
    int *likely;

    if (query_lock())
	return;			/* All changes has been locked out */

    if (type >= 0 && type < W_NO_T)
    {
	wep_wt = type;
	likely = W_DRAWBACKS[type];
	likely_dull = likely[0];
	likely_corr = likely[1];
	likely_break = likely[2];
    }
}

/*
 * Function name: query_wt
 * Description:   Query the weapon type
 */
int query_wt() { return wep_wt; }

/*
 * Function name: set_dt
 * Description:   Set the damage type of the weapon
 */
void
set_dt(int type)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    if (F_LEGAL_DT(type))
	wep_dt = type;
}

/*
 * Function name: query_dt
 * Description:   Query damage type of weapon
 */
int query_dt() { return wep_dt; }

/*
 * Function name: set_hands
 * Description:   Set how the weapon should be wielded
 */
void
set_hands(int which)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    if (F_LEGAL_HANDS(which))
	wep_hands = which;
}

/*
 * Description: The hands that we can use for this weapon
 */
int query_hands() { return wep_hands; }

/*
 * Description: This is the attack that it supports
 */
int query_attack_id() { return wielded_in_hand; }

/*
 * Description: This is the tool slot that the weapon occupies now
 */
int *
query_slots()
{
    int abit, *hids;

    for (hids = ({}), abit = 2; abit <= wielded_in_hand; abit *= 2)
    {
        if (wielded_in_hand & abit)
        {
            hids = hids + ({ abit });
        }
    }
    return hids;
}

/*
 * Description: The slots the weapon protects
 */
int *
query_protects()
{
    switch (wielded_in_hand)
    {
	case W_LEFT:
	    return ({ A_L_ARM });
	case W_RIGHT:
	    return ({ A_R_ARM });
	case W_BOTH:
	    return ({ A_L_ARM, A_R_ARM });
	case W_FOOTR:
	    return ({ A_R_FOOT });
	case W_FOOTL:
	    return ({ A_L_FOOT });
        default:
	    return ({ });
    }
}

/*
 * Sets the object to call wield/unwield in when this occurs.
 * Those functions can return:
 *		0 - No affect the weapon can be wielded / unwielded
 *		1 - It can be wielded / unwielded but no text should be printed
 *		    (it was done in the function)
 *		-1  It can not be wielded / unwielded default failmsg will be 
 *		    written
 *             string  It can not be wielded / unwielded 'string' is the 
 *		       fail message to print
 */
void
set_wf(object obj)
{
    if (query_lock())
	return;			/* All changes has been locked out */

    wield_func = obj;
}

/*
 * Function name: set_corroded
 * Description:   Use this to increases the corroded status on weapons.
 * Arguments:     cond - The new condition we want (can only be raised)
 * Returns:       1 if new condition accepted, 0 if no corrosion
 */
int
set_corroded(int corr)
{
    if (corr > corroded)
    {
        corroded = corr;
        if (F_WEAPON_BREAK(dull - repair_dull, corroded - repair_corr,
			likely_break))
            call_out("remove_broken", 1);
        if (wielded && wielder)
            wielder->update_weapon(this_object());
        return 1;
    }
    return 0;
}

/*
 * Function name: query_corroded
 * Description:   Returns how many times this weapon has become corroded
 * Returns:	  The number of times
 */
int query_corroded() { return corroded; }

/*
 * Function name: set_likely_corr
 * Description:   Set how likely it is this weapon will corrode when in acid 
 *		  or something like that. 0 means it won|t corrode at all.
 * Arguments:	  i - how likely it will corrode, probably corrode if random(i)
 *		      [0, 20] recommended
 */
void set_likely_corr(int i) { likely_corr = i; }

/*
 * Function name: query_likely_corr
 * Description:   Query how likely the weapon will corrode, use it to test
 * 		  if weapon survived your rustmonster or acid pool :)
 */
int query_likely_corr() { return likely_corr; }

/*
 * Function name: set_dull
 * Description:   Use this to increases the dull status on weapons.
 * Arguments:     cond - The new condition we want (can only be raised)
 * Returns:       1 if new condition accepted
 */
int
set_dull(int du)
{
    if (du > dull)
    {
        dull = du;
        if (F_WEAPON_BREAK(dull - repair_dull, corroded - repair_corr,
			likely_break))
            call_out("remove_broken", 1);
        if (wielded && wielder)
            wielder->update_weapon(this_object());
        return 1;
    }
    return 0;
}

/* 
 * Function name: query_dull
 * Description:   Returns how many times this weapon has become duller 
 * Returns:	  The number of times
 */
int query_dull() { return dull; }

/*
 * Function name: set_likely_dull
 * Description:   Set how likely the weapon will get dull or in case of a club
 *		  or mace, wear down in combat Mainly used from did_hit().
 * Arguments:     i - how likely [0, 30] recommended
 */
void set_likely_dull(int i) { likely_dull = i; }

/*
 * Function name: query_likely_dull
 * Description:   How likely it is this weapon will become duller when used
 * Returns:       How likely it is
 */
int query_likely_dull() { return likely_dull; }

/*
 * Function name: set_likely_break
 * Description:   Set how likely the weapon is to break if you use it.
 * Argument:	  i - How likely, [0, 20] recommended
 */
void set_likely_break(int i) { likely_break = i; }

/*
 * Function name: query_likely_break
 * Description:   How likely is it this weapon will break with use
 * Returns:	  How likely it is
 */
int query_likely_break() { return likely_break; }

/*
 * Function name: remove_broken
 * Description:   The armour got broken
 */
remove_broken()
{
    add_prop(OBJ_I_BROKEN, 1);
    if (!wielded || !wielder)
        return;

    tell_object(wielder, "The " + short(wielder) + " breaks!!!\n");
    say("The " + QSHORT(this_object()) + " wielded by " + QTNAME(wielder) +
	" breaks!!!\n", wielder);

    wielder->unwield(this_object());
    wielded = 0;
}

/*
 * Function name: set_repair_dull
 * Description:   When trying to repair the weapon, call this function. Repairs
 *                can only increase the repair factor. (This is sharpening)
 * Arguments:     rep - The new repair number
 * Returns:       1 if new repair status accepted
 */
int
set_repair_dull(int rep)
{
    if (rep > repair_dull && F_LEGAL_WEAPON_REPAIR_DULL(rep, dull))
    {
        repair_dull = rep;
        if (wielded && wielder)
            wielder->update_weapon(this_object());
        return 1;
    }
    return 0;
}

/*
 * Function name: query_repair_dull
 * Description:   How many times has this weapon been sharpened
 * Returns:	  The number of times
 */
int query_repair_dull() { return repair_dull; }

/*
 * Function name: set_repair_corr
 * Description:   When trying to repair the weapon, call this function. Repairs
 *                can only increase the repair factor. This repairs corroded.
 * Arguments:     rep - The new repair number
 * Returns:       1 if new repair status accepted
 */
int
set_repair_corr(int rep)
{
    if (rep > repair_corr && F_LEGAL_WEAPON_REPAIR_CORR(rep, corroded))
    {
        repair_corr = rep;
        if (wielded && wielder)
            wielder->update_weapon(this_object());
        return 1;
    }
    return 0;
}

/*
 * Function name: query_repair_corr
 * Description:	  How many times this weapon has been repaired from corrosion
 * Returns:	  How many times
 */
int query_repair_corr() { return repair_corr; }

/*
 * Function name: set_weapon_hits
 * Description:   By setting the hits counter you will have influence over how
 *                likely the weapon is to get in a worse condition. The hits
 *                variable keeps track of how many times this piece of weapon
 *                has hit something.
 * Argument:      new_hits - integer
 */
public void
set_weapon_hits(int new_hits) { hits = new_hits; }

/*
 * Function name: query_weapon_hits
 * Description:   hits variable keeps track of how many times this weapon has
 *		  hit something.
 * Returns:	  The number of times
 */
public int
query_weapon_hits() { return hits; }

/*
 * Function name: add_prop_obj_i_value
 * Description:   Someone is adding the value prop to this object.
 * Arguments:     val - The new value (mixed)
 * Returns:       1 if not to let the val variable through to the prop
 */
int
add_prop_obj_i_value(mixed val)
{
    if (!max_value)
    {
        max_value = 10;
        return 0;
    }

    if (intp(val) && val)
        max_value = val;

    return 1;
}

/*
 * Function name: query_value
 * Description:   Qhat's the value of this armour
 */
int
query_value()
{
    if (query_prop(OBJ_I_BROKEN))
        return 0;

    return max_value *
	F_WEAPON_VALUE_REDUCE(dull - repair_dull, corroded - repair_corr) / 100;
}

/*
 * Function name: query_wf
 * Description:   Query if/what object defines wield/unwield functions
 */
query_wf() { return wield_func; }

/*
 * Function name: try_hit
 * Description:   Called from living when weapon used.
 * Arguments:     target - Who I intend to hit.
 * Returns:       False if weapon miss. If true it might hit.
 */
int try_hit(object target) { return 1; }

/*
 * Function name: did_hit
 * Description:   Tells us that we hit something. Should produce combat
 *                messages to all relevant parties. If the weapon
 *                chooses not to handle combat messages then a default
 *                message is generated.
 * Arguments:     aid:   The attack id
 *                hdesc: The hitlocation description.
 *                phurt: The %hurt made on the enemy
 *                enemy: The enemy who got hit
 *		  dt:	 The current damagetype
 *		  phit:  The %success that we made with our weapon
 *		  dam:   The actual damage caused by this weapon in hit points
 * Returns:       True if it handled combat messages, returning a 0 will let
 *		  the normal routines take over
 */
public varargs int
did_hit(int aid, string hdesc, int phurt, object enemy, int dt,
		int phit, int dam)
{
    hits++;

    if (F_WEAPON_CONDITION_DULL(hits, wep_pen, likely_dull))
    {
	hits = 0;
	set_dull(query_dull() + 1);
    }

    return 0;
}

/*
 * Function name: check_weapon
 * Description:   Check file for security.
 */
int check_weapon() { return 1; }

/*
 * Function name: set_default_weapon
 * Description:   Configures the weapon
 * Arguments:
 * Returns:
 */
varargs void
set_default_weapon(int hit, int pen, int wt, int dt, int hands, object obj)
{
    /* Sets the weapon "to hit" value.
    */
    if (hit) set_hit(hit);
    else set_hit(5);

    /* Sets the weapon penetration value.
    */
    if (pen) set_pen(pen);
    else set_pen(10);

    /* Set the weapon type.
    */
    if (hit) set_wt(wt);
    else set_wt(W_FIRST);

    /* Set the damage type.
    */
    if (hit) set_dt(dt);
    else set_dt(W_IMPALE | W_SLASH);

    /* Set the hand(s) used to wield the weapon.
    */
    if (hands) set_hands(hands);
    else set_hands(W_NONE);
    
    /* Sets the name of the object that contains the function
       to call for extra defined wield() and unwield()
       functions.
    */
    if (obj) set_wf(obj);
}


/*
 * Function name: query_wield_desc
 * Description:   Describe this weapon as wielded by a something.
 * Argumensts:    p: Possessive description of wielder
 * Returns:       Description string.
 */
public nomask string 
query_wield_desc(string p)
{
    string str;

    if (check_seen(this_player()))
	str = LANG_ADDART(short(this_player()));
    else
	str = "something";

    switch(wep_hands)
    {
    case W_RIGHT:return str + " in " + p + " right hand";
    case W_LEFT: return str + " in " + p + " left hand";
    case W_BOTH: return str + " in both hands";
    case W_FOOTR:return str + " on " + p + " right foot";
    case W_FOOTL:return str + " on " + p + " left foot";
    }
    return str;
}

/*
 * Function name: update_prop_settings
 * Description:   Will uppdate weight and value of this object to be legal
 */
nomask void
update_prop_settings()
{
    if (query_prop(OBJ_I_VALUE) < F_VALUE_WEAPON(wep_hit, wep_pen) &&
	    !query_prop(OBJ_I_IS_MAGIC_WEAPON))
        add_prop(OBJ_I_VALUE, F_VALUE_WEAPON(wep_hit, wep_pen));
 
    if (F_WEIGHT_FAULT_WEAPON(query_prop(OBJ_I_WEIGHT), wep_pen, wep_wt) &&
	    !query_prop(OBJ_I_IS_MAGIC_WEAPON))
        add_prop(OBJ_I_WEIGHT, F_WEIGHT_DEFAULT_WEAPON(wep_pen, wep_wt));
}

/*
 * Function name: query_wielded
 * Description:   If this object is wielded or not
 * Returns:       The object who wields this object if this object is wielded
 */
object
query_wielded()
{
    if (wielded) return wielder;
}

/*
 * Function name: query_am
 * Description:   Called when wielding the weapon, to check for the parry.
 * Returns:       The armour modifier
 */
public nomask int *
query_am() { return ({ -3, 2, 1}); }

/*
 * Function name: query_ac
 * Description:   Called when wielding the weapon, to check for parry.
 * Returns: 	  The ac the weapon contributes with.
 */
public nomask int
query_ac()
{
    if (wielder)
    	return wielder->query_skill(SS_PARRY) * wep_hit / 500 +
		wep_hit / 2;
    else
	return wep_hit / 2;
}

/*
 * Function name: stat_object
 * Description:   This function is called when a wizard wants to get more
 *                information about an object.
 * Returns:       str - The string to write..
 */
string
stat_object()
{
    string str;

    str = ::stat_object();

    str += "Hit: " + wep_hit + "\t\tPen: " + wep_pen + "\n";

    str += "Type: " + wep_wt + "\n";
    str += "Hands: " + wep_hands + "\n";

    return str;
}

/*
 * Function name: wep_condition_desc
 * Description:   Returns the description of the condition of the weapon
 */
string
wep_condition_desc()
{
    string hand, hand2;

    if (query_prop(OBJ_I_BROKEN))
	return "It is broken.\n";

    switch (corroded - repair_corr)
    {
	case 0:
	    hand = ""; break;
	case 1:
	    hand = "You find some rust marks on it!\n"; break;
	case 2:
	    hand = "You can spot some rust on it!\n"; break;
	case 3:
	    hand = "The weapon is full of rust!\n"; break;
	case 4:
	case 5:
	    hand = "It looks like it have been bathing in acid!!\n"; break;
 	default:
	    hand = "It looks like it is falling apart from corrosion.\n"; break;
    }

    switch (dull - repair_dull)
    {
	case 0:
	    hand2 = "in prime condition"; break;
	case 1:
	    hand2 = "in a fine condition"; break;
	case 2:
	    hand2 = "a little touched by battle"; break;
	case 3:
	    hand2 = "scarred by battle"; break;
	case 4:
	case 5:
	    hand2 = "very scarred by battle"; break;
	case 6:
	case 7:
	case 8:
	    hand2 = "in big need of a smith"; break;
	default:
	    hand2 = "going to break any second"; break;
    }

    return hand + "It looks like it is " + hand2 + ".\n";
}

/*
 * Function name: weapon_type
 * Description:   This function shuold return the type of the weapon in
 * 		  text.
 * Returns:	  The type
 */
string
weapon_type()
{
    switch (wep_wt)
    {
	case 0:
	    return "sword";
	case 1:
	    return "polearm";
	case 2:
	    return "axe";
	case 3:
	    return "knife";
	case 4:
	    return "club";
	case 5:
	    return "bow";
	case 6:
	    return "spear";
	default:
	    return "item";
    }
}

/*
 * Function name: appraise_object
 * Description:   Someone tries to appraise the object
 */
void
appraise_object(int num)
{
    string hand;

    ::appraise_object(num);

    switch (wep_hands)
    {
        case W_RIGHT:
            hand = "in the right hand"; break;
        case W_LEFT:
            hand = "in the left hand"; break;
        case W_BOTH:
            hand = "in both hands"; break;
        case W_FOOTR:
            hand = "on the right foot"; break;
        case W_FOOTL:
            hand = "on the left foot"; break;
        case W_ANYH:
            hand = "in any hand"; break;
        default:
            hand = "by some strange creature"; break;
    }

    write("\nYou notice that the " + weapon_type() +
	" is made to be wielded " + hand + ".\n");
    /*
     * Condition is in long description now.
    write(wep_condition_desc());
     */
}

/*
 * Function name: query_wep_recover
 * Description:   Return the recover strings for changing weapon variables.
 * Returns:	  A recover string
 */
string
query_wep_recover()
{
    return "#w_h#" + hits + "#w_d#" + dull + "#w_c#" + corroded +
	"#w_rd#" + repair_dull + "#w_rc#" + repair_corr +
	"#w_b#" + query_prop(OBJ_I_BROKEN) + "#";
}

/*
 * Function name: init_wep_recover
 * Description:   Initialize the weapon variables at recover.
 * Arguments:	  arg - String with variables to recover
 */
void
init_wep_recover(string arg)
{
    string foobar;
    int tmp;

    sscanf(arg, "%s#w_h#%d#%s", foobar, hits, foobar);
    sscanf(arg, "%s#w_d#%d#%s", foobar, dull, foobar);
    sscanf(arg, "%s#w_c#%d#%s", foobar, corroded, foobar);
    sscanf(arg, "%s#w_rd#%d#%s", foobar, repair_dull, foobar);
    sscanf(arg, "%s#w_rc#%d#%s", foobar, repair_corr, foobar);
    sscanf(arg, "%s#w_b#%d#%s", foobar, tmp, foobar);
    if (tmp)
	add_prop(OBJ_I_BROKEN, tmp);
}
