/*
 * /sys/global/adverbs.c
 *
 * This file holds the functions for manipulating adverbs. I stripped the
 * adverbs from /cmd/std/soul_cmd.c and made it accessible for everyone.
 * However, if you want to make a soul that uses adverbs, you'd rather access
 * the adverbs throught /cmd/std/command_driver.c which holds the functions
 * parse_adverb and parse_adverb_with_space. For more information... see that
 * file.
 *
 * /Mercade, 10 November 1993
 *
 * Revision history:
 */

#include <adverbs.h>   /* For the adverb definitions    */
#include <macros.h>    /* For the macro CALL_BY(x)      */
#include <composite.h> /* FOr the macro COMPOSITE_WORDS */

/*
 * Global variable. This is the list of all adverbs known to the mudlib. For
 * the algorithm it is VITAL that new adverbs are added in alphabetical order
 * for the search algorithm uses the fact that the list is ordered
 * alphabetically.
 */
string *adverbs;

/*
 * Function name: create
 * Description  : Called upon initialization
 */
void
create()
{
    seteuid(getuid(this_object()));
    restore_object(ADVERB_SAVE_FILE);

    if (!sizeof(adverbs))
    {
        adverbs = DEFAULT_ADVERB_ARRAY;
        save_object(ADVERB_SAVE_FILE);
    }
}

/*
 * Function name: short
 * Description  : returns the short description of this object
 * Returns      : the short description
 */
string
short()
{
    return "the fabric of space";
}

/*
 * Function name: pattern_is_adverb
 * Description  : Returns whether a pattern matches the adverb at 'position'
 *                in the array of adverbs
 * Arguments    : pattern  - the pattern to match
 *                position - the index of the wanted adverb
 * Returns      : 1 - match
 *                0 - no match
 */
static nomask int
pattern_is_adverb(string pattern, int position)
{
    return (extract(adverbs[position], 0, (strlen(pattern) - 1)) == pattern);
}

/*
 * Function name: member_adverb
 * Description  : This routine uses a binary search to match a pattern against
 *                all adverbs known. The binary search will be a lot faster
 *                than the old adverb check with a for-loop. It is implemented
 *                with a loop rather than with recursion to avoid a too deep
 *                recursion problem if the adverb list extends.
 * Arguments    : pattern - the pattern to check on being an adverb
 * Returns      : int     - >= 0 - the position
 *                          -1   - if the adverb does
 *                          -2   - it is a special service adverb
 */
public nomask int
member_adverb(string pattern)
{
    int low = 0;
    int high = (sizeof(adverbs) - 1);
    int half;

    if (member_array(pattern, SERVICE_ADVERBS_ARRAY) != -1)
    {
        return -2;
    }

    if (strlen(pattern) < 3)
    {
        return -1;
    }

    while ((low + 1) < high)
    {
        half = ((low + high) / 2);

        if (pattern < adverbs[half])
        {
            high = half;
            continue;
        }
        low = half;
    }

    if (pattern_is_adverb(pattern, low))
    {
        return low;
    }
    if (pattern_is_adverb(pattern, high))
    {
        return high;
    }

    return -1;
}

/*
 * Function name: full_adverb
 * Description  : If the first part of an adverb is given, the complete
 *                adverb is returned. If the pattern was not recognized,
 *                an empty string will be returned.
 * Arguments    : pattern - the pattern to check on being an adverb
 * Returns      : string - the complete adverb or "" if nothing was found.
 */
public nomask string
full_adverb(string pattern)
{
    int i = member_adverb(pattern);

    if (i == -2)
    {
        return pattern;
    }

    if (i != -1)
    {
        return adverbs[i];
    }

    return NO_ADVERB;
}

/*
 * Function name: adverb_at_pos
 * Description  : Give the adverb with a certain index in the array
 * Arguments    : i - the index in the array
 * Returns      : string - the i'th adverb if the array is that long
 */
public nomask string
adverb_at_pos(int i)
{
    if ((i >= 0) && (i < sizeof(adverbs)))
    {
        return adverbs[i];
    }

    return NO_ADVERB;
}

/*
 * Function name: get_all_adverbs
 * Description  : Returns all adverbs. I have no idea why someone would want
 *                to get that array, but wants to, he can get is as far as I
 *                am concened.
 */
/*
 * As long as no one gives such a good reason I outcommented it to start with.
 *
public nomask string *
get_all_adverbs()
{
    return adverbs;
}
 */

/*
 * Function name: add_adverb
 * Description  : This function will add an adverb to the list of known
 *                addverbs. A check is made for double adverbs and the list
 *                if adverbs is sorted. Note that you will need to specify
 *                the completele adverb. Only archwizards can call this
 *                function.
 * Arguments    : adverb - the adverb to add
 * Returns      : 1 - the adverb could be added
 *                0 - the adverb was not added (it existed already)
 */
int
add_adverb(string adverb)
{
    if (!CALL_BY("/cmd/wiz_cmd_arch"))
    {
        return 0;
    }

    if (!sizeof(adverbs))
    {
        adverbs = DEFAULT_ADVERB_ARRAY;
    }

    if (member_array(adverb, adverbs) != -1)
    {
        return 0;
    }

    adverbs = adverbs + ({ adverb });
    adverbs = sort_array(adverbs);

    save_object(ADVERB_SAVE_FILE);

    return 1;
}

/*
 * Function name: remove_adverb
 * Description  : This function will remove an adverb from the list of known
 *                adverbs. Note that you will have to specify the complete
 *                adverb. Only an archwizard can call this function.
 * Arguments    : adverb - the adverb to remove
 * Returns      : 1 - the adverb could be removed
 *                0 - the adverb could not be removed (didn't exist)
 */
int
remove_adverb(string adverb)
{
    int i;

    if (!CALL_BY("/cmd/wiz_cmd_arch"))
    {
        return 0;
    }

    /*
     * You may not remove the default adverbs.. The adverb searching
     * algorithm always needs at least two adverbs..
     */
    if (member_array(adverb, DEFAULT_ADVERB_ARRAY) != -1)
    {
        return 0;
    }

    if ((i = member_array(adverb, adverbs)) == -1)
    {
        return 0;
    }

    adverbs = exclude_array(adverbs, i, i);

    save_object(ADVERB_SAVE_FILE);

    return 1;
}

/*
 * Function name: query_adverbs
 * Description  : Prints all adverbs that are possible. Note that this
 *                function does not return anything... It only lists them.
 */
void
query_adverbs()
{
    this_interactive()->catch_msg("All possible adverbs are:\n\n" +
        break_string(COMPOSITE_WORDS(adverbs), 75) + ".\n");
}
